@@ -34,11 +34,9 @@
 
 #include "poll.h"
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/select.h>
-#include <errno.h>
-
 #include <sys/time.h>
+#include <errno.h>
+#include "socketx.h"
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
@@ -88,13 +86,20 @@ int poll(struct pollfd *pfd, int nfd, int timeout)
     if (pfd[i].fd < 0)
       continue;
 
-    if (pfd[i].events & POLLIN)
+    if (pfd[i].events & (POLLIN|POLLRDNORM))
       FD_SET(pfd[i].fd, &rfds);
-    if (pfd[i].events & POLLOUT)
+
+    /* see select(2): "the only exceptional condition detectable
+       is out-of-band data received on a socket", hence we push
+       POLLWRBAND events onto wfds instead of efds. */
+    if (pfd[i].events & (POLLOUT|POLLWRNORM|POLLWRBAND))
       FD_SET(pfd[i].fd, &wfds);
-    if (pfd[i].events & POLLPRI)
+    if (pfd[i].events & (POLLPRI|POLLRDBAND))
       FD_SET(pfd[i].fd, &efds);
-    if (pfd[i].fd >= maxfd && (pfd[i].events & (POLLIN|POLLOUT|POLLPRI)))
+    if (pfd[i].fd >= maxfd &&
+	(pfd[i].events & (POLLIN|POLLOUT|POLLPRI|
+			  POLLRDNORM|POLLRDBAND|
+                          POLLWRNORM|POLLWRBAND)))
       maxfd = pfd[i].fd;
   }
   if (maxfd == -1) {
@@ -116,24 +121,23 @@ int poll(struct pollfd *pfd, int nfd, int timeout)
 	continue;
       }
       if (FD_ISSET(pfd[i].fd, &rfds)) {
-	pfd[i].revents |= POLLIN;
-	ok++;
 	/* support for POLLHUP */
 	if (recv(pfd[i].fd, data, 64, MSG_PEEK) == -1) {
-	  if (errno == ESHUTDOWN || errno == ECONNRESET ||
-	      errno == ECONNABORTED || errno == ENETRESET) {
-	    pfd[i].revents &= ~(POLLIN);
+	  if (is_socket_error (ESHUTDOWN) || is_socket_error (ECONNRESET) ||
+	      is_socket_error (ECONNABORTED) || is_socket_error (ENETRESET))
 	    pfd[i].revents |= POLLHUP;
-	    ok--;
-	  }
+	}
+	else {
+	  pfd[i].revents |= pfd[i].events & (POLLIN|POLLRDNORM);
+	  ok++;
 	}
       }
       if (FD_ISSET(pfd[i].fd, &wfds)) {
-	pfd[i].revents |= POLLOUT;
+	pfd[i].revents |= pfd[i].events & (POLLOUT|POLLWRNORM|POLLWRBAND);
 	ok++;
       }
       if (FD_ISSET(pfd[i].fd, &efds)) {
-	pfd[i].revents |= POLLPRI;
+	pfd[i].revents |= pfd[i].events & (POLLPRI|POLLRDBAND);
 	ok++;
       }
       if (ok)
