@@ -1,9 +1,10 @@
 /******************************** -*- C -*- ****************************
  *
- *	C Callin facility
+ *	C Callin facilities
  *
- *	This module provides the routines necessary to allow C code to
- *	invoke Smalltalk messages on objects.
+ *	This module provides many routines to allow C code to invoke 
+ *	Smalltalk messages on objects, most of which are based on
+ *	low-level facilities exposed by interp.c and dict.c.
  *
  *
  ***********************************************************************/
@@ -36,7 +37,6 @@
 
 #include "gst.h"
 #include "gstpriv.h"
-#include "rbtrees.h"
 
 #include <stdio.h>
 #if STDC_HEADERS
@@ -97,7 +97,7 @@ VMProxy gst_interpreter_proxy = {
 
   _gst_oop_to_c, _gst_oop_to_id, _gst_oop_to_int, _gst_oop_to_float,
   _gst_oop_to_bool, _gst_oop_to_char,
-  _gst_oopto_string, _gst_oop_to_byte_array, _gst_oop_to_cobject,
+  _gst_oop_to_string, _gst_oop_to_byte_array, _gst_oop_to_cobject,
 
 /* Smalltalk process support */
   _gst_async_signal, _gst_sync_wait, _gst_async_signal_and_unregister,
@@ -108,34 +108,33 @@ _gst_msg_send (OOP receiver,
 	       OOP selector,
 	       ...)
 {
-  va_list args;
-  OOP anArg, result;
+  va_list ap, save;
+  OOP *args, anArg;
   int numArgs;
 
-  va_start (args, selector);
-
   if (!_gst_smalltalk_initialized)
     gst_init_smalltalk ();
 
-  /* These objects don't need to be incubated because they are being
-     pushed onto the Smalltalk stack which will make them visible to
-     the GC */
-  _gst_prepare_execution_environment ();
-  PUSH_OOP (receiver);
-  for (numArgs = 0; (anArg = va_arg (args, OOP)) != NULL; numArgs++)
-    PUSH_OOP (anArg);
+  va_start (ap, selector);
+
+#ifdef __va_copy
+  __va_copy (save, ap);
+#else
+  save = ap;
+#endif
+
+  for (numArgs = 0; (anArg = va_arg (ap, OOP)) != NULL; numArgs++);
 
   if (numArgs != _gst_selector_num_args (selector))
-    result = _gst_nil_oop;
+    return (_gst_nil_oop);
   else
     {
-      SEND_MESSAGE (selector, numArgs, false);
-      _gst_interpret ();
-      result = POP_OOP ();
-    }
-  _gst_finish_execution_environment ();
+      args = (OOP *) alloca (sizeof (OOP) * numArgs);
+      for (numArgs = 0; (anArg = va_arg (save, OOP)) != NULL; numArgs++)
+	args[numArgs] = anArg;
 
-  return (result);
+      return _gst_nvmsg_send (receiver, selector, args, numArgs);
+    }
 }
 
 OOP
@@ -143,210 +142,86 @@ _gst_vmsg_send (OOP receiver,
 		OOP selector,
 		OOP * args)
 {
-  OOP anArg, result;
   int numArgs;
 
   if (!_gst_smalltalk_initialized)
     gst_init_smalltalk ();
 
-  /* These objects don't need to be incubated because they are being
-     pushed onto the Smalltalk stack which will make them visible to
-     the GC */
-  _gst_prepare_execution_environment ();
-  PUSH_OOP (receiver);
-  for (numArgs = 0; (anArg = *args++) != NULL; numArgs++)
-    PUSH_OOP (anArg);
+  for (numArgs = 0; args[numArgs]; numArgs++);
 
   if (numArgs != _gst_selector_num_args (selector))
-    result = _gst_nil_oop;
-  else
-    {
-      SEND_MESSAGE (selector, numArgs, false);
-      _gst_interpret ();
-      result = POP_OOP ();
-    }
-  _gst_finish_execution_environment ();
-
-  return (result);
-}
-
-OOP
-_gst_nvmsg_send (OOP receiver,
-		 OOP selector,
-		 OOP * args,
-		 int nargs)
-{
-  OOP result;
-  int numArgs;
-
-  if (!_gst_smalltalk_initialized)
-    gst_init_smalltalk ();
-
-  if (nargs != _gst_selector_num_args (selector))
     return (_gst_nil_oop);
-
-  /* These objects don't need to be incubated because they are being
-     pushed onto the Smalltalk stack which will make them visible to
-     the GC */
-  _gst_finish_execution_environment ();
-  _gst_prepare_execution_environment ();
-  PUSH_OOP (receiver);
-  for (numArgs = 0; nargs--; numArgs++)
-    PUSH_OOP (*args++);
-
-  SEND_MESSAGE (selector, numArgs, false);
-  _gst_interpret ();
-  result = POP_OOP ();
-  _gst_finish_execution_environment ();
-
-  return (result);
+  else
+    return _gst_nvmsg_send (receiver, selector, args, numArgs);
 }
 
 OOP
 _gst_str_msg_send (OOP receiver, 
-		   char *sel,
+		   const char *sel,
 		   ...)
 {
-  va_list args;
-  OOP selector, anArg, result;
+  va_list ap, save;
+  OOP *args, anArg;
   int numArgs;
-  inc_ptr incPtr;
-
-  va_start (args, sel);
+  OOP selector;
 
   if (!_gst_smalltalk_initialized)
     gst_init_smalltalk ();
 
+  va_start (ap, sel);
   selector = _gst_intern_string (sel);
 
-  /* It's possible (there is an existing case) that in SEND_MESSAGE a
-     GC can occur, and selector is not protected pb: but isn't the
-     selector a gst_symbol, and hence in the root set? */
-  incPtr = INC_SAVE_POINTER ();
-  INC_ADD_OOP (selector);
+#ifdef __va_copy
+  __va_copy (save, ap);
+#else
+  save = ap;
+#endif
 
-  _gst_prepare_execution_environment ();
-  PUSH_OOP (receiver);
-  for (numArgs = 0; (anArg = va_arg (args, OOP)) != NULL; numArgs++)
-    PUSH_OOP (anArg);
+  for (numArgs = 0; (anArg = va_arg (ap, OOP)) != NULL; numArgs++);
 
   if (numArgs != _gst_selector_num_args (selector))
-    result = _gst_nil_oop;
+    return (_gst_nil_oop);
   else
     {
-      SEND_MESSAGE (selector, numArgs, false);
-      _gst_interpret ();
-      result = POP_OOP ();
-    }
-  _gst_finish_execution_environment ();
+      args = (OOP *) alloca (sizeof (OOP) * numArgs);
+      for (numArgs = 0; (anArg = va_arg (save, OOP)) != NULL; numArgs++)
+	args[numArgs] = anArg;
 
-  INC_RESTORE_POINTER (incPtr);
-  return (result);
+      return _gst_nvmsg_send (receiver, selector, args, numArgs);
+    }
 }
 
 /* Use like printf */
 void
 _gst_msg_sendf (PTR resultPtr, 
-		char *fmt, 
+		const char *fmt, 
 		...)
 {
-  va_list args;
-  OOP selector, anArg, result;
-  int numArgs;
-  char *fp, *s, selectorBuf[256];
+  va_list ap;
+  OOP selector, *args, result;
+  int i, numArgs;
+  const char *fp;
+  char *s, selectorBuf[256];
   inc_ptr incPtr;
 
-  va_start (args, fmt);
+  va_start (ap, fmt);
 
   if (!_gst_smalltalk_initialized)
     gst_init_smalltalk ();
 
   incPtr = INC_SAVE_POINTER ();
-  _gst_prepare_execution_environment ();
 
-  numArgs = -1;
+  numArgs = 0;
   for (s = selectorBuf, fp = &fmt[2]; *fp; fp++)
     {
       if (*fp == '%')
 	{
 	  fp++;
 	  numArgs++;
-	  switch (*fp)
+	  if (*fp == '%')
 	    {
-	    case 'i':
-	      PUSH_INT (va_arg (args, long));
-	      break;
-
-	    case 'f':
-	      anArg = floatd_new (va_arg (args, double));
-	      PUSH_OOP (anArg);
-	      break;
-
-	    case 'F':
-	      anArg = floatq_new (va_arg (args, long double));
-	      PUSH_OOP (anArg);
-	      break;
-
-	    case 'b':
-	      if (va_arg (args, int))
-		  PUSH_OOP (_gst_true_oop);
-	      else
-		PUSH_OOP (_gst_false_oop);
-	      break;
-
-	    case 'c':
-	      anArg = CHAR_OOP_AT ((char) va_arg (args, int));
-	      PUSH_OOP (anArg);
-	      break;
-
-	    case 'C':
-	      anArg = COBJECT_NEW (va_arg (args, PTR));
-	      PUSH_OOP (anArg);
-	      break;
-
-	    case 's':
-	      anArg = _gst_string_new (va_arg (args, char *));
-	      PUSH_OOP (anArg);
-	      break;
-
-	    case 'S':
-	      anArg = _gst_intern_string (va_arg (args, char *));
-	      PUSH_OOP (anArg);
-	      break;
-
-	    case 'o':
-	      anArg = va_arg (args, OOP);
-	      PUSH_OOP (anArg);
-	      break;
-
-	    case 't':		/* type string, followed by a void * */
-	      {
-		OOP ctype;
-		ctype = _gst_type_name_to_oop (va_arg (args, char *));
-		INC_ADD_OOP (ctype);
-
-		anArg =
-		  _gst_c_object_new_typed (va_arg (args, PTR), ctype);
-		PUSH_OOP (anArg);
-	      }
-	      break;
-
-
-	    case 'T':		/* existing type instance, and a void * 
-				 */
-	      {
-		OOP ctype;
-		ctype = va_arg (args, OOP);
-		anArg =
-		  _gst_c_object_new_typed (va_arg (args, PTR), ctype);
-		PUSH_OOP (anArg);
-	      }
-	      break;
-
-	    case '%':
 	      *s++ = '%';
 	      numArgs--;
-	      break;
 	    }
 	}
       else if (*fp != ' ' && *fp != '\t')
@@ -357,19 +232,91 @@ _gst_msg_sendf (PTR resultPtr,
 
   selector = _gst_intern_string (selectorBuf);
 
-  INC_ADD_OOP (selector);	/* not automatically protected! */
-  /* pb: but isn't the selector a gst_symbol, and hence in the root
-     set? */
+  if (numArgs != 1 + _gst_selector_num_args (selector))
+    return;
 
-  if (numArgs != _gst_selector_num_args (selector))
-    result = _gst_nil_oop;
-  else
+  args = (OOP *) alloca (sizeof (OOP) * numArgs);
+  for (i = -1, s = selectorBuf, fp = &fmt[2]; *fp; fp++)
     {
-      SEND_MESSAGE (selector, numArgs, false);
-      _gst_interpret ();
-      result = POP_OOP ();
+      if (*fp != '%')
+	continue;
+
+      fp++;
+      switch (*fp)
+        {
+        case 'i':
+          args[++i] = FROM_INT (va_arg (ap, long));
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 'f':
+          args[++i] = floatd_new (va_arg (ap, double));
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 'F':
+          args[++i] = floatq_new (va_arg (ap, long double));
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 'b':
+          args[++i] = va_arg (ap, int) ? _gst_true_oop : _gst_false_oop;
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 'c':
+          args[++i] = CHAR_OOP_AT ((char) va_arg (ap, int));
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 'C':
+          args[++i] = COBJECT_NEW (va_arg (ap, PTR));
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 's':
+          args[++i] = _gst_string_new (va_arg (ap, const char *));
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 'S':
+          args[++i] = _gst_intern_string (va_arg (ap, const char *));
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 'o':
+          args[++i] = va_arg (ap, OOP);
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 't':		/* type string, followed by a void * */
+          {
+	    OOP ctype;
+	    ctype = _gst_type_name_to_oop (va_arg (ap, const char *));
+	    INC_ADD_OOP (ctype);
+
+	    args[++i] =
+	      _gst_c_object_new_typed (va_arg (ap, PTR), ctype);
+
+	    INC_ADD_OOP (args[i]);
+          }
+          break;
+
+
+        case 'T':		/* existing type instance, and a void * */
+          {
+    	    OOP ctype;
+    	    ctype = va_arg (ap, OOP);
+    	    args[++i] =
+    	      _gst_c_object_new_typed (va_arg (ap, PTR), ctype);
+
+	    INC_ADD_OOP (args[i]);
+          }
+          break;
+	}
     }
-  _gst_finish_execution_environment ();
+
+  result = _gst_nvmsg_send (args[0], selector, args + 1, numArgs - 1);
 
   if (resultPtr)
     {
@@ -424,7 +371,7 @@ _gst_msg_sendf (PTR resultPtr,
 }
 
 OOP
-_gst_type_name_to_oop (char *name)
+_gst_type_name_to_oop (const char *name)
 {
   OOP result;
   char buf[300];
@@ -437,7 +384,7 @@ _gst_type_name_to_oop (char *name)
 
 
 void
-_gst_eval_code (char *str)
+_gst_eval_code (const char *str)
 {
   if (!_gst_smalltalk_initialized)
     gst_init_smalltalk ();
@@ -445,12 +392,12 @@ _gst_eval_code (char *str)
   _gst_compile_code = true;
   _gst_push_cstring (str);
   _gst_parse_stream ();
-  _gst_pop_stream (false);
+  _gst_pop_stream (true);
 }
 
 
 OOP
-_gst_eval_expr (char *str)
+_gst_eval_expr (const char *str)
 {
   OOP result;
 
@@ -460,7 +407,7 @@ _gst_eval_expr (char *str)
   _gst_compile_code = false;
   _gst_push_cstring (str);
   _gst_parse_stream ();
-  _gst_pop_stream (false);
+  _gst_pop_stream (true);
   result = _gst_last_returned_value;
 
   return (result);
@@ -495,7 +442,7 @@ _gst_basic_size (OOP oop)
  ***********************************************************************/
 
 OOP
-_gst_class_name_to_oop (char *name)
+_gst_class_name_to_oop (const char *name)
 {
   OOP result, key;
 
@@ -567,7 +514,7 @@ _gst_string_to_oop (const char *str)
 }
 
 OOP
-_gst_byte_array_to_oop (char *str,
+_gst_byte_array_to_oop (const char *str,
 			int n)
 {
   if (!_gst_smalltalk_initialized)
@@ -715,7 +662,7 @@ _gst_oop_to_char (OOP oop)
 }
 
 char *
-_gst_oopto_string (OOP oop)
+_gst_oop_to_string (OOP oop)
 {
   if (!_gst_smalltalk_initialized)
     gst_init_smalltalk ();
@@ -875,6 +822,34 @@ _gst_unregister_oop_array (OOP **first)
 }
 
 
+void
+_gst_copy_registered_oops (void)
+{
+  rb_node_t *node;
+  rb_traverse_t t;
+
+  /* Walk the OOP registry... */
+  for (node = rb_first(&(oop_registry_root->rb), &t); 
+       node; node = rb_next(&t))
+    {
+      oop_registry *k = (oop_registry *) node;
+      MAYBE_COPY_OOP (k->oop);
+    }
+
+  /* ...and then the OOP-array registry. */
+  for (node = rb_first(&(oop_array_registry_root->rb), &t); 
+       node; node = rb_next(&t))
+    {
+      oop_array_registry *k = (oop_array_registry *) node;
+
+      /* Dereference the pointers in the tree to obtain where the array
+	 lies. */
+      OOP *first = *(k->first);
+      OOP *last = *(k->last);
+      COPY_OOP_RANGE (first, last);
+    }
+}
+
 void
 _gst_mark_registered_oops (void)
 {
