@@ -32,7 +32,6 @@
 
 #include "gst.h"
 #include "gstpriv.h"
-#include "avltrees.h"
 
 #include <ltdl.h>
 
@@ -46,6 +45,10 @@
 #include <errno.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #ifdef HAVE_DIRENT_H
 # include <dirent.h>
 # define NAMLEN(dirent) strlen((dirent)->d_name)
@@ -63,16 +66,8 @@
 # endif
 #endif
 
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
-#ifdef HAVE_IO_H
-# include <io.h>
-#endif
-
-#define ARG_VEC_SIZE		20	/* 20 shorts, 10 longs or ptrs, 
-					   5 dbls */
 
+#define ARG_VEC_SIZE		20	/* 20 shorts, 10 longs or ptrs, 5 dbls */
 
 typedef enum
 {
@@ -86,11 +81,10 @@ cparam_type;
 
 typedef enum
 {				/* types for C parameters */
-  CDATA_UNKNOWN,			/* when there is no type a priori */
+  CDATA_UNKNOWN,		/* when there is no type a priori */
   CDATA_CHAR,
   CDATA_STRING,
-  CDATA_STRINGOUT,		/* for things that modify string params 
-				 */
+  CDATA_STRINGOUT,		/* for things that modify string params */
   CDATA_SYMBOL,
   CDATA_BYTEARRAY,
   CDATA_BYTEARRAYOUT,
@@ -102,17 +96,17 @@ typedef enum
   CDATA_FLOAT,
   CDATA_DOUBLE,
   CDATA_VOID,			/* valid only as a return type */
-  CDATA_VARIADIC,			/* for parameters, this param is an
+  CDATA_VARIADIC,		/* for parameters, this param is an
 				   array to be interpreted as
 				   arguments.  Note that only simple
 				   conversions are performed in this
 				   case. */
-  CDATA_VARIADIC_OOP,	/* for parameters, this param is an
+  CDATA_VARIADIC_OOP,		/* for parameters, this param is an
 				   array whose elements are OOPs to be
 				   passed. */
-  CDATA_COBJECT,			/* a C object is being passed */
+  CDATA_COBJECT,		/* a C object is being passed */
   CDATA_COBJECT_PTR,		/* a C object pointer is being passed */
-  CDATA_OOP,		/* no conversion to-from C (OOP) */
+  CDATA_OOP,			/* no conversion to-from C (OOP) */
   CDATA_SELF,			/* pass self as the corresponding
 				   argument */
   CDATA_SELF_OOP		/* pass self as an OOP */
@@ -126,12 +120,10 @@ typedef struct gst_cfunc_descriptor
 				   addr */
   OOP cFunctionName;		/* Name of C function in mapping table */
   OOP returnType;		/* Smalltalk return type */
-  OOP numFixedArgs;		/* number of real arguments passed from 
-				 */
-  /* smalltalk (excluding "self" parameters */
-  /* which are synthetically added by the */
-  /* callout mechanism when calling the C */
-  /* function). */
+  OOP numFixedArgs;		/* number of real arguments passed from
+				   smalltalk (excluding "self" parameters
+  				   which are synthetically added when
+				   calling the C function). */
   OOP argTypes[1];		/* variable length, really numFixedArgs 
 				   long */
 }
@@ -168,7 +160,7 @@ cparam_union;
 typedef struct cfunc_info
 {
   avl_node_t avl;
-  char *funcName;
+  const char *funcName;
   p_void_func funcAddr;
 }
 cfunc_info;
@@ -194,7 +186,7 @@ static void push_obj (cparam_union * up,
 
 /* Calls the function represented by DESC (of which the return type
    and pointer to the function are looked up). */
-static void call_cfunction (gst_cfunc_descriptor desc);
+static OOP call_cfunction (gst_cfunc_descriptor desc);
 
 /* Prints an error message... this should really make the primitive
    fail so that a WrongClass exception is generated (FIXME) */
@@ -226,13 +218,6 @@ static void test_cstring_array (int numElements,
 static cdata_type get_ctype (gst_cfunc_descriptor desc,
 			     int index);
 
-/* Retrieves the C function descriptor (a C structure, not an OOP),
-   from a method METHODOOP.  This function is part of the policy that
-   the 0th method literal in a method which is a C callout method is
-   the holder of a gst_cobject instance for the C function
-   descriptor. */
-static gst_cfunc_descriptor get_cfunc_descriptor (OOP methodOOP);
-
 /* Converts a Symbol to a SmallInteger so that it is stored in the
    CFunctionDescriptor.  For the return type isReturn is passed as
    true and a CType is alwso considered valid. */
@@ -243,11 +228,11 @@ static OOP classify_type_symbol (OOP symbolOOP,
 static int get_errno (void);
 
 /* Encapsulate binary incompatibilities between various C libraries. */
-static int my_stat (char *name,
+static int my_stat (const char *name,
 		    gst_stat * out);
-static int my_putenv (char *str);
-static int my_chdir (char *str);
-static DIR *my_opendir (char *str);
+static int my_putenv (const char *str);
+static int my_chdir (const char *str);
+static DIR *my_opendir (const char *str);
 static char *extract_dirent_name (struct dirent *dir);
 
 /* Provide access to the arguments passed via -a. */
@@ -371,7 +356,7 @@ get_errno (void)
 }
 
 int
-my_stat (char *name,
+my_stat (const char *name,
 	 gst_stat * out)
 {
   int result;
@@ -394,7 +379,7 @@ my_stat (char *name,
 }
 
 int
-my_putenv (char *str)
+my_putenv (const char *str)
 {
   char *clone;
   int len;
@@ -407,7 +392,7 @@ my_putenv (char *str)
 
 
 int
-my_chdir (char *dir)
+my_chdir (const char *dir)
 {
   int status;
 
@@ -419,7 +404,7 @@ my_chdir (char *dir)
 }
 
 DIR *
-my_opendir (char *dir)
+my_opendir (const char *dir)
 {
   DIR *result;
 
@@ -476,14 +461,13 @@ get_argc (void)
 char *
 get_argv (int n)
 {
-  return (n <=
-	  _gst_smalltalk_passed_argc ? _gst_smalltalk_passed_argv[n -
-								  1] :
-	  NULL);
+  return (n <= _gst_smalltalk_passed_argc
+	  ? _gst_smalltalk_passed_argv[n - 1]
+	  : NULL);
 }
 
 PTR
-_gst_dld_open (char *filename)
+_gst_dld_open (const char *filename)
 {
 #ifdef HAVE_DLD
   lt_dlhandle handle;
@@ -574,7 +558,7 @@ _gst_init_cfuncs (void)
 
 
 void
-_gst_define_cfunc (char *funcName,
+_gst_define_cfunc (const char *funcName,
 		   PTR funcAddr)
 {
   avl_node_t **p = (avl_node_t **) &c_func_root;
@@ -598,7 +582,7 @@ _gst_define_cfunc (char *funcName,
 	}
     }
 
-  node = (cfunc_info *) malloc(sizeof(cfunc_info));
+  node = (cfunc_info *) xmalloc(sizeof(cfunc_info));
   node->avl.avl_parent = (avl_node_t *) cfi;
   node->avl.avl_left = node->avl.avl_right = NULL;
   node->funcName = strdup (funcName);
@@ -611,7 +595,7 @@ _gst_define_cfunc (char *funcName,
 
 
 p_void_func
-_gst_lookup_function (char *funcName)
+_gst_lookup_function (const char *funcName)
 {
   cfunc_info *cfi = c_func_root;
 
@@ -630,14 +614,14 @@ _gst_lookup_function (char *funcName)
 }
 
 
-mst_Boolean
-_gst_invoke_croutine (long int numArgs,
-		      OOP methodOOP)
+OOP
+_gst_invoke_croutine (OOP cFuncOOP,
+		      OOP receiver,
+		      OOP *args)
 {
   gst_cfunc_descriptor desc;
   cdata_type cType;
   OOP oop;
-  OOP selfObject;
   int i, si, fixedArgs;
   string_info stringInfo[ARG_VEC_SIZE], *localSip;
   p_void_func cFunction;
@@ -645,17 +629,15 @@ _gst_invoke_croutine (long int numArgs,
 
   incPtr = INC_SAVE_POINTER ();
 
-  desc = get_cfunc_descriptor (methodOOP);
+  desc = (gst_cfunc_descriptor) OOP_TO_OBJ (cFuncOOP);
   cFunction = (p_void_func) COBJECT_VALUE (desc->cFunction);
   if (!cFunction)
-    return (false);
+    return (NULL);
 
   c_arg = c_arg_vec;
   string_info_base = stringInfo;
   sip = stringInfo;
 
-  selfObject = STACK_AT (numArgs);	/* 0 == stop of stack, stack at 
-					   n is n away */
   /* from the top, the receiver is pushed */
   /* first before the other arguments */
   fixedArgs = TO_INT (desc->numFixedArgs);
@@ -664,23 +646,15 @@ _gst_invoke_croutine (long int numArgs,
     {
       cType = get_ctype (desc, i);
       if (cType >= CDATA_SELF)
-	{
-	  push_smalltalk_obj (selfObject,
-			      cType ==
-			      CDATA_SELF ? CDATA_UNKNOWN : CDATA_OOP);
-	}
+	push_smalltalk_obj (receiver,
+			    cType == CDATA_SELF ? CDATA_UNKNOWN : CDATA_OOP);
       else if (cType != CDATA_VOID)
-	{
-	  /* Do nothing if it is a void */
-	  oop = STACK_AT (numArgs - si - 1);
-	  push_smalltalk_obj (oop, cType);
-	  si++;
-	}
+	/* Do nothing if it is a void */
+	push_smalltalk_obj (args[si++], cType);
     }
 
-  POP_N_OOPS (numArgs);
   localSip = sip;
-  call_cfunction (desc);
+  oop = call_cfunction (desc);
 
   /* Fixup all returned string variables */
   for (; localSip-- != stringInfo;)
@@ -695,19 +669,7 @@ _gst_invoke_croutine (long int numArgs,
     }
 
   INC_RESTORE_POINTER (incPtr);
-  return (true);
-}
-
-gst_cfunc_descriptor
-get_cfunc_descriptor (OOP methodOOP)
-{
-  OOP associationOOP, descOOP, literalsOOP;
-
-  literalsOOP =
-    ((gst_compiled_method) OOP_TO_OBJ (methodOOP))->literals;
-  associationOOP = OOP_TO_OBJ (literalsOOP)->data[0];
-  descOOP = ASSOCIATION_VALUE (associationOOP);
-  return ((gst_cfunc_descriptor) OOP_TO_OBJ (descOOP));
+  return (oop);
 }
 
 cdata_type
@@ -975,27 +937,28 @@ push_obj (cparam_union * up,
     }
 }
 
-void
+OOP
 call_cfunction (gst_cfunc_descriptor desc)
 {
   int intResult;
   unsigned int uIntResult;
+  PTR ptrResult;
   long longResult;
   unsigned long uLongResult;
   double doubleResult;
   p_void_func cFunction;
   cdata_type returnType;
-  OOP returnTypeOOP;
+  OOP returnTypeOOP, resultOOP;
 
   errno = 0;
   cFunction = (p_void_func) COBJECT_VALUE (desc->cFunction);
 
-#define DO_THE_CALL(type)							\
-	(* (type (*)()) cFunction)(					\
-	      c_arg_vec[0],  c_arg_vec[1],  c_arg_vec[2],  c_arg_vec[3],	\
-	      c_arg_vec[4],  c_arg_vec[5],  c_arg_vec[6],  c_arg_vec[7],	\
-	      c_arg_vec[8],  c_arg_vec[9],  c_arg_vec[10], c_arg_vec[11],	\
-	      c_arg_vec[12], c_arg_vec[13], c_arg_vec[14], c_arg_vec[15],	\
+#define DO_THE_CALL(type)						  \
+	(* (type (*)()) cFunction)(					  \
+	      c_arg_vec[0],  c_arg_vec[1],  c_arg_vec[2],  c_arg_vec[3],  \
+	      c_arg_vec[4],  c_arg_vec[5],  c_arg_vec[6],  c_arg_vec[7],  \
+	      c_arg_vec[8],  c_arg_vec[9],  c_arg_vec[10], c_arg_vec[11], \
+	      c_arg_vec[12], c_arg_vec[13], c_arg_vec[14], c_arg_vec[15], \
 	      c_arg_vec[16], c_arg_vec[17], c_arg_vec[18], c_arg_vec[19]);
 
   if (IS_INT (desc->returnType))
@@ -1013,36 +976,37 @@ call_cfunction (gst_cfunc_descriptor desc)
     {
     case CDATA_VOID:
       DO_THE_CALL (void);
+      resultOOP = _gst_nil_oop;
       break;
 
     case CDATA_CHAR:
       intResult = DO_THE_CALL (int);
-      SET_STACKTOP (CHAR_OOP_AT ((gst_uchar) intResult));
+      resultOOP = CHAR_OOP_AT ((gst_uchar) intResult);
       break;
 
     case CDATA_BOOLEAN:
       intResult = DO_THE_CALL (int);
-      SET_STACKTOP_BOOLEAN (intResult);
+      resultOOP = intResult ? _gst_true_oop : _gst_false_oop;
       break;
 
     case CDATA_INT:
       intResult = DO_THE_CALL (int);
-      SET_STACKTOP_INT ((long) intResult);
+      resultOOP = FROM_INT ((long) intResult);
       break;
 
     case CDATA_UINT:
       uIntResult = DO_THE_CALL (unsigned int);
-      SET_STACKTOP_INT ((long) uIntResult);
+      resultOOP = FROM_INT ((long) uIntResult);
       break;
 
     case CDATA_LONG:
       longResult = DO_THE_CALL (long);
-      SET_STACKTOP (FROM_C_LONG (longResult));
+      resultOOP = FROM_C_LONG (longResult);
       break;
 
     case CDATA_ULONG:
       uLongResult = DO_THE_CALL (unsigned long);
-      SET_STACKTOP (FROM_C_ULONG (uLongResult));
+      resultOOP = FROM_C_ULONG (uLongResult);
       break;
 
     case CDATA_STRING:
@@ -1050,50 +1014,51 @@ call_cfunction (gst_cfunc_descriptor desc)
     case CDATA_SYMBOL:
     case CDATA_COBJECT:
     case CDATA_OOP:
-      longResult = DO_THE_CALL (long);
-      if (longResult == 0)
+      ptrResult = DO_THE_CALL (PTR);
+      if (!ptrResult)
 	{
-	  SET_STACKTOP (_gst_nil_oop);
+	  resultOOP = _gst_nil_oop;
 	  break;
 	}
       if (returnType == CDATA_SYMBOL)
 	{
-	  SET_STACKTOP (_gst_intern_string ((char *) longResult));
+	  resultOOP = _gst_intern_string ((char *) ptrResult);
 	  break;
 	}
       if (returnType == CDATA_COBJECT)
 	{
 	  if (returnTypeOOP)
-	    SET_STACKTOP (_gst_c_object_new_typed
-			  ((PTR) longResult, returnTypeOOP));
+	    resultOOP = _gst_c_object_new_typed (ptrResult, returnTypeOOP);
 	  else
-	    SET_STACKTOP (COBJECT_NEW ((PTR) longResult));
+	    resultOOP = COBJECT_NEW (ptrResult);
 	  break;
 	}
       if (returnType == CDATA_OOP)
 	{
-	  SET_STACKTOP ((OOP) longResult);
+	  resultOOP = (OOP) ptrResult;
 	  break;
 	}
-      SET_STACKTOP (_gst_string_new ((char *) longResult));
+      resultOOP = _gst_string_new ((char *) ptrResult);
       if (returnType == CDATA_STRINGOUT)
-	xfree ((char *) longResult);
+	xfree (ptrResult);
       break;
 
     case CDATA_DOUBLE:
       doubleResult = DO_THE_CALL (double);
-      SET_STACKTOP (floatd_new (doubleResult));
+      resultOOP = floatd_new (doubleResult);
       break;
 
     case CDATA_FLOAT:
       doubleResult = (double) DO_THE_CALL (float);
-      SET_STACKTOP (floate_new (doubleResult));
+      resultOOP = floate_new (doubleResult);
       break;
 
     default:
       _gst_errorf
 	("Invalid C function return type specified, index %d\n",
 	 returnType);
+
+      resultOOP = _gst_nil_oop;
       break;
     }
 
@@ -1106,6 +1071,8 @@ call_cfunction (gst_cfunc_descriptor desc)
 #else
   saved_errno = errno;
 #endif
+
+  return resultOOP;
 }
 
 void
