@@ -38,22 +38,22 @@
 /* Element type for the name-to-C-function mapping table. */
 typedef void (*p_void_func) ();
 
-/* Invokes a C routine.  NUMARGS Smalltalk arguments have been popped
-   off the Smalltalk stack when this routine returns, if it has
-   succeeded.  METHODOOP is the C function wrapper method which
-   contains the CObject which contains the C function descriptor used
+/* Invokes a C routine.  Arguments passed from Smalltalk are stored starting
+   from ARGS, and the object to which the message that called-out was
+   sent is RECEIVER.  CFUNCOOP is the C function descriptor used
    to control the mapping of argument types from Smalltalk to C types
    and determines the mapping of the C function's return type into a
-   Smalltalk type.  The result is true if the call was successful,
-   false otherwise.  */
-extern mst_Boolean _gst_invoke_croutine (long int numArgs,
-					 OOP methodOOP);
+   Smalltalk type.  The result is NULL if the call was not successful,
+   an OOP holding the result otherwise.  */
+extern OOP _gst_invoke_croutine (OOP cFuncOOP,
+				 OOP receiver,
+				 OOP *args);
 
 /* Defines the mapping between a string function name FUNCNAME and the
    address FUNCADDR of that function, for later use in
    _gst_lookup_function.  The mapping table will expand as needed to
    hold new entries as they are added. */
-extern void _gst_define_cfunc (char *funcName, PTR funcAddr);
+extern void _gst_define_cfunc (const char *funcName, PTR funcAddr);
 
 /* Adds to the mapping table the standard C functions supported by
    GNU Smalltalk.  */
@@ -62,7 +62,7 @@ extern void _gst_init_cfuncs (void);
 /* Returns the address of the latest C function which has been
    registered using _gst_define_cfunc with the name FUNCNAME.  Returns
    NULL if there is no such function. */
-extern p_void_func _gst_lookup_function (char *funcName);
+extern p_void_func _gst_lookup_function (const char *funcName);
 
 /* This routine is called during image loading to restore a C function
    descriptor pointer.  This is because between the time that the image
