@@ -35,12 +35,9 @@
 #include "obstack.h"
 
 #include <setjmp.h>
+#include <assert.h>
 #include <stdio.h>		/* for printf */
 
-#ifdef HAVE_GETRUSAGE
-# include <sys/resource.h>
-#endif
-
 #if defined(HAVE_SYS_TIME_H) || defined(TIME_WITH_SYS_TIME)
 # include <sys/time.h>
 #endif
@@ -48,6 +45,10 @@
 # include <time.h>
 #endif
 
+#ifdef HAVE_SYS_RESOURCE_H
+# include <sys/resource.h>
+#endif
+
 #if STDC_HEADERS
 #include <string.h>
 #include <stdlib.h>
@@ -244,10 +245,13 @@ static OOP file_segment_new (void);
 /* This create a CompiledBlock for the given BYTECODES.  The bytecodes
    are passed through the peephole optimizer and stored, the header is
    filled according to the given number of arguments ARGS and
-   temporaries TEMPS, and to the cleanness of the block.  */
+   temporaries TEMPS, and to the cleanness of the block.  STACK_DEPTH
+   contains the number of stack slots needed by the block except for
+   arguments and temporaries.  */
 static OOP make_block_closure (int args,
 			       int temps,
-			       bytecodes bytecodes);
+			       bytecodes bytecodes,
+			       int stack_depth);
 
 /* Compiles a block tree node, EXPR, in a separate context and return
    the resulting bytecodes.  The block's argument declarations are
@@ -270,8 +274,7 @@ static bytecodes compile_sub_expression_and_jump (tree_node expr,
    object, added to the constants of the method, and the new index is
    returned.
 
-   For -1, 0, 1, 2, returns a negative number; in fact, the value
-   returned is such that 3+returnValue is the numerical value. */
+   For small integers (between -128 and 255) return -1.  */
 static int add_constant (tree_node constExpr);
 
 /* Like add_constant, this routine adds "selector" to the set of
@@ -323,7 +326,7 @@ static void compile_variable (tree_node varName);
 static void compile_constant (tree_node constExpr);
 
 /* Compiles an instruction to push an Integer constant INTCONST on the
-   stack.  Special cases out the literals -1..2, and tries to emit the
+   stack.  Special cases out the literals -128..255, and tries to emit the
    shortest possible byte sequence to get the job done. */
 static void compile_push_int_constant (int CONST_INT);
 
@@ -438,6 +441,9 @@ static void install_method (OOP methodOOP);
    _gst_prepare_execution_environment. */
 static OOP termination_method;
 
+/* Used to number the lines starting from 1.  */
+static int line_offset;
+
 /* Used to abort really losing compiles, jumps back to the top level
    of the compiler */
 static jmp_buf bad_method;
@@ -450,10 +456,6 @@ static OOP *literal_vec = NULL;
    method being compiled, and the first slot past the literal vector */
 static OOP *literal_vec_curr, *literal_vec_max;
 
-/* These indicate the current stack depth in the method/block being
-   compiled, and the maximum reached stack_depth */
-static int stack_depth, max_stack_depth;
-
 /* This indicates whether we are compiling a block */
 static int inside_block;
 
@@ -470,22 +472,6 @@ static int inside_block;
 static mst_Boolean dup_message_receiver = false;
 
 
-/* Add 1 to the current stack depth and adjust the maximum depth
-   accordingly. */
-#define INCR_STACK_DEPTH()	do { 					\
-	if (++stack_depth > max_stack_depth) max_stack_depth++;		\
-} while (0)
-
-/* Add N to the current stack depth and adjust the maximum depth
-   accordingly. */
-#define ADD_STACK_DEPTH(n)	do {					\
-	stack_depth += (n);						\
-	if (stack_depth > max_stack_depth) max_stack_depth = stack_depth;	\
-} while (0)
-
-/* Subtract N from the current stack depth. */
-#define SUB_STACK_DEPTH(n)	stack_depth -= (n)
-
 /* Exit a really losing compilation */
 #define EXIT_COMPILATION()						\
 	longjmp(bad_method, 1)
@@ -528,10 +514,11 @@ methodsFor: aCategoryString \
   _gst_set_compilation_category (_gst_string_new ("compiling methods"));
   _gst_push_smalltalk_string (_gst_string_new (methodsForString));
   _gst_parse_stream ();
-  _gst_pop_stream (false);	/* can't close a string! */
+  _gst_pop_stream (true);
 
   _gst_set_compilation_class (_gst_undefined_object_class);
   _gst_set_compilation_category (_gst_string_new ("private"));
+  _gst_alloc_bytecodes ();
   _gst_compile_byte (EXIT_INTERPRETER);
   _gst_compile_byte (RETURN_CONTEXT_STACK_TOP);
 
@@ -617,7 +604,7 @@ _gst_display_compilation_trace (char *string,
 }
 
 
-void
+OOP
 _gst_execute_statements (tree_node temporaries,
 			 tree_node statements,
 			 mst_Boolean quiet)
@@ -628,12 +615,26 @@ _gst_execute_statements (tree_node temporaries,
   struct rusage startRusage, endRusage;
 #endif
   OOP methodOOP;
+  OOP oldClass, oldCategory;
   inc_ptr incPtr;
 
+  quiet = quiet || _gst_quiet_execution || _gst_emacs_process
+	  || !_gst_get_cur_stream_prompt ();
+
+  quiet = quiet && !_gst_regression_testing;
+
+  oldClass = _gst_this_class;
+  oldCategory = _gst_this_category;
+  _gst_register_oop (oldClass);
+  _gst_register_oop (oldCategory);
+
   _gst_set_compilation_class (_gst_undefined_object_class);
   _gst_set_compilation_category (_gst_nil_oop);
 
-  messagePattern = _gst_make_unary_expr (NULL, "executeStatements");
+  messagePattern = _gst_make_unary_expr (&statements->location,
+					 NULL,
+					 "executeStatements");
+
   _gst_display_compilation_trace ("Compiling doit code", false);
 
   /* This is a big hack to let doits access the variables and classes
@@ -642,19 +643,25 @@ _gst_execute_statements (tree_node temporaries,
 			 _gst_current_namespace);
   methodOOP =
     _gst_compile_method (_gst_make_method
-			 (messagePattern, temporaries, 0, statements),
-			 false);
+			 (&statements->location, messagePattern,
+			  temporaries, 0, statements),
+			 true, false);
 
   SET_CLASS_ENVIRONMENT (_gst_undefined_object_class,
 			 _gst_smalltalk_dictionary);
 
+  _gst_set_compilation_class (oldClass);
+  _gst_set_compilation_category (oldCategory);
+  _gst_unregister_oop (oldClass);
+  _gst_unregister_oop (oldCategory);
+
   if (_gst_had_error)		/* don't execute on error */
-    return;
+    return (_gst_nil_oop);
 
   incPtr = INC_SAVE_POINTER ();
   INC_ADD_OOP (methodOOP);
 
-  if (!quiet)
+  if (!quiet && (_gst_regression_testing || _gst_verbose))
     printf ("\nExecution begins...\n");
 
   _gst_bytecode_counter = _gst_primitives_executed =
@@ -671,40 +678,36 @@ _gst_execute_statements (tree_node temporaries,
 
   /* send a message to NIL, which will find this synthetic method
      definition in Object and execute it */
-  _gst_prepare_execution_environment ();
-  PUSH_OOP (_gst_nil_oop);
-  _gst_send_method (methodOOP);
+  _gst_last_returned_value = _gst_nvmsg_send (_gst_nil_oop, methodOOP, NULL, 0);
+  INC_ADD_OOP (_gst_last_returned_value);
 
-  if (!IS_NIL (_gst_this_method))
-    {
-      _gst_init_process_system ();
-      _gst_interpret ();
-    }
+  endTime = _gst_get_milli_time ();
 #ifdef HAVE_GETRUSAGE
   getrusage (RUSAGE_SELF, &endRusage);
 #endif
-  endTime = _gst_get_milli_time ();
-  _gst_last_returned_value = _gst_finish_execution_environment ();
-  INC_RESTORE_POINTER (incPtr);
 
   if (_gst_kernel_initialized)
     _gst_invoke_hook ("afterEvaluation");
 
-  if (quiet)
-    return;
+  INC_RESTORE_POINTER (incPtr);
 
-  /* Do more frequent flushing to ensure the result are well
-     placed */
+  if (quiet)
+    return (_gst_last_returned_value);
+
+  /* Do more frequent flushing to ensure the result are well placed */
   if (_gst_regression_testing)
     fflush(stdout);
 
-  printf ("returned value is %O\n", _gst_last_returned_value);
+  if (_gst_regression_testing || _gst_verbose)
+    printf ("returned value is ");
+
+  printf ("%O\n", _gst_last_returned_value);
 
   if (_gst_regression_testing)
-    {
-      fflush(stdout);
-      return;
-    }
+    fflush(stdout);
+
+  if (!_gst_verbose)
+    return (_gst_last_returned_value);
 
   deltaTime = endTime - startTime;
   deltaTime += (deltaTime == 0);	/* it could be zero which would 
@@ -742,23 +745,18 @@ _gst_execute_statements (tree_node temporaries,
 	  endRusage.ru_nswap - startRusage.ru_nswap,
 	  endRusage.ru_minflt - startRusage.ru_minflt,
 	  endRusage.ru_majflt - startRusage.ru_majflt);
-
-  printf
-    ("(%ld voluntary context switches, %ld involuntary context switches)\n",
-     endRusage.ru_nvcsw - startRusage.ru_nvcsw,
-     endRusage.ru_nivcsw - startRusage.ru_nivcsw);
 #endif
 
 #ifdef USE_JIT_TRANSLATION
   if (!_gst_sample_counter)
-    return;
+    return (_gst_last_returned_value);
 
   printf
     ("%lu primitives, %lu inline cache misses since last cache cleanup\n",
      _gst_primitives_executed, _gst_sample_counter);
 #else
   if (!_gst_bytecode_counter)
-    return;
+    return (_gst_last_returned_value);
 
   printf ("%lu primitives, percent %.2f\n", _gst_primitives_executed,
 	  100.0 * _gst_primitives_executed / _gst_bytecode_counter);
@@ -778,8 +776,9 @@ _gst_execute_statements (tree_node temporaries,
 	    (100.0 * cacheHits) / _gst_sample_counter);
   else
     printf ("\n");
-}
 
+  return (_gst_last_returned_value);
+}
 #undef GIVING_X_BYTECODES_PER_SEC
 #undef BYTECODES_PER_SEC
 
@@ -787,6 +786,7 @@ _gst_execute_statements (tree_node temporaries,
 
 OOP
 _gst_compile_method (tree_node method,
+		     mst_Boolean returnLast,
 		     mst_Boolean install)
 {
   tree_node statement;
@@ -794,20 +794,22 @@ _gst_compile_method (tree_node method,
   OOP methodOOP;
   bytecodes bytecodes;
   int primitiveIndex;
+  int stack_depth;
   inc_ptr incPtr;
 
   dup_message_receiver = false;
   _gst_unregister_oop (_gst_latest_compiled_method);
   _gst_latest_compiled_method = _gst_nil_oop;
+  _gst_line_number (-1, true);
+  line_offset = method->location.first_line - 1;
 
   incPtr = INC_SAVE_POINTER ();
 
+  _gst_alloc_bytecodes ();
   _gst_push_new_scope ();
   inside_block = 0;
-  stack_depth = _gst_declare_arguments (method->v_method.selectorExpr);
-  stack_depth +=
-    _gst_declare_temporaries (method->v_method.temporaries);
-  max_stack_depth = stack_depth;
+  _gst_declare_arguments (method->v_method.selectorExpr);
+  _gst_declare_temporaries (method->v_method.temporaries);
 
   selector = compute_selector (method->v_method.selectorExpr);
   INC_ADD_OOP (selector);
@@ -821,43 +823,64 @@ _gst_compile_method (tree_node method,
 
   if (setjmp (bad_method) == 0)
     {
-      for (statement = method->v_method.statements; statement;
-	   statement = statement->v_list.next)
+      for (statement = method->v_method.statements; statement; )
 	{
+	  mst_Boolean wasReturn = statement->v_list.value->nodeType == TREE_RETURN_EXPR;
 	  compile_statement (statement->v_list.value);
-	  if (statement->v_list.value->nodeType != TREE_RETURN_EXPR)
-	    {
-	      /* ignore the result of the last statement if it's not
-	         used */
-	      SUB_STACK_DEPTH (1);
-	      _gst_compile_byte (POP_STACK_TOP);
 
-	      if (statement->v_list.next == NULL)
-		{
-		  /* compile a return of _gst_self.  Note that in
-		     theory the pop above is not necessary in this case 
-		     (and in fact previous versions did not put it),
-		     but having it simplifies the optimizer's task
-		     because it reduces the number of patterns it has
-		     to look for.  If necessary, the optimizer itself
-		     will remove the pop. */
-		  _gst_compile_byte (RETURN_INDEXED | RECEIVER_INDEX);
-		}
+	  statement = statement->v_list.next;
+	  if (wasReturn)
+	    continue;
+
+	  if (!statement && returnLast)
+	    /* compile a return of the last evaluated value.  Note that in
+	       theory the pop above is not necessary in this case 
+	       (and in fact older versions did not put it),
+	       but having it simplifies the optimizer's task
+	       because it reduces the number of patterns it has
+	       to look for.  If necessary, the optimizer itself
+	       will remove the pop. */
+	    {
+	      _gst_compile_byte (RETURN_CONTEXT_STACK_TOP);
+	      break;
 	    }
+
+	    /* ignore the result of the last statement if it's not
+	       used */
+	    SUB_STACK_DEPTH (1);
+	    _gst_compile_byte (POP_STACK_TOP);
+
+	    if (!statement)
+	      {
+	        /* compile a return of self.  Note that in
+	           theory the pop above is not necessary in this case 
+	           (and in fact older versions did not put it),
+	           but having it simplifies the optimizer's task
+	           because it reduces the number of patterns it has
+	           to look for.  If necessary, the optimizer itself
+	           will remove the pop. */
+	        _gst_compile_byte (RETURN_INDEXED | RECEIVER_INDEX);
+		break;
+	      }
 	}
 
       if (method->v_method.statements == NULL)
 	{
-	  /* special case an empty statement body to return _gst_self */
-	  _gst_compile_byte (RETURN_INDEXED | RECEIVER_INDEX);
+	  if (returnLast)
+	    /* special case an empty statement body to return _gst_self */
+	    _gst_compile_byte (RETURN_INDEXED | NIL_INDEX);
+	  else
+	    /* special case an empty statement body to return _gst_self */
+	    _gst_compile_byte (RETURN_INDEXED | RECEIVER_INDEX);
 	}
 
+      stack_depth = GET_STACK_DEPTH ();
       bytecodes = _gst_get_bytecodes ();
 
       methodOOP = _gst_make_new_method (primitiveIndex,
 					_gst_get_arg_count (),
 					_gst_get_temp_count (),
-					max_stack_depth, NULL, bytecodes,
+					stack_depth, NULL, bytecodes,
 					_gst_this_class, selector);
       INC_ADD_OOP (methodOOP);
 
@@ -877,7 +900,6 @@ _gst_compile_method (tree_node method,
     }
 
   _gst_pop_all_scopes ();
-  _gst_free_tree (method);
 
   INC_RESTORE_POINTER (incPtr);
   return (_gst_latest_compiled_method);
@@ -899,6 +921,7 @@ compile_statement (tree_node stmt)
     {
       compile_expression (receiver);
       _gst_compile_byte (RETURN_METHOD_STACK_TOP);
+      SUB_STACK_DEPTH (1);
       return;
     }
 
@@ -907,7 +930,7 @@ compile_statement (tree_node stmt)
       OOP symbol = _gst_intern_string (receiver->v_list.name);
       symbol_entry variable;
 
-      if (_gst_find_variable (&variable, symbol)
+      if (_gst_find_variable (&variable, receiver)
 	  && variable.scope == SCOPE_SPECIAL
 	  && symbol != _gst_this_context_symbol)
 	{
@@ -916,8 +939,9 @@ compile_statement (tree_node stmt)
 	}
     }
 
-  compile_expression (stmt->v_expr.receiver);
+  compile_expression (receiver);
   _gst_compile_byte (RETURN_CONTEXT_STACK_TOP);
+  SUB_STACK_DEPTH (1);
 }
 
 void
@@ -935,6 +959,7 @@ compile_expression (tree_node expr)
 void
 compile_simple_expression (tree_node expr)
 {
+  _gst_line_number (expr->location.first_line - line_offset, false);
   switch (expr->nodeType)
     {
     case TREE_VARIABLE_NODE:
@@ -971,19 +996,22 @@ compile_variable (tree_node varName)
 {
   symbol_entry variable;
   int location;
-  OOP symbol;
 
-  symbol = _gst_intern_string (varName->v_list.name);
-  if (!_gst_find_variable (&variable, symbol))
+  INCR_STACK_DEPTH ();
+  if (!_gst_find_variable (&variable, varName))
     {
-      _gst_errorf ("Undefined variable %s referenced",
-		   varName->v_list.name);
+      if (varName->v_list.next)
+        _gst_errorf_at (varName->location.first_line,
+			"invalid scope resolution");
+      else
+	_gst_errorf_at (varName->location.first_line,
+			"undefined variable %s referenced",
+		        varName->v_list.name);
       EXIT_COMPILATION ();
     }
 
   if (variable.scope == SCOPE_SPECIAL)
     {
-      INCR_STACK_DEPTH ();
       _gst_compile_byte (PUSH_SPECIAL + variable.varIndex);
       return;
     }
@@ -994,7 +1022,13 @@ compile_variable (tree_node varName)
       return;
     }
 
-  INCR_STACK_DEPTH ();
+  if (varName->v_list.next)
+    {
+      _gst_errorf_at (varName->location.first_line,
+		      "invalid scope resolution");
+      EXIT_COMPILATION ();
+    }
+
   if (variable.scopeDistance != 0)
     {
       /* must be a temporary from an outer scope */
@@ -1027,14 +1061,25 @@ compile_variable (tree_node varName)
 void
 compile_constant (tree_node constExpr)
 {
-  compile_push_literal (add_constant (constExpr));
+  int index = add_constant (constExpr);
+  int value;
+
+  INCR_STACK_DEPTH ();
+  if COMMON (index != -1)
+      compile_push_literal (index);
+  else
+    {
+      value = constExpr->v_const.val.iVal;
+      compile_push_int_constant(value);
+    }
 }
 
 void
 compile_block (tree_node blockExpr)
 {
   bytecodes current_bytecodes, blockByteCodes;
-  int oldStackDepth, oldMaxStackDepth, argCount, tempCount;
+  int argCount, tempCount;
+  int stack_depth;
   OOP blockClosureOOP;
   gst_block_closure blockClosure;
   gst_compiled_block block;
@@ -1043,21 +1088,16 @@ compile_block (tree_node blockExpr)
   current_bytecodes = _gst_save_bytecode_array ();
 
   _gst_push_new_scope ();
-  oldStackDepth = stack_depth;
-  oldMaxStackDepth = max_stack_depth;
-  stack_depth = max_stack_depth = 0;
-  argCount =
-    _gst_declare_block_arguments (blockExpr->v_block.arguments);
+  argCount = _gst_declare_block_arguments (blockExpr->v_block.arguments);
   tempCount = _gst_declare_temporaries (blockExpr->v_block.temporaries);
-  ADD_STACK_DEPTH (argCount + tempCount);	/* this updates
-						   max_stack_depth too */
   compile_statements (blockExpr->v_block.statements, true);
   _gst_pop_old_scope ();
 
+  stack_depth = GET_STACK_DEPTH ();
   blockByteCodes = _gst_get_bytecodes ();
 
   blockClosureOOP =
-    make_block_closure (argCount, tempCount, blockByteCodes);
+    make_block_closure (argCount, tempCount, blockByteCodes, stack_depth);
   blockClosure = (gst_block_closure) OOP_TO_OBJ (blockClosureOOP);
   block = (gst_compiled_block) OOP_TO_OBJ (blockClosure->block);
 
@@ -1067,6 +1107,7 @@ compile_block (tree_node blockExpr)
      push literal (a BlockClosure)
      push thisContext -> may become push nil when optimizing
      blockCopy send -> may not be there at all when optimizing */
+  INCR_STACK_DEPTH ();
   compile_push_literal (add_literal (blockClosureOOP));
   switch (block->header.clean)
     {
@@ -1083,8 +1124,6 @@ compile_block (tree_node blockExpr)
       _gst_compile_byte (BLOCK_COPY_COLON_SPECIAL);
       break;
     }
-  stack_depth = oldStackDepth;
-  max_stack_depth = oldMaxStackDepth;
 }
 
 
@@ -1107,7 +1146,10 @@ compile_statements (tree_node statementList,
     }
 
   if (isBlock)
-    inside_block++;
+    {
+      _gst_line_number (statementList->location.first_line - line_offset, true);
+      inside_block++;
+    }
 
   for (stmt = statementList;; stmt = stmt->v_list.next)
     {
@@ -1129,7 +1171,10 @@ compile_statements (tree_node statementList,
     _gst_compile_byte (RETURN_CONTEXT_STACK_TOP);
 
   if (isBlock)
-    inside_block--;
+    {
+      _gst_line_number (-1, false);
+      inside_block--;
+    }
 
   return (stmt->v_list.value->nodeType == TREE_RETURN_EXPR);
 }
@@ -1184,11 +1229,6 @@ compile_unary_expr (tree_node expr)
   dup_message_receiver = false;
 
   selector = expr->v_expr.selector;
-  if (IS_NIL (selector))
-    {
-      _gst_errorf ("Nil selector in unary expression");
-      EXIT_COMPILATION ();
-    }
 
   /* check for optimized cases of messages to blocks and handle them
      specially */
@@ -1211,7 +1251,7 @@ compile_unary_expr (tree_node expr)
       if (savedDupFlag)
 	{
 	  _gst_compile_byte (DUP_STACK_TOP);
-	  INCR_STACK_DEPTH();
+          INCR_STACK_DEPTH ();
 	}
 
       if (is_super (expr->v_expr.receiver))
@@ -1233,7 +1273,8 @@ compile_unary_expr (tree_node expr)
 	    }
 	  else
 	    {
-	      _gst_errorf ("gst_message send cannot be represented");
+	      _gst_errorf_at (expr->location.first_line,
+			      "message send cannot be represented");
 	      EXIT_COMPILATION ();
 	    }
 	  return;
@@ -1265,7 +1306,7 @@ compile_binary_expr (tree_node expr)
       if (savedDupFlag)
 	{
 	  _gst_compile_byte (DUP_STACK_TOP);
-	  INCR_STACK_DEPTH();
+          INCR_STACK_DEPTH ();
 	}
     }
 
@@ -1295,19 +1336,14 @@ compile_binary_expr (tree_node expr)
 	    }
 	  else
 	    {
-	      _gst_errorf ("gst_message send cannot be represented");
+	      _gst_errorf_at (expr->location.first_line,
+	      		      "message send cannot be represented");
 	      EXIT_COMPILATION ();
 	    }
 	  return;
 	}
     }
 
-  if (IS_NIL (selector))
-    {
-      _gst_errorf ("nil selector in binary expression");
-      EXIT_COMPILATION ();
-    }
-
   if (selectorIndex < 0)
     _gst_compile_byte (-selectorIndex);
   else
@@ -1341,7 +1377,7 @@ compile_keyword_expr (tree_node expr)
       if (savedDupFlag)
 	{
 	  _gst_compile_byte (DUP_STACK_TOP);
-	  INCR_STACK_DEPTH();
+          INCR_STACK_DEPTH ();
 	}
     }
 
@@ -1411,7 +1447,8 @@ compile_keyword_expr (tree_node expr)
 	    }
 	  else
 	    {
-	      _gst_errorf ("gst_message send cannot be represented");
+	      _gst_errorf_at (expr->location.first_line,
+	      		      "message send cannot be represented");
 	      EXIT_COMPILATION ();
 	    }
 	  return;
@@ -1553,20 +1590,19 @@ compile_times_repeat (tree_node expr)
 
   _gst_compile_byte (DUP_STACK_TOP);
   INCR_STACK_DEPTH ();
-  _gst_compile_byte (PUSH_SPECIAL | LIT_ONE_INDEX);
+  _gst_compile_byte (PUSH_ONE);
   INCR_STACK_DEPTH ();
   _gst_compile_byte (GREATER_EQUAL_SPECIAL);
   SUB_STACK_DEPTH (1);
 
   /* skip the loop if there are no more occurrences */
   compile_jump (_gst_bytecode_length (loopExprCodes) + 5, JUMP_FALSE);
-  SUB_STACK_DEPTH (1);
-  _gst_compile_byte (PUSH_SPECIAL | LIT_ONE_INDEX);
+  _gst_compile_byte (PUSH_ONE);
   INCR_STACK_DEPTH ();
   _gst_compile_byte (MINUS_SPECIAL);
   SUB_STACK_DEPTH (1);
 
-  /* we don't care about while expr's value */
+  /* we don't care about block expr's value */
   _gst_compile_and_free_bytecodes (loopExprCodes);
   _gst_compile_byte (POP_STACK_TOP);
   SUB_STACK_DEPTH (1);
@@ -1631,7 +1667,6 @@ compile_to_by_do (tree_node to,
     {
       bytecodes current_bytecodes;
       current_bytecodes = _gst_save_bytecode_array ();
-
       compile_expression (by);
       stepCodes = _gst_get_bytecodes ();
       _gst_restore_bytecode_array (current_bytecodes);
@@ -1673,7 +1708,7 @@ compile_to_by_do (tree_node to,
     _gst_compile_and_free_bytecodes (stepCodes);
   else
     {
-      _gst_compile_byte (PUSH_SPECIAL | LIT_ONE_INDEX);
+      _gst_compile_byte (PUSH_ONE);
       INCR_STACK_DEPTH ();
     }
 
@@ -1807,10 +1842,13 @@ compile_and_or_statement (OOP selector,
 bytecodes
 compile_sub_expression (tree_node expr)
 {
+  mst_Boolean returns;
   bytecodes current_bytecodes, subExprByteCodes;
 
   current_bytecodes = _gst_save_bytecode_array ();
-  compile_statements (expr->v_block.statements, false);
+  returns = compile_statements (expr->v_block.statements, false);
+  if (returns)
+    INCR_STACK_DEPTH ();
 
   subExprByteCodes = _gst_get_bytecodes ();
   _gst_restore_bytecode_array (current_bytecodes);
@@ -1828,6 +1866,9 @@ compile_sub_expression_and_jump (tree_node expr,
 
   current_bytecodes = _gst_save_bytecode_array ();
   returns = compile_statements (expr->v_block.statements, false);
+  if (returns)
+    INCR_STACK_DEPTH ();
+
   if (!returns)
     compile_jump (branchLen, JUMP_UNCONDITIONAL);
 
@@ -1843,13 +1884,13 @@ compile_jump (int len,
 {
   if (len <= 0)
     {
-      _gst_errorf ("Invalid length jump %d -- internal error\n", len);
+      _gst_errorf ("invalid length jump %d -- internal error\n", len);
       EXIT_COMPILATION ();
     }
 
   if (len >= 1024)
     {
-      _gst_errorf ("Invalid length jump %d -- method too complex\n",
+      _gst_errorf ("invalid length jump %d -- method too complex\n",
 		   len);
       EXIT_COMPILATION ();
     }
@@ -1892,7 +1933,7 @@ compile_long_jump (int len)
   len += 1024;
   if (len <= 0 || len >= 2048)
     {
-      _gst_errorf ("Invalid length jump %d -- method too complex\n",
+      _gst_errorf ("invalid length jump %d -- method too complex\n",
 		   len);
       EXIT_COMPILATION ();
     }
@@ -1902,18 +1943,31 @@ compile_long_jump (int len)
 }
 
 void
-compile_push_int_constant (int CONST_INT)
+compile_push_int_constant (int value)
 {
   OOP intOOP;
 
-  if (CONST_INT >= -1 && CONST_INT <= 2)
+  if (value == 0 || value == 1)
     {
-      INCR_STACK_DEPTH ();
-      _gst_compile_byte (PUSH_SPECIAL | (CONST_INT + 5));
+      _gst_compile_byte (PUSH_ZERO + value);
       return;
     }
 
-  intOOP = FROM_INT ((long) CONST_INT);
+  if (value >= 0 && value <= 255)
+    {
+      _gst_compile_byte (PUSH_UNSIGNED_8);
+      _gst_compile_byte (value);
+      return;
+    }
+
+  if (value >= -128 && value <= 127)
+    {
+      _gst_compile_byte (PUSH_SIGNED_8);
+      _gst_compile_byte (value);
+      return;
+    }
+
+  intOOP = FROM_INT ((long) value);
   compile_push_literal (_gst_add_forced_object (intOOP));
 }
 
@@ -1950,7 +2004,7 @@ compile_send (int selectorIndex,
     }
   else
     {
-      _gst_errorf ("gst_message send cannot be represented (yet)");
+      _gst_errorf ("message send cannot be represented (yet)");
       EXIT_COMPILATION ();
     }
 }
@@ -1985,23 +2039,31 @@ compile_assignments (tree_node varList)
 {
   symbol_entry variable;
   int locationIndex;
-  OOP symbol;
 
   for (; varList; varList = varList->v_list.next)
     {
-      symbol = _gst_intern_string (varList->v_list.name);
-      if (!_gst_find_variable (&variable, symbol))
+      tree_node varName = varList->v_list.value;
+
+      _gst_line_number (varList->location.first_line - line_offset, false);
+      if (!_gst_find_variable (&variable, varName))
 	{
-	  _gst_errorf ("assignment to undeclared variable %s",
-		       varList->v_list.name);
+          if (varName->v_list.next)
+            _gst_errorf_at (varName->location.first_line,
+			    "invalid scope resolution");
+          else
+            _gst_errorf_at (varName->location.first_line,
+    	    		    "assignment to undeclared variable %s",
+		            varName->v_list.name);
 	  EXIT_COMPILATION ();
 	}
+
       if (variable.readOnly)
 	{
-	  _gst_errorf ("invalid assignment to %s %s",
-		       variable.scope ==
-		       SCOPE_SPECIAL ? "special variable" : "argument",
-		       varList->v_list.name);
+	  _gst_errorf_at (varName->location.first_line,
+                          "invalid assignment to %s %s",
+		          variable.scope ==
+		          SCOPE_SPECIAL ? "special variable" : "argument",
+		          varName->v_list.name);
 
 	  EXIT_COMPILATION ();
 	}
@@ -2010,6 +2072,13 @@ compile_assignments (tree_node varList)
          variable, temporary variable, "literal" variable (reference by 
          association). */
 
+      if (variable.scope != SCOPE_GLOBAL && varName->v_list.next)
+        {
+          _gst_errorf_at (varName->location.first_line,
+                          "invalid scope resolution");
+          EXIT_COMPILATION ();
+        }
+
       if (variable.scope == SCOPE_TEMPORARY
 	  && variable.scopeDistance > 0)
 	{
@@ -2062,8 +2131,8 @@ add_constant (tree_node constExpr)
   if (IS_INT (constantOOP))
     {
       intVal = TO_INT (constantOOP);
-      if (intVal >= -1 && intVal <= 2)
-	return ((int) (intVal - 3));
+      if (intVal >= -128 && intVal <= 255)
+	return (-1);
     }
   else
     MAKE_OOP_READONLY (constantOOP, true);
@@ -2170,7 +2239,7 @@ equal_constant (OOP oop,
 	    }
 	}
       break;
-  
+
     default:
       break;
     }
@@ -2275,7 +2344,8 @@ make_constant_oop (tree_node constExpr)
 OOP
 make_block_closure (int args,
 		    int temps,
-		    bytecodes bytecodes)
+		    bytecodes bytecodes,
+		    int stack_depth)
 {
   gst_block_closure closure;
   OOP blockOOP, closureOOP;
@@ -2298,8 +2368,7 @@ make_block_closure (int args,
      closure's value might be no longer valid!) */
   incPtr = INC_SAVE_POINTER ();
   blockOOP =
-    _gst_block_new (args, temps, bytecodes, max_stack_depth,
-		    literal_vec);
+    _gst_block_new (args, temps, bytecodes, stack_depth, literal_vec);
   INC_ADD_OOP (blockOOP);
 
   closure = (gst_block_closure) new_instance (_gst_block_closure_class, 
@@ -2438,7 +2507,7 @@ list_length (tree_node listExpr)
     {
       if (len > (1L << (sizeof (int) * 8 - 1)))
 	{
-	  _gst_errorf ("List too long, %ld", len);
+	  _gst_errorf ("list too long, %ld", len);
 	  len = 1L << (sizeof (int) * 8 - 1);
 	}
     }
@@ -2516,11 +2585,7 @@ get_literals_array (void)
 void
 compile_push_literal (int index)
 {
-  INCR_STACK_DEPTH ();
-  if (index < 0)
-    _gst_compile_byte (PUSH_SPECIAL | 4 | (index & 3));
-
-  else if (index <= 31)
+  if (index <= 31)
     _gst_compile_byte (PUSH_LIT_CONSTANT | index);
 
   else if (index <= 63)
@@ -2599,12 +2664,13 @@ _gst_make_new_method (int primitiveIndex,
   OOP method;
   inc_ptr incPtr;
 
+  maximumStackDepth += numArgs + numTemps;
+
   if (primitiveIndex)
     {
       if (_gst_declare_tracing)
-	{
-	  printf ("  Primitive Index %d\n", primitiveIndex);
-	}
+	printf ("  Primitive Index %d\n", primitiveIndex);
+
       header.headerFlag = 4;
     }
   else if (numArgs == 0 && numTemps == 0 &&
@@ -2639,6 +2705,9 @@ _gst_make_new_method (int primitiveIndex,
       bytecodes = _gst_optimize_bytecodes (bytecodes);
     }
 
+  if (_gst_declare_tracing)
+    printf ("  Allocated stack slots %d\n", maximumStackDepth);
+
   if (_gst_declare_tracing)
     _gst_print_bytecodes (bytecodes, OOP_TO_OBJ (literals)->data);
 
@@ -2711,15 +2780,15 @@ OOP
 _gst_block_new (int numArgs,
 		int numTemps,
 		bytecodes bytecodes,
-		int depth,
+		int maximumStackDepth,
 		OOP * literals)
 {
-  int numByteCodes, maximumStackDepth;
+  int numByteCodes;
   OOP blockOOP;
   gst_compiled_block block;
   block_header header;
 
-  maximumStackDepth = depth;
+  maximumStackDepth += numArgs + numTemps;
   maximumStackDepth += (1 << DEPTH_SCALE) - 1;	/* round */
   maximumStackDepth >>= DEPTH_SCALE;
   maximumStackDepth++;		/* just to be sure */
