@@ -225,12 +225,11 @@ extern mst_Boolean _gst_had_error;
 
 /* Called to compile and execute an "immediate expression"; i.e. a set
    of Smalltalk statements that are not part of a method definition.
-   The parse trees are in TEMPORARIES and STATEMENTS.  QUIET indicates
-   whether messages are to be output indicating the commencement of
-   execution and some timing results at the end of execution. */
-extern void _gst_execute_statements (tree_node temporaries,
-				     tree_node statements,
-				     mst_Boolean quiet);
+   The parse trees are in TEMPORARIES and STATEMENTS.  Return the object
+   that was returned by the expression. */
+extern OOP _gst_execute_statements (tree_node temporaries,
+				    tree_node statements,
+				    mst_Boolean quiet);
 
 /* This function will print a message describing the method category
    and class being compiled.  The message can have the form "STRING
@@ -289,6 +288,7 @@ extern void _gst_init_compiler (void);
    installs it in the current method dictionary with the selector
    derived from the method expression. */
 extern OOP _gst_compile_method (tree_node method,
+				mst_Boolean returnLast,
 				mst_Boolean install);
 
 /* Returns the descriptor for the given CompiledMethod METHODOOP,
