@@ -71,7 +71,9 @@ OOP _gst_object_class, _gst_magnitude_class, _gst_char_class,
   _gst_object_memory_class, _gst_large_integer_class,
   _gst_large_negative_integer_class, _gst_large_zero_integer_class,
   _gst_large_positive_integer_class, _gst_association_class,
+  _gst_homed_association_class, _gst_variable_binding_class,
   _gst_link_class, _gst_process_class, _gst_sym_link_class,
+  _gst_callin_process_class,
   _gst_collection_class, _gst_lookup_key_class,
   _gst_sequenceable_collection_class, _gst_linked_list_class,
   _gst_semaphore_class, _gst_character_array_class,
@@ -81,6 +83,7 @@ OOP _gst_object_class, _gst_magnitude_class, _gst_char_class,
   _gst_interval_class, _gst_ordered_collection_class,
   _gst_sorted_collection_class, _gst_bag_class,
   _gst_mapped_collection_class, _gst_set_class, _gst_dictionary_class,
+  _gst_abstract_namespace_class, _gst_binding_dictionary_class,
   _gst_namespace_class, _gst_system_dictionary_class,
   _gst_identity_set_class, _gst_hashed_collection_class,
   _gst_method_dictionary_class, _gst_identity_dictionary_class,
@@ -90,7 +93,8 @@ OOP _gst_object_class, _gst_magnitude_class, _gst_char_class,
   _gst_shared_queue_class, _gst_behavior_class,
   _gst_root_namespace_class, _gst_class_description_class,
   _gst_class_class, _gst_metaclass_class, _gst_smalltalk_dictionary,
-  _gst_message_class, _gst_method_context_class,
+  _gst_message_class, _gst_directed_message_class,
+  _gst_method_context_class,
   _gst_block_context_class, _gst_block_closure_class,
   _gst_byte_stream_class, _gst_stream_class,
   _gst_positionable_stream_class, _gst_read_stream_class,
@@ -114,6 +118,12 @@ static void init_proto_oops (void);
 /* Create a new instance of IdentityDictionary and answer it. */
 static OOP identity_dictionary_new (int size);
 
+/* Create a new instance of Namespace with the given SIZE, NAME and
+   superspace (SUPERSPACEOOP). */
+static OOP namespace_new (int size,
+			  char *name,
+			  OOP superspaceOOP);
+
 /* Create new subclass of SUPERCLASSOOP, whose instances have
    a shape defined by the remaining parameters. */
 static OOP new_class (OOP superClassOOP,
@@ -193,16 +203,30 @@ static const char *feature_strings[] = {
 static const class_definition class_info[] = {
   {&_gst_object_class, NULL,
    true, false, false, 0,
-   "Object", NULL, NULL, NULL },
+   "Object", NULL, "Dependencies FinalizableObjects", NULL },
 
   {&_gst_object_memory_class, &_gst_object_class,
-   true, false, false, 0,
-   "ObjectMemory", NULL, NULL, NULL },
+   true, false, false, 34,
+   "ObjectMemory", "bytesPerOOP bytesPerOTE "
+   "edenSize survSpaceSize oldSpaceSize fixedSpaceSize "
+   "edenUsedBytes survSpaceUsedBytes oldSpaceUsedBytes "
+   "fixedSpaceUsedBytes rememberedTableEntries "
+   "numScavenges numGlobalGCs numCompactions numGrowths "
+   "numOldOOPs numFixedOOPs numWeakOOPs numOTEs numFreeOTEs "
+   "timeBetweenScavenges timeBetweenGlobalGCs timeBetweenGrowths "
+   "timeToScavenge timeToCollect timeToCompact "
+   "reclaimedBytesPerScavenge tenuredBytesPerScavenge "
+   "reclaimedBytesPerGlobalGC reclaimedPercentPerScavenge "
+   "allocFailures allocMatches allocSplits allocProbes", NULL, NULL },
 
   {&_gst_message_class, &_gst_object_class,
    true, false, false, 2,
    "Message", "selector args", NULL, NULL },
 
+  {&_gst_directed_message_class, &_gst_message_class,
+   true, false, false, 1,
+   "DirectedMessage", "receiver", NULL, NULL },
+
   {&_gst_magnitude_class, &_gst_object_class,
    true, false, false, 0,
    "Magnitude", NULL, NULL, NULL },
@@ -280,14 +304,28 @@ static const class_definition class_info[] = {
    true, false, false, 1,
    "Association", "value", NULL, NULL },
 
+  {&_gst_homed_association_class, &_gst_association_class,
+   true, false, false, 1,
+   "HomedAssociation", "environment", NULL, NULL },
+
+  {&_gst_variable_binding_class, &_gst_homed_association_class,
+   true, false, false, 0,
+   "VariableBinding", NULL, NULL, NULL },
+
   {&_gst_link_class, &_gst_object_class,
    true, false, false, 1,
    "Link", "nextLink", NULL, NULL },
 
   {&_gst_process_class, &_gst_link_class,
-   true, false, false, 5,
+   true, false, false, 7,
    "Process",
-   "suspendedContext priority myList name unwindPoints",
+   "suspendedContext priority myList name unwindPoints interrupts interruptLock",
+   NULL, NULL },
+
+  {&_gst_callin_process_class, &_gst_process_class,
+   true, false, false, 1,
+   "CallinProcess",
+   "returnedValue",
    NULL, NULL },
 
   {&_gst_sym_link_class, &_gst_link_class,
@@ -307,8 +345,8 @@ static const class_definition class_info[] = {
    "LinkedList", "firstLink lastLink", NULL, NULL },
 
   {&_gst_semaphore_class, &_gst_linked_list_class,
-   true, false, false, 1,
-   "Semaphore", "signals", NULL, NULL },
+   true, false, false, 2,
+   "Semaphore", "signals name", NULL, NULL },
 
   {&_gst_arrayed_collection_class, &_gst_sequenceable_collection_class,
    true, false, true, 0,
@@ -399,13 +437,21 @@ static const class_definition class_info[] = {
    true, false, true, 0,
    "MethodDictionary", NULL, NULL, NULL },
 
-  /* These three MUST have the same structure as dictionary; they're
+  /* These five MUST have the same structure as dictionary; they're
      used interchangeably within the C portion of the system */
-  {&_gst_root_namespace_class, &_gst_dictionary_class,
+  {&_gst_binding_dictionary_class, &_gst_dictionary_class,
+   true, false, true, 1,
+   "BindingDictionary", "environment", NULL, NULL },
+
+  {&_gst_abstract_namespace_class, &_gst_binding_dictionary_class,
+   true, false, true, 2,
+   "AbstractNamespace", "name subspaces", NULL, NULL },
+
+  {&_gst_root_namespace_class, &_gst_abstract_namespace_class,
    true, false, true, 0,
    "RootNamespace", NULL, NULL, NULL },
 
-  {&_gst_namespace_class, &_gst_root_namespace_class,
+  {&_gst_namespace_class, &_gst_abstract_namespace_class,
    true, false, true, 0,
    "Namespace", NULL, "Current", NULL },
 
@@ -565,7 +611,7 @@ static const class_definition class_info[] = {
 void
 init_proto_oops()
 {
-  gst_dictionary smalltalkDictionary;
+  gst_namespace smalltalkDictionary;
   mst_Object symbolTable, processorScheduler;
   int numWords;
 
@@ -575,17 +621,7 @@ init_proto_oops()
   /* Also finish the creation of the OOPs with reserved indices in
      oop.h */
 
-  numWords = OBJ_HEADER_SIZE_WORDS + INITIAL_SMALLTALK_SIZE + 1;
-
-  smalltalkDictionary = (gst_dictionary) _gst_alloc_words (numWords);
-  SET_OOP_OBJECT (_gst_smalltalk_dictionary, smalltalkDictionary);
-
-  smalltalkDictionary->objClass = _gst_system_dictionary_class;
-  smalltalkDictionary->tally = FROM_INT(0);
-  nil_fill (smalltalkDictionary->assoc,
-	    INITIAL_SMALLTALK_SIZE);
-
-  /* ... now the symbol table ... */
+  /* the symbol table ... */
   numWords = OBJ_HEADER_SIZE_WORDS + SYMBOL_TABLE_SIZE;
   symbolTable = _gst_alloc_words (numWords);
   SET_OOP_OBJECT (_gst_symbol_table, symbolTable);
@@ -594,6 +630,20 @@ init_proto_oops()
   nil_fill (symbolTable->data,
 	    numWords - OBJ_HEADER_SIZE_WORDS);
 
+  numWords = OBJ_HEADER_SIZE_WORDS + INITIAL_SMALLTALK_SIZE + 4;
+
+  /* ... now the Smalltalk dictionary ... */
+  smalltalkDictionary = (gst_namespace) _gst_alloc_words (numWords);
+  SET_OOP_OBJECT (_gst_smalltalk_dictionary, smalltalkDictionary);
+
+  smalltalkDictionary->objClass = _gst_system_dictionary_class;
+  smalltalkDictionary->tally = FROM_INT(0);
+  smalltalkDictionary->name = _gst_intern_string ("Smalltalk");
+  smalltalkDictionary->superspace = _gst_nil_oop;
+  smalltalkDictionary->subspaces = _gst_nil_oop;
+  nil_fill (smalltalkDictionary->assoc,
+	    INITIAL_SMALLTALK_SIZE);
+
   /* ... and finally Processor */
   numWords = sizeof (struct gst_processor_scheduler) / sizeof (long);
   processorScheduler = _gst_alloc_words (numWords);
@@ -608,9 +658,9 @@ void
 _gst_init_dictionary (void)
 {
   /* The order of this must match the indices defined in oop.h!! */
-  _gst_smalltalk_dictionary = alloc_oop (NULL, 0);
-  _gst_processor_oop = alloc_oop (NULL, 0);
-  _gst_symbol_table = alloc_oop (NULL, 0);
+  _gst_smalltalk_dictionary = alloc_oop (NULL, _gst_mem.active_flag);
+  _gst_processor_oop = alloc_oop (NULL, _gst_mem.active_flag);
+  _gst_symbol_table = alloc_oop (NULL, _gst_mem.active_flag);
 
   create_classes_pass1 ();
 
@@ -622,8 +672,10 @@ _gst_init_dictionary (void)
   add_smalltalk ("KernelFilePath",
 		 _gst_string_new (_gst_kernel_file_default_path));
 
-  _gst_init_symbols ();
   init_runtime_objects ();
+
+  _gst_scavenge ();
+  _gst_tenure_all_survivors ();
 }
 
 void
@@ -683,17 +735,14 @@ create_classes_pass2 (void)
 	subClasses->data[0] = FROM_INT (index);
 
       if (class_oop == _gst_class_class)
-	{
-	  /* we don't want the meta class to have a subclass if we're special
-	     casing Object class, so back off the number of sub classes for
-	     the meta class. */
-	  index--;
-	}
+	/* we don't want the meta class to have a subclass if we're special
+	   casing Object class, so back off the number of sub classes for
+	   the meta class. */
+	index--;
+
       if (class_oop == _gst_object_class)
-	{			/* is this Object? */
-	  /* _gst_nil_oop wasn't available during pass1, but now it is */
-	  class->superclass = _gst_nil_oop;
-	}
+	/* _gst_nil_oop wasn't available during pass1, but now it is */
+	class->superclass = _gst_nil_oop;
       else
 	{
 	  /* hack the parent's subclass array */
@@ -709,13 +758,14 @@ create_classes_pass2 (void)
 	      add_subclass (class_oop, OOP_CLASS (_gst_object_class));
 	    }
 	}
+
       class->objClass = new_metaclass (class_oop, index);
       class->environment = _gst_smalltalk_dictionary;
       class->instanceVariables =
 	_gst_make_instance_variable_array (class->superclass,
 					   ci->instVarNames);
       class->classVariables =
-	_gst_make_class_variable_dictionary (ci->classVarNames);
+	_gst_make_class_variable_dictionary (ci->classVarNames, class_oop);
       class->sharedPools = _gst_make_pool_array (ci->sharedPoolNames);
 
       class->comment = _gst_nil_oop;	/* not used yet. */
@@ -812,20 +862,27 @@ init_smalltalk_dictionary (void)
   sprintf (fullVersionString, "GNU Smalltalk version %s", VERSION);
 
   add_smalltalk ("Smalltalk", _gst_smalltalk_dictionary);
-  add_smalltalk ("CFunctionDescs", _gst_dictionary_new (32));
-  add_smalltalk ("CSymbols", _gst_dictionary_new (32));
   add_smalltalk ("Version", _gst_string_new (fullVersionString));
   add_smalltalk ("CObjectType", _gst_c_object_type_ctype);
   add_smalltalk ("KernelInitialized", _gst_false_oop);
   add_smalltalk ("SymbolTable", _gst_symbol_table);
   add_smalltalk ("Processor", _gst_processor_oop);
-  add_smalltalk ("Undeclared", _gst_dictionary_new (32));
-
-  /* Smalltalk must be a proper namespace! */
-  add_smalltalk ("Super", _gst_nil_oop);
-
   add_smalltalk ("Features", featuresArrayOOP);
 
+  _gst_init_symbols ();
+
+  /* Add subspaces */
+  add_smalltalk ("CSymbols", _gst_dictionary_new (32));
+
+  add_smalltalk ("Undeclared",
+    namespace_new (32, "Undeclared", _gst_nil_oop));
+  add_smalltalk ("SystemExceptions",
+    namespace_new (32, "SystemExceptions", _gst_smalltalk_dictionary));
+  add_smalltalk ("NetClients",
+    namespace_new (32, "NetClients", _gst_smalltalk_dictionary));
+  add_smalltalk ("VFS",
+    namespace_new (32, "VFS", _gst_smalltalk_dictionary));
+
   _gst_init_process_system ();
 }
 
@@ -833,8 +890,8 @@ static OOP
 add_smalltalk (char *globalName,
 	       OOP globalValue)
 {
-  DICTIONARY_AT_PUT (_gst_smalltalk_dictionary,
-		     _gst_intern_string (globalName), globalValue);
+  NAMESPACE_AT_PUT (_gst_smalltalk_dictionary,
+		    _gst_intern_string (globalName), globalValue);
 
   return globalValue;
 }
@@ -876,83 +933,90 @@ init_c_symbols ()
   OOP cSymbolsOOP = dictionary_at (_gst_smalltalk_dictionary,
 				   _gst_intern_string ("CSymbols"));
 
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CIntSize"),
-		     FROM_INT (sizeof (int)));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CShortSize"),
-		     FROM_INT (sizeof (short)));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongSize"),
-		     FROM_INT (sizeof (long)));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatSize"),
-		     FROM_INT (sizeof (float)));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleSize"),
-		     FROM_INT (sizeof (double)));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleSize"),
-		     FROM_INT (sizeof (long double)));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CPtrSize"),
-		     FROM_INT (sizeof (PTR)));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CIntSize"),
+		    FROM_INT (sizeof (int)));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CShortSize"),
+		    FROM_INT (sizeof (short)));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongSize"),
+		    FROM_INT (sizeof (long)));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatSize"),
+		    FROM_INT (sizeof (float)));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleSize"),
+		    FROM_INT (sizeof (double)));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleSize"),
+		    FROM_INT (sizeof (long double)));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CPtrSize"),
+		    FROM_INT (sizeof (PTR)));
 
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMin"),
-		     floatd_new (DBL_MIN));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMax"),
-		     floatd_new (DBL_MAX));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoublePInf"),
-		     floatd_new (DBL_MAX * 2));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleNInf"),
-		     floatd_new (DBL_MAX * -2));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleNaN"),
-		     floatd_new (0.0 / 0.0));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleDigits"),
-		     FROM_INT (DBL_DIG));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleBinaryDigits"),
-		     FROM_INT (DBL_MANT_DIG));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMinExp"),
-		     FROM_INT (DBL_MIN_EXP));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMaxExp"),
-		     FROM_INT (DBL_MAX_EXP));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleAlignment"),
-		     FROM_INT (DOUBLE_ALIGNMENT));
+#ifndef INFINITY
+#define INFINITY LDBL_MAX * 2
+#endif
+#ifndef NAN
+#define NAN (0.0 / 0.0)
+#endif
 
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatPInf"),
-		     floate_new (FLT_MAX * 2));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatNInf"),
-		     floate_new (FLT_MAX * -2));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatNaN"),
-		     floate_new (0.0 / 0.0));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatMin"),
-		     floate_new (FLT_MIN));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatMax"),
-		     floate_new (FLT_MAX));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatDigits"),
-		     FROM_INT (FLT_DIG));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatBinaryDigits"),
-		     FROM_INT (FLT_MANT_DIG));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatMinExp"),
-		     FROM_INT (FLT_MIN_EXP));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatMaxExp"),
-		     FROM_INT (FLT_MAX_EXP));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatAlignment"),
-		     FROM_INT (sizeof (float)));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMin"),
+		    floatd_new (DBL_MIN));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMax"),
+		    floatd_new (DBL_MAX));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoublePInf"),
+		    floatd_new ((double) INFINITY));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleNInf"),
+		    floatd_new ((double) -INFINITY));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleNaN"),
+		    floatd_new ((double) NAN));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleDigits"),
+		    FROM_INT (DBL_DIG));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleBinaryDigits"),
+		    FROM_INT (DBL_MANT_DIG));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMinExp"),
+		    FROM_INT (DBL_MIN_EXP));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMaxExp"),
+		    FROM_INT (DBL_MAX_EXP));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleAlignment"),
+		    FROM_INT (DOUBLE_ALIGNMENT));
 
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoublePInf"),
-		     floatq_new (LDBL_MAX * 2));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleNInf"),
-		     floatq_new (LDBL_MAX * -2));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleNaN"),
-		     floatq_new (0.0 / 0.0));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleMin"),
-		     floatq_new (LDBL_MIN));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleMax"),
-		     floatq_new (LDBL_MAX));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleDigits"),
-		     FROM_INT (LDBL_DIG));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleBinaryDigits"),
-		     FROM_INT (LDBL_MANT_DIG));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleMinExp"),
-		     FROM_INT (LDBL_MIN_EXP));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleMaxExp"),
-		     FROM_INT (LDBL_MAX_EXP));
-  DICTIONARY_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleAlignment"),
-		     FROM_INT (LONG_DOUBLE_ALIGNMENT));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatPInf"),
+		    floate_new ((float) INFINITY));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatNInf"),
+		    floate_new ((float) -INFINITY));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatNaN"),
+		    floate_new ((float) NAN));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatMin"),
+		    floate_new (FLT_MIN));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatMax"),
+		    floate_new (FLT_MAX));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatDigits"),
+		    FROM_INT (FLT_DIG));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatBinaryDigits"),
+		    FROM_INT (FLT_MANT_DIG));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatMinExp"),
+		    FROM_INT (FLT_MIN_EXP));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatMaxExp"),
+		    FROM_INT (FLT_MAX_EXP));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CFloatAlignment"),
+		    FROM_INT (sizeof (float)));
+
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoublePInf"),
+		    floatq_new ((long double) INFINITY));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleNInf"),
+		    floatq_new ((long double) -INFINITY));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleNaN"),
+		    floatq_new ((long double) NAN));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleMin"),
+		    floatq_new (LDBL_MIN));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleMax"),
+		    floatq_new (LDBL_MAX));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleDigits"),
+		    FROM_INT (LDBL_DIG));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleBinaryDigits"),
+		    FROM_INT (LDBL_MANT_DIG));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleMinExp"),
+		    FROM_INT (LDBL_MIN_EXP));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleMaxExp"),
+		    FROM_INT (LDBL_MAX_EXP));
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CLongDoubleAlignment"),
+		    FROM_INT (LONG_DOUBLE_ALIGNMENT));
 }
 
 void
@@ -1078,14 +1142,13 @@ _gst_init_dictionary_on_image_load (long numOOPs)
 
   init_runtime_objects ();
 
-  for (oop = _gst_all_oops_table;
-       oop < &_gst_oop_table[numOOPs]; oop++)
+  for (oop = _gst_mem.ot_base; oop < &_gst_mem.ot[numOOPs]; oop++)
     {
-      if (!OOP_VALID (oop))
+      if (!IS_OOP_VALID (oop))
 	continue;
 
       if UNCOMMON (OOP_CLASS (oop) == _gst_c_func_descriptor_class)
-	_gst_restore_cfunc_descriptor (oop);	/* in cint.c */
+        _gst_restore_cfunc_descriptor (oop); /* in cint.c */
       else if UNCOMMON (OOP_CLASS (oop) == _gst_compiled_method_class)
         _gst_restore_primitive_number (oop, primitive_numbers_map);
     }
@@ -1098,7 +1161,7 @@ prepare_primitive_numbers_map (int *map)
 {
   int i;
 
-  memzero (map, NUM_PRIMITIVES * sizeof (map));
+  memzero (map, NUM_PRIMITIVES * sizeof (int));
   for (i = 0; i < NUM_PRIMITIVES; i++)
     {
       prim_table_entry *pte = _gst_get_primitive_attributes (i);
@@ -1169,21 +1232,20 @@ _gst_resolve_primitive_name (char *name)
   primitivesDictionaryOOP = dictionary_at (_gst_smalltalk_dictionary, 
 					   _gst_vm_primitives_symbol);
 
-  index =
-    identity_dictionary_find_key_or_nil (primitivesDictionaryOOP,
-					 symbolOOP);
+  index = identity_dictionary_find_key (primitivesDictionaryOOP,
+					symbolOOP);
+
+  if (index < 0)
+    {
+      _gst_errorf ("bad primitive name");
+      return index;
+    }
 
   primitivesDictionary =
     (gst_identity_dictionary) OOP_TO_OBJ (primitivesDictionaryOOP);
 
   valueOOP = ARRAY_AT (primitivesDictionary->values, index + 1);
-  if (IS_NIL (valueOOP))
-    {
-      _gst_errorf ("bad primitive name");
-      return -1;
-    }
-  else
-    return TO_INT (valueOOP);
+  return TO_INT (valueOOP);
 }
 
 OOP
@@ -1201,12 +1263,20 @@ _gst_find_class_method (OOP class_oop,
     return (_gst_nil_oop);
 
   index =
-    identity_dictionary_find_key_or_nil (method_dictionary_oop,
-					 selector);
+    identity_dictionary_find_key (method_dictionary_oop,
+				  selector);
+
+  if (index < 0)
+    return (_gst_nil_oop);
 
   methodDictionary =
     (gst_identity_dictionary) OOP_TO_OBJ (method_dictionary_oop);
 
+#ifndef OPTIMIZE
+  if (!IS_OOP_ADDR (ARRAY_AT (methodDictionary->values, index + 1))
+    abort ();
+#endif
+
   return (ARRAY_AT (methodDictionary->values, index + 1));
 }
 
@@ -1246,18 +1316,23 @@ _gst_find_shared_pool_variable (OOP poolOOP,
 				OOP symbol)
 {
   OOP assocOOP;
+  gst_namespace pool;
+
+  if (is_a_kind_of (OOP_CLASS (poolOOP), _gst_class_class))
+    poolOOP = _gst_class_variable_dictionary (poolOOP);
+
+  if (IS_NIL (poolOOP))
+    return _gst_nil_oop;
 
   assocOOP = dictionary_association_at (poolOOP, symbol);
   while (IS_NIL (assocOOP))
     {
       /* Try to find a super-namespace */
-      assocOOP =
-	dictionary_association_at (poolOOP,
-				   _gst_uppercase_super_symbol);
-      if (IS_NIL (assocOOP))
+      if (!is_a_kind_of (OOP_CLASS (poolOOP), _gst_abstract_namespace_class))
 	return (_gst_nil_oop);
 
-      poolOOP = ASSOCIATION_VALUE (assocOOP);
+      pool = (gst_namespace) OOP_TO_OBJ (poolOOP);
+      poolOOP = pool->superspace;
       if (IS_NIL (poolOOP))
 	return (_gst_nil_oop);
 
@@ -1291,76 +1366,77 @@ static int
 find_key_or_nil (OOP dictionaryOOP,
 		 OOP keyOOP)
 {
-  long count, numFields;
+  long count, numFields, numFixedFields;
   long index;
-  gst_dictionary dictionary;
+  mst_Object dictionary;
   OOP associationOOP;
   gst_association association;
 
-  dictionary = (gst_dictionary) OOP_TO_OBJ (dictionaryOOP);
-  numFields = NUM_WORDS (dictionary) - 1;
+  dictionary = (mst_Object) OOP_TO_OBJ (dictionaryOOP);
+  numFixedFields = OOP_FIXED_FIELDS (dictionaryOOP);
+  numFields = NUM_WORDS (dictionary) - numFixedFields;
   index = scramble (OOP_INDEX (keyOOP));
-  index &= numFields - 1;
   count = numFields;
 
   for (; count; count--)
     {
-      if COMMON (IS_NIL (dictionary->assoc[index]))
+      index &= numFields - 1;
+      associationOOP = dictionary->data[numFixedFields + index];
+      if COMMON (IS_NIL (associationOOP))
 	return (index);
 
-      associationOOP = dictionary->assoc[index];
       association = (gst_association) OOP_TO_OBJ (associationOOP);
 
       if (association->key == keyOOP)
 	return (index);
 
       /* linear reprobe -- it is simple and guaranteed */
-      if (++index == numFields)
-	index = 0;
+      index++;
     }
 
   _gst_errorf
     ("Error - searching dictionary for nil, but it is full!\n");
-  exit (1);
+
+  abort ();
 }
 
-
-gst_dictionary
+mst_Object
 _gst_grow_dictionary (OOP oldDictionaryOOP)
 {
-  gst_dictionary oldDictionary, dictionary;
-  long oldNumFields, numFields, i, index;
+  mst_Object oldDictionary, dictionary;
+  long oldNumFields, numFields, i, index, numFixedFields;
   OOP associationOOP;
   gst_association association;
   OOP dictionaryOOP;
 
-  oldDictionary = (gst_dictionary) OOP_TO_OBJ (oldDictionaryOOP);
-  oldNumFields = NUM_WORDS (oldDictionary) - 1;
+  oldDictionary = OOP_TO_OBJ (oldDictionaryOOP);
+  numFixedFields = OOP_FIXED_FIELDS (oldDictionaryOOP);
+  oldNumFields = NUM_WORDS (oldDictionary) - numFixedFields;
 
   numFields = new_num_fields (oldNumFields);
 
   /* no need to use the incubator here.  We are instantiating just one
      object, the new dictionary itself */
 
-  dictionary = (gst_dictionary) instantiate_with (OOP_CLASS (oldDictionaryOOP), 
+  dictionary = instantiate_with (OOP_CLASS (oldDictionaryOOP), 
 						  numFields, &dictionaryOOP);
-  dictionary->tally = oldDictionary->tally;
-  oldDictionary = (gst_dictionary) OOP_TO_OBJ (dictionaryOOP);
+  memcpy (dictionary->data, oldDictionary->data, SIZEOF_LONG * numFixedFields);
+  oldDictionary = OOP_TO_OBJ (dictionaryOOP);
 
   /* rehash all associations from old dictionary into new one */
   for (i = 0; i < oldNumFields; i++)
     {
-      if COMMON (!IS_NIL (oldDictionary->assoc[i]))
+      associationOOP = oldDictionary->data[numFixedFields + i];
+      if COMMON (!IS_NIL (associationOOP))
 	{
-	  associationOOP = oldDictionary->assoc[i];
 	  association = (gst_association) OOP_TO_OBJ (associationOOP);
 	  index = find_key_or_nil (dictionaryOOP, association->key);
-	  dictionary->assoc[index] = associationOOP;
+	  dictionary->data[numFixedFields + index] = associationOOP;
 	}
     }
 
   _gst_swap_objects (dictionaryOOP, oldDictionaryOOP);
-  return (dictionary);
+  return (OOP_TO_OBJ (oldDictionaryOOP));
 }
 
 gst_identity_dictionary
@@ -1415,7 +1491,7 @@ _gst_grow_identity_dictionary (OOP oldIdentityDictionaryOOP)
 
   INC_RESTORE_POINTER (incPtr);
   _gst_swap_objects (identityDictionaryOOP, oldIdentityDictionaryOOP);
-  return (identityDictionary);
+  return ((gst_identity_dictionary) OOP_TO_OBJ (oldIdentityDictionaryOOP));
 }
 
 
@@ -1479,6 +1555,26 @@ _gst_identity_dictionary_at_put (OOP identityDictionaryOOP,
   return (oldValueOOP);
 }
 
+OOP
+namespace_new (int size, char *name, OOP superspaceOOP)
+{
+  gst_namespace ns;
+  OOP namespaceOOP, classOOP;
+
+  size = new_num_fields (size);
+  classOOP = IS_NIL (superspaceOOP)
+    ? _gst_root_namespace_class : _gst_namespace_class;
+
+  ns = (gst_namespace) instantiate_with (classOOP, size, &namespaceOOP);
+
+  ns->tally = FROM_INT (0);
+  ns->superspace = superspaceOOP;
+  ns->subspaces = _gst_nil_oop;
+  ns->name = _gst_intern_string (name);
+
+  return (namespaceOOP);
+}
+
 OOP
 _gst_dictionary_new (int size)
 {
@@ -1494,13 +1590,30 @@ _gst_dictionary_new (int size)
   return (dictionaryOOP);
 }
 
+OOP
+_gst_binding_dictionary_new (int size, OOP environmentOOP)
+{
+  gst_binding_dictionary dictionary;
+  OOP dictionaryOOP;
+
+  size = new_num_fields (size);
+  dictionary = (gst_binding_dictionary)
+    instantiate_with (_gst_binding_dictionary_class, size, &dictionaryOOP);
+
+  dictionary->tally = FROM_INT (0);
+  dictionary->environment = environmentOOP;
+
+  return (dictionaryOOP);
+}
+
 OOP
 _gst_dictionary_add (OOP dictionaryOOP,
 		     OOP associationOOP)
 {
   long index;
   gst_association association;
-  gst_dictionary dictionary;
+  mst_Object dictionary;
+  gst_dictionary dict;
   OOP value;
   inc_ptr incPtr;		/* I'm not sure clients are protecting
 				   association OOP */
@@ -1509,21 +1622,26 @@ _gst_dictionary_add (OOP dictionaryOOP,
   INC_ADD_OOP (associationOOP);
 
   association = (gst_association) OOP_TO_OBJ (associationOOP);
-  dictionary = (gst_dictionary) OOP_TO_OBJ (dictionaryOOP);
-  if UNCOMMON (TO_INT (dictionary->tally) >= 
-	       TO_INT (dictionary->objSize) * 3 / 4)
-    dictionary = _gst_grow_dictionary (dictionaryOOP);
+  dictionary = OOP_TO_OBJ (dictionaryOOP);
+  dict = (gst_dictionary) dictionary;
+  if UNCOMMON (TO_INT (dict->tally) >= 
+	       TO_INT (dict->objSize) * 3 / 4)
+    {
+      dictionary = _gst_grow_dictionary (dictionaryOOP);
+      dict = (gst_dictionary) dictionary;
+    }
 
   index = find_key_or_nil (dictionaryOOP, association->key);
-  if COMMON (IS_NIL (dictionary->assoc[index]))
+  index += OOP_FIXED_FIELDS (dictionaryOOP);
+  if COMMON (IS_NIL (dictionary->data[index]))
     {
-      dictionary->tally = INCR_INT (dictionary->tally);
-      dictionary->assoc[index] = associationOOP;
+      dict->tally = INCR_INT (dict->tally);
+      dictionary->data[index] = associationOOP;
     }
   else
     {
       value = ASSOCIATION_VALUE (associationOOP);
-      associationOOP = dictionary->assoc[index];
+      associationOOP = dictionary->data[index];
       SET_ASSOCIATION_VALUE (associationOOP, value);
     }
 
@@ -1579,7 +1697,7 @@ _gst_string_new (const char *s)
 }
 
 OOP
-_gst_counted_string_new (char *s,
+_gst_counted_string_new (const char *s,
 			 int len)
 {
   gst_string string;
@@ -1594,7 +1712,7 @@ _gst_counted_string_new (char *s,
 
 void
 _gst_set_oopstring (OOP stringOOP,
-		    char *s)
+		    const char *s)
 {
   OOP newStringOOP;
 
@@ -1619,7 +1737,7 @@ _gst_to_cstring (OOP stringOOP)
 }
 
 OOP
-_gst_byte_array_new (gst_uchar * bytes,
+_gst_byte_array_new (const gst_uchar * bytes,
 		     int len)
 {
   gst_byte_array byteArray;
@@ -1679,6 +1797,24 @@ _gst_message_new_args (OOP selectorOOP,
   return (messageOOP);
 }
 
+OOP
+_gst_directed_message_new_args (OOP receiverOOP,
+				OOP selectorOOP,
+			        OOP argsArray)
+{
+  gst_directed_message dirMessage;
+  OOP dirMessageOOP;
+
+  dirMessage = (gst_directed_message) new_instance (_gst_directed_message_class,
+						    &dirMessageOOP);
+
+  dirMessage->selector = selectorOOP;
+  dirMessage->args = argsArray;
+  dirMessage->receiver = receiverOOP;
+
+  return (dirMessageOOP);
+}
+
 OOP
 _gst_c_object_new_typed (PTR cObjPtr,
 			 OOP typeOOP)
