@@ -63,20 +63,43 @@ typedef struct gst_dictionary
 {
   OBJ_HEADER;
   OOP tally;			/* really, an int */
-  OOP assoc[1];			/* variable sized array of associations 
-				 */
+
   /* Other, indexable fields that are the associations for this
-     dictionary */
+     dictionary.  */
 }
  *gst_dictionary;
 
+typedef struct gst_binding_dictionary
+{
+  OBJ_HEADER;
+  OOP tally;			/* really, an int */
+  OOP environment;
+  OOP assoc[1];
+
+  /* Other, indexable fields that are the associations for this
+     dictionary.  */
+}
+ *gst_binding_dictionary;
+
+typedef struct gst_namespace
+{
+  OBJ_HEADER;
+  OOP tally;			/* really, an int */
+  OOP superspace;
+  OOP name;
+  OOP subspaces;
+  OOP assoc[1];
+
+  /* Other, indexable fields that are the associations for this
+     dictionary.  */
+}
+ *gst_namespace;
+
 typedef struct gst_identity_dictionary
 {
   OBJ_HEADER;
   OOP tally;			/* really, an int */
-  OOP values;			/* an gst_array */
-  OOP keys[1];			/* variable sized array of OOPS
-				   (symbols) */
+  OOP keys[1];			/* variable sized array of OOPS (keys/values) */
 }
  *gst_identity_dictionary;
 
@@ -86,7 +109,7 @@ typedef struct gst_identity_dictionary
   OOP		superclass; \
   OOP		subClasses; \
   OOP		methodDictionary; \
-  long		instanceSpec; \
+  intptr_t	instanceSpec; \
   OOP		instanceVariables
 
 typedef struct gst_behavior
@@ -126,6 +149,15 @@ typedef struct gst_association
 }
  *gst_association;
 
+typedef struct gst_variable_binding
+{
+  OBJ_HEADER;
+  OOP key;
+  OOP value;
+  OOP environment;
+}
+ *gst_variable_binding;
+
 typedef struct gst_floatd
 {
   OBJ_HEADER;
@@ -155,6 +187,15 @@ typedef struct gst_message
 }
  *gst_message;
 
+typedef struct gst_directed_message
+{
+  OBJ_HEADER;
+  OOP selector;
+  OOP args;
+  OOP receiver;
+}
+ *gst_directed_message;
+
 typedef struct gst_string
 {
   OBJ_HEADER;
@@ -213,7 +254,9 @@ extern OOP _gst_object_class, _gst_magnitude_class, _gst_char_class,
   _gst_object_memory_class, _gst_large_integer_class,
   _gst_large_negative_integer_class, _gst_large_zero_integer_class,
   _gst_large_positive_integer_class, _gst_association_class,
+  _gst_homed_association_class, _gst_variable_binding_class,
   _gst_link_class, _gst_process_class, _gst_sym_link_class,
+  _gst_callin_process_class,
   _gst_collection_class, _gst_lookup_key_class,
   _gst_sequenceable_collection_class, _gst_linked_list_class,
   _gst_semaphore_class, _gst_method_dictionary_class,
@@ -223,6 +266,7 @@ extern OOP _gst_object_class, _gst_magnitude_class, _gst_char_class,
   _gst_interval_class, _gst_ordered_collection_class,
   _gst_sorted_collection_class, _gst_bag_class,
   _gst_mapped_collection_class, _gst_set_class, _gst_dictionary_class,
+  _gst_binding_dictionary_class, _gst_abstract_namespace_class,
   _gst_character_array_class, _gst_system_dictionary_class,
   _gst_namespace_class, _gst_identity_dictionary_class,
   _gst_undefined_object_class, _gst_boolean_class, _gst_false_class,
@@ -232,6 +276,7 @@ extern OOP _gst_object_class, _gst_magnitude_class, _gst_char_class,
   _gst_root_namespace_class, _gst_byte_stream_class,
   _gst_class_description_class, _gst_class_class, _gst_metaclass_class,
   _gst_smalltalk_dictionary, _gst_message_class,
+  _gst_directed_message_class,
   _gst_method_context_class, _gst_block_context_class,
   _gst_block_closure_class, _gst_stream_class,
   _gst_positionable_stream_class, _gst_read_stream_class,
@@ -243,91 +288,103 @@ extern OOP _gst_object_class, _gst_magnitude_class, _gst_char_class,
   _gst_file_segment_class, _gst_c_object_type_ctype, _gst_processor_oop;
 
 /* Creates a new instance of the Dictionary class with room for SIZE
-   items. */
+   items.  */
 extern OOP _gst_dictionary_new (int size);
 
+/* Creates a new instance of the BindingDictionary class with room for SIZE
+   items.  The object has the ENVIRONMENTOOP environment (which for
+   dictionary of class variables is the class that hosts the dictionary).  */
+extern OOP _gst_binding_dictionary_new (int size,
+			        	OOP environmentOOP);
+
 /* Gets the method dictionary associated with CLASS_OOP, and returns
    it.  If the methodDictionary associated with CLASS_OOP is nil, one
-   is created and installed into that class. */
-extern OOP _gst_valid_class_method_dictionary (OOP class_oop) FN_PURE;
+   is created and installed into that class.  */
+extern OOP _gst_valid_class_method_dictionary (OOP class_oop) ATTRIBUTE_PURE;
 
 /* This returns the dictionary of class variables for CLASS_OOP */
-extern OOP _gst_class_variable_dictionary (OOP class_oop) FN_PURE;
+extern OOP _gst_class_variable_dictionary (OOP class_oop) ATTRIBUTE_PURE;
 
 /* This finds the key SYMBOL into the dictionary POOLOOP and, if any,
-   in all of its super-namespaces. */
+   in all of its super-namespaces.  */
 extern OOP _gst_find_shared_pool_variable (OOP poolOOP,
-					   OOP symbol) FN_PURE;
+					   OOP symbol) ATTRIBUTE_PURE;
 
 /* Adds the Association in ASSOCIATIONOOP to the Dictionary (or a
-   subclass sharing the same representation) DICTIONARYOOP. */
+   subclass sharing the same representation) DICTIONARYOOP.  */
 extern OOP _gst_dictionary_add (OOP dictionaryOOP,
 				OOP associationOOP);
 
 /* Creates a String object starting from the NUL-terminated string
-   S. */
+   S.  */
 extern OOP _gst_string_new (const char *s);
 
 /* Look in the Smalltalk dictionary for a class whose name is in the
-   Symbol CLASSNAMEOOP. */
-extern OOP _gst_find_class (OOP classNameOOP) FN_PURE;
+   Symbol CLASSNAMEOOP.  */
+extern OOP _gst_find_class (OOP classNameOOP) ATTRIBUTE_PURE;
 
 /* Look for an implementation of SELECTOR (a Symbol) into CLASS_OOP's
-   method dictionary or in the method dictionary of a superclass. */
+   method dictionary or in the method dictionary of a superclass.  */
 extern OOP _gst_find_class_method (OOP class_oop,
-				   OOP selector) FN_PURE;
+				   OOP selector) ATTRIBUTE_PURE;
 
 /* Create a new Message object for the given SELECTOR (a Symbol) and
-   Array of arguments. */
+   Array of arguments.  */
 extern OOP _gst_message_new_args (OOP selectorOOP,
 				  OOP argsArray);
 
+/* Create a new DirectedMessage object for the given RECEIVER,
+   SELECTOR (a Symbol) and Array of arguments.  */
+extern OOP _gst_directed_message_new_args (OOP receiverOOP,
+					   OOP selectorOOP,
+					   OOP argsArray);
+
 /* Returns the name of CLASS_OOP (a Smalltalk Class) */
-extern OOP _gst_get_class_symbol (OOP class_oop) FN_PURE;
+extern OOP _gst_get_class_symbol (OOP class_oop) ATTRIBUTE_PURE;
 
 /* Create and return an exact copy of OOP, which can be any kind
    of OOP.  Builtin OOPs and integers are returned unchanged,
    while for other objects this is a "shallow copy"; all the
    instance variables in the new object are the exact same ones
-   that are in the original object. */
+   that are in the original object.  */
 extern OOP _gst_object_copy (OOP oop);
 
 /* Returns the array of the names of the instance variables of
-   CLASS_OOP (a Smalltalk Class). */
-extern OOP _gst_instance_variable_array (OOP class_oop) FN_PURE;
+   CLASS_OOP (a Smalltalk Class).  */
+extern OOP _gst_instance_variable_array (OOP class_oop) ATTRIBUTE_PURE;
 
 /* Returns the array of the names of the pool dictionaries of
-   CLASS_OOP (a Smalltalk Class). */
-extern OOP _gst_shared_pool_dictionary (OOP class_oop) FN_PURE;
+   CLASS_OOP (a Smalltalk Class).  */
+extern OOP _gst_shared_pool_dictionary (OOP class_oop) ATTRIBUTE_PURE;
 
 /* Creates a new CObject pointing to cObjPtr, extracting the name of
-   the class to be instantiated from the CType, TYPEOOP. */
+   the class to be instantiated from the CType, TYPEOOP.  */
 extern OOP _gst_c_object_new_typed (PTR cObjPtr,
 				    OOP typeOOP);
 
 /* Allocates a new CObject by malloc-ing SIZE bytes; CLASS_OOP is the
-   class to be instantiated. */
+   class to be instantiated.  */
 extern OOP _gst_alloc_cobject (OOP class_oop,
-			       long unsigned int size);
+			       size_t size);
 
 /* Creates a new CType that when passed to _gst_c_object_new_typed
-   creates an instance of COBJECTSUBCLASSOOP. */
+   creates an instance of COBJECTSUBCLASSOOP.  */
 extern OOP _gst_c_type_new (OOP cObjectSubclassOOP);
 
-/* Creates a new String with LEN indexed instance variables. */
-extern OOP _gst_new_string (int len);
+/* Creates a new String with LEN indexed instance variables.  */
+extern OOP _gst_new_string (size_t len);
 
-/* Creates a new ByteArray containing LEN bytes starting at BYTES. */
-extern OOP _gst_byte_array_new (gst_uchar * bytes,
-				int len);
+/* Creates a new ByteArray containing LEN bytes starting at BYTES.  */
+extern OOP _gst_byte_array_new (const gst_uchar * bytes,
+				size_t len);
 
-/* Creates a new String containing LEN characters starting at S. */
-extern OOP _gst_counted_string_new (char *s,
-				    int len);
+/* Creates a new String containing LEN characters starting at S.  */
+extern OOP _gst_counted_string_new (const char *s,
+				    size_t len);
 
 /* Adds the key KEYOOP, associated with VALUEOOP, to the
    IdentityDictionary (or a subclass sharing the same representation)
-   IDENTITYDICTIONARYOOP. */
+   IDENTITYDICTIONARYOOP.  */
 extern OOP _gst_identity_dictionary_at_put (OOP identityDictionaryOOP,
 					    OOP keyOOP,
 					    OOP valueOOP);
@@ -335,39 +392,39 @@ extern OOP _gst_identity_dictionary_at_put (OOP identityDictionaryOOP,
 /* Called when a dictionary becomes full, this routine replaces the
    dictionary instance that DICTIONARYOOP is pointing to with a new,
    larger dictionary, and returns this new dictionary (the object
-   pointer, not the OOP). */
-extern gst_dictionary _gst_grow_dictionary (OOP dictionaryOOP);
+   pointer, not the OOP).  */
+extern mst_Object _gst_grow_dictionary (OOP dictionaryOOP);
 
 /* Called when an IdentityDictionary becomes full, this routine
    replaces the IdentityDictionary instance that IDENTITYDICTIONARYOOP
    is pointing to with a new, larger dictionary, and returns this new
-   dictionary (the object pointer, not the OOP). */
+   dictionary (the object pointer, not the OOP).  */
 extern gst_identity_dictionary _gst_grow_identity_dictionary (OOP identityDictionaryOOP);
 
 /* Allocates and returns a new C (ASCIZ) string that has the same
    contents as STRINGOOP.  Even if there are embedded NULs, the
-   allocated area has always a size of "stringOOP size + 1" bytes. */
+   allocated area has always a size of "stringOOP size + 1" bytes.  */
 extern char *_gst_to_cstring (OOP stringOOP);
 
 /* Allocates and returns a new memory block that has the same contents
    as BYTEARRAYOOP.  Even if there are embedded NULs, the allocated
-   area has always a size of "byteArrayOOP size" bytes. */
+   area has always a size of "byteArrayOOP size" bytes.  */
 extern gst_uchar *_gst_to_byte_array (OOP byteArrayOOP);
 
 /* Creates the kernel classes of the Smalltalk system.  Operates in two
    passes: pass1 creates the class objects, but they're not completely
    initialized.  pass2 finishes the initialization process.  The garbage
-   collector can NOT run during this time. */
+   collector can NOT run during this time.  */
 extern void _gst_init_dictionary (void);
 
 /* Copies the first bytes of S into STRINGOOP (as many bytes as the
-   OOP can hold). */
+   OOP can hold).  */
 extern void _gst_set_oopstring (OOP stringOOP,
-				char *s);
+				const char *s);
 
 /* Set the instance variables of the FileStream object,
    FILESTREAMOOP.  If BUFFERED is true, the variables
-   inherited by Streams are set to a 1024-bytes String. */
+   inherited by Streams are set to a 1024-bytes String.  */
 extern void _gst_set_file_stream_file (OOP fileStreamOOP,
 				       int fd,
 				       OOP fileNameOOP,
@@ -376,20 +433,20 @@ extern void _gst_set_file_stream_file (OOP fileStreamOOP,
 				       mst_Boolean buffered);
 
 /* Copies the first bytes of BYTES into BYTEARRAYOOP (as many bytes as
-   the OOP can hold). */
+   the OOP can hold).  */
 extern void _gst_set_oop_bytes (OOP byteArrayOOP,
 				gst_uchar * bytes);
 
-/* Frees the memory pointed to by the CObject, COBJOOP. */
+/* Frees the memory pointed to by the CObject, COBJOOP.  */
 extern void _gst_free_cobject (OOP cObjOOP);
 
 /* Loads the contents of the global variable from the Smalltalk
    dictionary after NUMOOPS objects have been loaded from an
-   image. */
-extern mst_Boolean _gst_init_dictionary_on_image_load (long numOOPs);
+   image.  */
+extern mst_Boolean _gst_init_dictionary_on_image_load (size_t numOOPs);
 
 /* Transforms a primitive name into a primitive index, looking up
-   the VMPrimitives dictionary. */
+   the VMPrimitives dictionary.  */
 extern int _gst_resolve_primitive_name (char *name);
 
 #endif /* GST_DICT_H */
