@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 2001, 2002 Free Software Foundation, Inc.
+ * Copyright 2001, 2002, 2003 Free Software Foundation, Inc.
  * Written by Paolo Bonzini.
  *
  * This file is part of GNU Smalltalk.
@@ -33,10 +33,10 @@
 
 #include <poll.h>
 #include <errno.h>
-#include <signal.h>
+#include "signalx.h"
 
 #ifdef HAVE_UNISTD_H
-# include <unistd.h>
+#include <unistd.h>
 #endif
 
 #if defined(HAVE_SYS_TIME_H) || defined(TIME_WITH_SYS_TIME)
@@ -63,7 +63,7 @@ polling_queue;
    array of pollfd structures must be scanned and kept sequential
    every time that I/O happens, so it does not bother us very much to
    have to scan the list to find the semaphore that is to be signaled.  */
-static polling_queue *head, *tail;
+static polling_queue *head, **p_tail_next;
 
 /* This variable holds a variable-sized array of pollfd structures.
    NUM_USED_POLLFDS of the total NUM_TOTAL_POLLFDS items available are
@@ -71,16 +71,9 @@ static polling_queue *head, *tail;
 static struct pollfd *pollfds;
 static int num_used_pollfds, num_total_pollfds;
 
-/* Max number of C-style signals on a machine */
-#ifdef NSIG
-#define NUM_SIGNALS	NSIG
-#else
-#define NUM_SIGNALS	64	/* should be enough */
-#endif
-
 /* Holds the semaphores to be signaled when the operating system sends
    us a C-style signal. */
-static volatile OOP sem_int_vec[NUM_SIGNALS];
+static volatile OOP sem_int_vec[NSIG];
 
 /* These are the signal handlers that we install to process
    asynchronous events and pass them to the Smalltalk virtual machine.
@@ -97,10 +90,11 @@ _gst_init_async_events (void)
 {
   int i;
 
-  for (i = 0; i < NUM_SIGNALS; i++)
+  for (i = 0; i < NSIG; i++)
     sem_int_vec[i] = _gst_nil_oop;
 
-  head = tail = NULL;
+  head = NULL;
+  p_tail_next = NULL;
 }
 
 RETSIGTYPE
@@ -119,7 +113,7 @@ signal_handler (int sig)
 	{
 	  _gst_errorf
 	    ("C signal trapped, but no semaphore was waiting");
-	  kill (getpid (), sig);
+	  raise (sig);
 	}
     }
 }
@@ -227,8 +221,8 @@ file_polling_handler (int sig)
 
 	      /* Pass over the current node */
 	      *pprev = node->next;
-	      if (tail == node)
-		tail = NULL;
+	      if (p_tail_next == &node->next)
+		p_tail_next = pprev;
 
 	      xfree (node);
 	      n--;
@@ -251,7 +245,6 @@ _gst_async_file_polling (int fd,
 			 int cond,
 			 OOP semaphoreOOP)
 {
-  int_state intState;
   int result;
   polling_queue *new;
 
@@ -265,7 +258,7 @@ _gst_async_file_polling (int fd,
   new->next = NULL;
 
   /* Disable interrupts while playing with global variables */
-  intState = _gst_disable_interrupts ();
+  _gst_disable_interrupts ();
 
   if (num_used_pollfds == num_total_pollfds)
     {
@@ -301,9 +294,10 @@ _gst_async_file_polling (int fd,
   if (result == 0)
     {
       if (!head)
-	tail = head = new;
+	head = new;
       else
-	tail = tail->next = new;
+	*p_tail_next = new;
+      p_tail_next = &new->next;
 
       num_used_pollfds++;
       _gst_register_oop (semaphoreOOP);
@@ -312,6 +306,6 @@ _gst_async_file_polling (int fd,
   else
     xfree (new);
 
-  _gst_enable_interrupts (intState);
+  _gst_enable_interrupts ();
   return (result);
 }
