@@ -33,15 +33,17 @@
 #define GST_GSTPRIV_H
 
 #include "config.h"
+#include "gst.h"
 
-#if SIZEOF_CHAR_P != SIZEOF_LONG
-#error longs and pointer have different sizes
-#endif
-
-#if SIZEOF_INT < 4
-#error 16-bits platform are not supported.
-#endif
 
+/* ENABLE_SECURITY enables security checks in the primitives as well as
+   special marking of untrusted objects.  Note that the code in the
+   class library to perform the security checks will be present
+   notwithstanding the setting of this flag, but they will be disabled
+   because the corresponding primitives will be made non-working.
+   It is undefined because the Makefiles take care of defining it for
+   security-enabled builds.  */
+#define ENABLE_SECURITY
 
 /* OPTIMIZE disables many checks, including consistency checks at GC
    time and bounds checking on instance variable accesses (not on #at:
@@ -50,8 +52,7 @@
    debugging painful because you know of a bug only when it's too
    late.  It is undefined because the Makefiles take care of defining
    it for optimized builds.  Bounds-checking and other errors will
-   call the _gst_debug() function, where you can set a breakpoint and
-   which acts a generic entry point for assertion failures. */
+   call abort().  */
 /* #define OPTIMIZE */
 
 typedef unsigned char gst_uchar;
@@ -66,7 +67,7 @@ typedef unsigned char gst_uchar;
 #  endif
 #endif
 
-/* If they have no const, they're likely to have no volatile, either. */
+/* If they have no const, they're likely to have no volatile, either.  */
 #ifdef const
 #define volatile
 #endif
@@ -76,29 +77,16 @@ extern char *strdup ();
 /* else it is in string.h */
 #endif
 
-/* Implementation note: on 64 bit architectures, bit-encoded 32 bit
-   words are made to live in the low order 32 bits -- the high 32 bits
-   are ignored, and are padded if necessary to achieve the useful
-   stuff in the lower 32.  This simplifies the current implementation,
-   as most bit fields, both in C and in Smalltalk do not have to be
-   modified when on a 64 bit architecture.  Smalltalk Integers will be
-   able to represent up to 62 bits + 1 sign bit. */
-#if SIZEOF_LONG == 8
-#define LONG_SHIFT 3		/* bits to shift for a long or pointer */
-#else
-#define LONG_SHIFT 2		/* bits to shift for a long or pointer */
-#endif
-
-/* Fallback to BSD functions if the standard ones are not available. */
-#ifndef HAVE_MEMCPY
-#define memcpy(s1, s2, n) bcopy((s2), (s1), (n))
-#define memcmp(s1, s2, n) bcmp ((s1), (s2), (n))
-#endif /* HAVE_MEMCPY */
-
 
 /* Run-time flags are allocated from the top, while those
    that live across image saves/restores are allocated
-   from the bottom. */
+   from the bottom.
+
+   bit 0-3: reserved for distinguishing byte objects and saving their size.
+   bit 4-11: non-volatile bits (special kinds of objects).
+   bit 12-23: volatile bits (GC/JIT-related).
+   bit 24-30: reserved for counting things.
+   bit 31: unused to avoid signedness mess. */
 enum {
   /* Place to save various OOP counts (how many fields we have marked
      in the object, how many pointer instance variables there are, 
@@ -113,92 +101,121 @@ enum {
   F_COUNT = (int) 0x7F000000U,
   F_COUNT_SHIFT = 24,
 
-  /* Set if the object is reachable, during the mark phases of garbage
-     collection. */
+  /* Set if the object is reachable, during the mark phases of oldspace
+     garbage collection.  */
   F_REACHABLE = 0x800000U,
 
   /* Set if a translation to native code is available, when running
-     with the JIT compiler enabled. */
-  F_XLAT = 0x200000U,
+     with the JIT compiler enabled.  */
+  F_XLAT = 0x400000U,
 
   /* Set if a translation to native code is used by the currently
-     reachable contexts. */
-  F_XLAT_REACHABLE = 0x100000U,
+     reachable contexts.  */
+  F_XLAT_REACHABLE = 0x200000U,
 
   /* Set if a translation to native code is available but not used by
      the reachable contexts at the time of the last GC.  We give
      another chance to the object, but if the translation is not used
-     for two consecutive GCs we discard it. */
-  F_XLAT_2NDCHANCE = 0x80000U,
+     for two consecutive GCs we discard it.  */
+  F_XLAT_2NDCHANCE = 0x100000U,
 
   /* Set if a translation to native code was discarded for this
      object (either because the programmer asked for this, or because
-     the method conflicted with a newly-installed method. */
-  F_XLAT_DISCARDED = 0x40000U,
+     the method conflicted with a newly-installed method).  */
+  F_XLAT_DISCARDED = 0x80000U,
 
-  /* Answer whether we want a finalizer method to be called when the
-     object is not reachable anymore. */
-  F_FINALIZE = 0x20000U,
-
-  /* The grouping of all the flags which are not valid across image
-     saves and loads. */
-  F_RUNTIME = 0xFE0000U,
-
-  /* Set if the OOP is currently unused. */
-  F_FREE = 0x100U,
+  /* One of this is set for objects that live in newspace.  */
+  F_SPACES = 0x60000U,
+  F_EVEN = 0x40000U,
+  F_ODD = 0x20000U,
 
   /* Set if the OOP is allocated by the pools of contexts maintained
      in interp.c (maybe it belongs above...) */
-  F_POOLED = 0x200U,
+  F_POOLED = 0x10000U,
+
+  /* Set if the bytecodes in the method have been verified. */
+  F_VERIFIED = 0x8000U,
+
+  /* The grouping of all the flags which are not valid across image
+     saves and loads.  */
+  F_RUNTIME = 0xFFF000U,
+
+  /* Set if the OOP is currently unused.  */
+  F_FREE = 0x10U,
 
   /* Set if the references to the instance variables of the object
-     are weak. */
-  F_WEAK = 0x400U,
+     are weak.  */
+  F_WEAK = 0x20U,
 
-  /* Set if the object is read-only. */
-  F_READONLY = 0x800U,
+  /* Set if the object is read-only.  */
+  F_READONLY = 0x40U,
 
   /* Set if the object is a context and hence its instance variables
-     are only valid up to the stack pointer. */
-  F_CONTEXT = 0x1000U,
+     are only valid up to the stack pointer.  */
+  F_CONTEXT = 0x80U,
 
-  /* Set if the object is allocated in FixedSpace (that is, its data
-     do not change address after a garbage collection -- of course,
-     they do after an image is saved and restored). */
-  F_FIXED = 0x2000U,
+  /* Answer whether we want to mark the key based on references found
+     outside the object.  */
+  F_EPHEMERON = 0x100U,
+
+  /* Set for objects that live in oldspace.  */
+  F_OLD = 0x200U,
+
+  /* Set together with F_OLD for objects that live in fixedspace.  */
+  F_FIXED = 0x400U,
+
+  /* Set for untrusted classes, instances of untrusted classes,
+     and contexts whose receiver is untrusted.  */
+  F_UNTRUSTED = 0x800U,
 
   /* Set to the number of bytes unused in an object with byte-sized
      instance variables.  Note that this field and the following one
      should be initialized only by INIT_BYTE_OBJECT (not really 
      aesthetic but...) */
-  EMPTY_BYTES = (SIZEOF_LONG - 1),
+  EMPTY_BYTES = (sizeof (PTR) - 1),
 
   /* A bit more than what is identified by EMPTY_BYTES.  Selects some
      bits that are never zero if and only if this OOP identifies an
      object with byte instance variables.  */
-  F_BYTE = EMPTY_BYTES | SIZEOF_LONG
+  F_BYTE = EMPTY_BYTES | sizeof (PTR)
 };
 
-/* Answer whether an object, OOP, is readonly. */
+/* Answer whether a method, OOP, has already been verified. */
+#define IS_OOP_VERIFIED(oop) \
+  ((oop)->flags & F_VERIFIED)
+
+/* Answer whether an object, OOP, is weak.  */
+#define IS_OOP_WEAK(oop) \
+  ((oop)->flags & F_WEAK)
+
+/* Answer whether an object, OOP, is readonly.  */
 #define IS_OOP_READONLY(oop) \
-  (((oop)->flags & F_READONLY) != 0)
+  (IS_INT ((oop)) ? F_READONLY : (oop)->flags & F_READONLY)
 
-/* Set whether an object, OOP, is readonly or readwrite. */
-#define MAKE_OOP_READONLY(oop, ro) do { \
-  if (ro) { (oop)->flags |= F_READONLY; } \
-  else { (oop)->flags &= ~F_READONLY; } \
-} while(0)
+/* Set whether an object, OOP, is readonly or readwrite.  */
+#define MAKE_OOP_READONLY(oop, ro) \
+  (((oop)->flags &= ~F_READONLY), \
+   ((oop)->flags |= (ro) ? F_READONLY : 0))
 
-/* Set whether an object, OOP, has a finalizer. */
-#define MARK_OOP_TO_FINALIZE(oop, fin) do { \
-  if (fin) { (oop)->flags |= F_FINALIZE; } \
-  else { (oop)->flags &= ~F_FINALIZE; } \
-} while(0)
+#ifdef ENABLE_SECURITY
 
-/* Consider the references to the instance variables of OOP to
-   be weak.  */
-#define MAKE_OOP_WEAK(oop) \
-  (oop)->flags |= F_WEAK;
+/* Answer whether an object, OOP, is untrusted.  */
+#define IS_OOP_UNTRUSTED(oop) \
+  (IS_INT ((oop)) ? 0 : ((oop)->flags & F_UNTRUSTED))
+
+/* Set whether an object, OOP, is trusted or untrusted.  */
+#define MAKE_OOP_UNTRUSTED(oop, untr) \
+  (((oop)->flags &= ~F_UNTRUSTED), \
+   ((oop)->flags |= (untr) ? F_UNTRUSTED : 0))
+
+#else
+#define IS_OOP_UNTRUSTED(oop) 0
+#define MAKE_OOP_UNTRUSTED(oop, untr) 0
+#endif
+
+/* Set whether an object, OOP, has ephemeron semantics.  */
+#define MAKE_OOP_EPHEMERON(oop) \
+  (oop)->flags |= F_EPHEMERON;
 
 
 /*
@@ -212,33 +229,35 @@ enum {
    allocating space for more flags in case they're needed.
    If you change ISP_NUMFIXEDFIELDS you should modify Behavior.st too.
    Remember to shift by ISP_NUMFIXEDFIELDS-1 there, since Smalltalk does
-   not see ISP_INTMARK!! */
+   not see ISP_INTMARK!!
+
+   FIXME: these should be exported in a pool dictionary.  */
 
 enum {
-  /* This is a shift count. */
+  /* This is a shift count.  */
   ISP_NUMFIXEDFIELDS = 13,
 
   /* These represent the shape of the indexed instance variables of
-     the instances of the class. */
+     the instances of the class.  */
   ISP_INDEXEDVARS = 28,         
 
-  /* Set if the indexed instance variables are pointers. */
+  /* Set if the indexed instance variables are pointers.  */
   ISP_ISPOINTERS = 16,
 
-  /* Set if the indexed instance variables are words (C longs). */
+  /* Set if the indexed instance variables are words (C longs).  */
   ISP_ISWORDS = 8,
 
   /* Set if the instances of the class have indexed instance
-     variables. */
+     variables.  */
   ISP_ISINDEXABLE = 4,
 
-  /* Set to 1 to mark a SmallInteger. */
+  /* Set to 1 to mark a SmallInteger.  */
   ISP_INTMARK = 1
 };
 
 
 /* the current execution stack pointer */
-#ifndef USE_JIT_TRANSLATION
+#ifndef ENABLE_JIT_TRANSLATION
 # define sp		_gst_sp
 #endif
 
@@ -250,28 +269,54 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 
 /* Some stack operations */
 #define UNCHECKED_PUSH_OOP(oop) \
-  *++sp = (oop)
+  (*++sp = (oop))
 
 #define UNCHECKED_SET_TOP(oop) \
-  *sp = (oop)
+  (*sp = (oop))
 
+#ifndef OPTIMIZE
 #define PUSH_OOP(oop) \
-  UNCHECKED_PUSH_OOP(oop)
+  do { \
+    OOP __pushOOP = (oop); \
+    if (IS_OOP (__pushOOP) && !IS_OOP_VALID (__pushOOP)) \
+      abort (); \
+    UNCHECKED_PUSH_OOP (__pushOOP); \
+  } while (0)
+#else
+#define PUSH_OOP(oop) \
+  do { \
+    OOP __pushOOP = (oop); \
+    UNCHECKED_PUSH_OOP (__pushOOP); \
+  } while (0)
+#endif
 
 #define POP_OOP() \
   (*sp--)
 
 #define POP_N_OOPS(n) \
-  sp -= (n)
+  (sp -= (n))
 
 #define UNPOP(n) \
-  sp += (n)
+  (sp += (n))
 
 #define STACKTOP() \
   (*sp)
 
+#ifndef OPTIMIZE
 #define SET_STACKTOP(oop) \
-  UNCHECKED_SET_TOP(oop)
+  do { \
+    OOP __pushOOP = (oop); \
+    if (IS_OOP (__pushOOP) && !IS_OOP_VALID (__pushOOP)) \
+      abort (); \
+    UNCHECKED_SET_TOP(__pushOOP); \
+  } while (0)
+#else
+#define SET_STACKTOP(oop) \
+  do { \
+    OOP __pushOOP = (oop); \
+    UNCHECKED_SET_TOP(__pushOOP); \
+  } while (0)
+#endif
 
 #define SET_STACKTOP_INT(i) \
   UNCHECKED_SET_TOP(FROM_INT(i))
@@ -289,122 +334,98 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
   TO_INT(POP_OOP())
 
 #define PUSH_BOOLEAN(exp) \
-  UNCHECKED_PUSH_OOP((exp) ? _gst_true_oop : _gst_false_oop)
-
-/* Answer whether OOP has been found during the marking phase of the
-   GC.  */
-#define IS_OOP_MARKED(oop) \
-  ((oop)->flags & F_REACHABLE)
-
-/* Answer whether OOP is allocated outside the main heap.  */
-#define IS_OOP_OUTSIDE_HEAP(oop) \
-  ((oop)->flags & (F_FIXED | F_POOLED))
-
-/* Answer whether OOP is free or not.  */
-#define IS_OOP_FREE(oop) \
-  ((oop)->flags & F_FREE)
+  PUSH_OOP((exp) ? _gst_true_oop : _gst_false_oop)
 
 
 /* Answer whether CLASS is the class that the object pointed to by OOP
-   belongs to.  OOP can also be a SmallInteger. */
+   belongs to.  OOP can also be a SmallInteger.  */
 #define IS_CLASS(oop, class) \
-  (IS_INT(oop) ? class == _gst_small_integer_class : class == OOP_CLASS(oop))
+  (OOP_INT_CLASS(oop) == class)
 
-/* Answer whether OOP is nil. */
+/* Answer the CLASS that the object pointed to by OOP belongs to.  OOP
+   can also be a SmallInteger. */
+#define OOP_INT_CLASS(oop) \
+  (IS_INT(oop) ? _gst_small_integer_class : OOP_CLASS(oop))
+
+
+/* Answer whether OOP is nil.  */
 #define IS_NIL(oop) \
   ((OOP)(oop) == _gst_nil_oop)
 
 
-/************ incubator support **************/
-
-extern OOP *_gst_inc_oop_base_ptr, *_gst_inc_oopptr,
-  *_gst_inc_oop_end_ptr;
-
-#define INC_ADD_OOP(oop) \
-  ((_gst_inc_oopptr >= _gst_inc_oop_end_ptr ? _gst_inc_grow_registry() : (void)0), \
-    *_gst_inc_oopptr++ = (oop))
-
-#define INC_SAVE_POINTER() \
-  (_gst_inc_oopptr - _gst_inc_oop_base_ptr)
-
-#define INC_RESTORE_POINTER(ptr) \
-  _gst_inc_oopptr = (ptr) + _gst_inc_oop_base_ptr;
-
-typedef unsigned long inc_ptr;
-
 /* This macro should only be used right after an alloc_oop, when the
    emptyBytes field is guaranteed to be zero.
 
    Note that F_BYTE is a bit more than EMPTY_BYTES, so that if value 
-   is a multiple of SIZEOF_LONG the flags identified by F_BYTE are
+   is a multiple of sizeof (PTR) the flags identified by F_BYTE are
    not zero.  */
 #define INIT_BYTE_OBJECT(oop, value) \
-    ((oop)->flags |= SIZEOF_LONG - ((value) & EMPTY_BYTES))
+    ((oop)->flags |= sizeof (PTR) - ((value) & EMPTY_BYTES))
 
 
 /* Generally useful conversion functions */
 #define SIZE_TO_BYTES(size) \
-  ((size) << LONG_SHIFT)
+  ((size) * sizeof (PTR))
 
 #define BYTES_TO_SIZE(bytes) \
-  ((bytes) >> LONG_SHIFT)
+  ((bytes) / sizeof (PTR))
 
 
-/* integer conversions and some information on SmallIntegers. */
+/* integer conversions and some information on SmallIntegers.  */
 
 #define TO_INT(oop) \
-  ((long)(oop) >> 1)
+  ((intptr_t)(oop) >> 1)
 
 #define FROM_INT(i) \
-  (OOP)( ((long)(i) << 1) | 1)
+  (OOP)( ((intptr_t)(i) << 1) + 1)
 
-#define ST_INT_SIZE        ((SIZEOF_LONG * 8) - 2)
+#define ST_INT_SIZE        ((sizeof (PTR) * 8) - 2)
 #define MAX_ST_INT         ((1L << ST_INT_SIZE) - 1)
 #define MIN_ST_INT         ( ~MAX_ST_INT)
 #define INT_OVERFLOW(i)    ( (i) > MAX_ST_INT || (i) < MIN_ST_INT )
 #define OVERFLOWING_INT    (MAX_ST_INT + 1)
 
-#define INCR_INT(i)         ((OOP) (((long)i) + 2))	/* 1 << 1 */
-#define DECR_INT(i)         ((OOP) (((long)i) - 2))	/* 1 << 1 */
+#define INCR_INT(i)         ((OOP) (((intptr_t)i) + 2))	/* 1 << 1 */
+#define DECR_INT(i)         ((OOP) (((intptr_t)i) - 2))	/* 1 << 1 */
 
 /* Endian conversions, using networking functions if they do
    the correct job (that is, on 32-bit little-endian systems)
-   because they are likely to be optimized. */
+   because they are likely to be optimized.  */
 
-#if SIZEOF_LONG == 4
+#if SIZEOF_INTPTR_T == 4
 # if defined(WORDS_BIGENDIAN) || !defined (HAVE_INET_SOCKETS)
 #  define BYTE_INVERT(x) \
-        ((unsigned long int)((((unsigned long int)(x) & 0x000000ffU) << 24) | \
-                             (((unsigned long int)(x) & 0x0000ff00U) <<  8) | \
-                             (((unsigned long int)(x) & 0x00ff0000U) >>  8) | \
-                             (((unsigned long int)(x) & 0xff000000U) >> 24)))
+        ((uintptr_t)((((uintptr_t)(x) & 0x000000ffU) << 24) | \
+                     (((uintptr_t)(x) & 0x0000ff00U) <<  8) | \
+                     (((uintptr_t)(x) & 0x00ff0000U) >>  8) | \
+                     (((uintptr_t)(x) & 0xff000000U) >> 24)))
 # else
 #  define BYTE_INVERT(x) htonl((x))
 # endif
 
-#else /* SIZEOF_LONG == 8 */
+#else /* SIZEOF_INTPTR_T == 8 */
 # define BYTE_INVERT(x) \
-        ((unsigned long int)((((unsigned long int)(x) & 0x00000000000000ffU) << 56) | \
-                             (((unsigned long int)(x) & 0x000000000000ff00U) << 40) | \
-                             (((unsigned long int)(x) & 0x0000000000ff0000U) << 24) | \
-                             (((unsigned long int)(x) & 0x00000000ff000000U) <<  8) | \
-                             (((unsigned long int)(x) & 0x000000ff00000000U) >>  8) | \
-                             (((unsigned long int)(x) & 0x0000ff0000000000U) >> 24) | \
-                             (((unsigned long int)(x) & 0x00ff000000000000U) >> 40) | \
-                             (((unsigned long int)(x) & 0xff00000000000000U) >> 56)))
-#endif /* SIZEOF_LONG == 8 */
+        ((uintptr_t)((((uintptr_t)(x) & 0x00000000000000ffU) << 56) | \
+                     (((uintptr_t)(x) & 0x000000000000ff00U) << 40) | \
+                     (((uintptr_t)(x) & 0x0000000000ff0000U) << 24) | \
+                     (((uintptr_t)(x) & 0x00000000ff000000U) <<  8) | \
+                     (((uintptr_t)(x) & 0x000000ff00000000U) >>  8) | \
+                     (((uintptr_t)(x) & 0x0000ff0000000000U) >> 24) | \
+                     (((uintptr_t)(x) & 0x00ff000000000000U) >> 40) | \
+                     (((uintptr_t)(x) & 0xff00000000000000U) >> 56)))
+#endif /* SIZEOF_INTPTR_T == 8 */
 
-/* The standard min/max macros... */
+/* The standard min/max macros...  */
 
+#ifndef ABS
+#define ABS(x) (x >= 0 ? x : -x)
+#endif
 #ifndef MAX
 #define MAX(x, y) 		( ((x) > (y)) ? (x) : (y) )
 #endif
 #ifndef MIN
 #define MIN(x, y) 		( ((x) > (y)) ? (y) : (x) )
 #endif
-#ifndef ABS
-#define ABS(x)	 		( ((x) > 0) ? (x) : -(x) )
-#endif
 
 /* Convenience macros to test the versions of GCC.  Note - they won't
    work for GCC1, since the _MINOR macros were not defined then, but
@@ -423,25 +444,15 @@ typedef unsigned long inc_ptr;
 
    GCC 2.96 also introduced branch prediction hints for basic block
    reordering.  We use a shorter syntax than the wordy one that GCC
-   wants. */
+   wants.  */
 #if GNUC_PREREQ (2, 96)
-#define FN_PURE      __attribute__ ((pure))
-#define UNCOMMON(x) (__builtin_expect (!!(x), 0))
-#define COMMON(x)   (__builtin_expect (!!(x), 1))
+#define UNCOMMON(x) (__builtin_expect ((x) != 0, 0))
+#define COMMON(x)   (__builtin_expect ((x) != 0, 1))
 #else
-#define FN_PURE
 #define UNCOMMON(x) (x)
 #define COMMON(x)   (x)
 #endif
 
-/* And this is the `printf' attributes to test against formatting
-   bugs.  */
-#if GNUC_PREREQ (2, 0)
-#define FN_PRINTF __attribute__ ((format(printf, 1, 2)))
-#else
-#define FN_PRINTF
-#endif
-
 /* Kill a warning when using GNU C.  Note that this allows using
    break or continue inside a macro, unlike do...while(0) */
 #ifdef __GNUC__
@@ -452,33 +463,104 @@ typedef unsigned long inc_ptr;
 #define END_MACRO else (void)0
 #endif
 
+#include <sys/types.h>
+#include <stdio.h>
+#include <stddef.h>
+#include <setjmp.h>
+#include <assert.h>
+#include <stdlib.h>
+#include <string.h>
+#include <obstack.h>
+#include <fcntl.h>
+#include <stdarg.h>
+#include <math.h>
+#include <float.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <limits.h>
+#include <poll.h>
+#include <ctype.h>
 
+#ifdef HAVE_DIRENT_H
+# include <dirent.h>
+# define NAMLEN(dirent) strlen((dirent)->d_name)
+#else
+# define dirent direct
+# define NAMLEN(dirent) (dirent)->d_namlen
+# ifdef HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif
+# ifdef HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif
+# ifdef HAVE_NDIR_H
+#  include <ndir.h>
+# endif
+#endif
+
+#if defined(HAVE_SYS_TIME_H) || defined(TIME_WITH_SYS_TIME)
+# include <sys/time.h>
+#endif
+#if !defined(HAVE_SYS_TIME_H) || defined(TIME_WITH_SYS_TIME)
+# include <time.h>
+#endif
+
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+
+#ifdef HAVE_EXECINFO_H
+#include <execinfo.h>
+#endif
+
+#ifdef HAVE_SYS_FILE_H
+#include <sys/file.h>
+#endif
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef HAVE_SYS_MMAN_H
+#include <sys/mman.h>
+#endif
+
+#include "stdintx.h"
+#include "ansidecl.h"
+#include "signalx.h"
+#include "mathl.h"
+#include "socketx.h"
+#include "strspell.h"
 #include "alloc.h"
 #include "md-config.h"
 #include "memzero.h"
+#include "avltrees.h"
+#include "rbtrees.h"
 
 #include "tree.h"
 #include "input.h"
-#include "byte.h"
 #include "callin.h"
 #include "cint.h"
 #include "dict.h"
 #include "events.h"
 #include "gstpub.h"
 #include "heap.h"
-#include "interp.h"
 #include "lex.h"
 #include "lib.h"
 #include "oop.h"
+#include "byte.h"
+#include "comp.h"
+#include "interp.h"
 #include "opt.h"
 #include "save.h"
 #include "str.h"
 #include "sysdep.h"
-#include "comp.h"
 #include "sym.h"
 #include "xlat.h"
 #include "mpz.h"
 #include "print.h"
+#include "security.h"
+#include "match.h"
 
 /* Include this last, it has the bad habit of #defining printf
    and this fools gcc's __attribute__ (format) */
