@@ -31,22 +31,16 @@
 #ifndef GST_HEAP_H
 #define GST_HEAP_H
 
-#include <stdlib.h>
-
 typedef char *heap;
 
 /* Initialize access to a heap managed region.
 
-   The heap managed region is mapped to "/dev/zero", and the data
-   will not exist in any filesystem object.
-
    On success, returns a "heap descriptor" which is used in subsequent
-   calls to other heap package functions.  It is explicitly "void *"
-   ("char *" for systems that don't fully support void) so that users
-   of the package don't have to worry about the actual implementation
-   details.
+   calls to other heap package functions.  It is explicitly "char *"
+   so that users of the package don't have to worry about the actual
+   implementation details.
 
-   On failure returns NULL. */
+   On failure returns NULL.  */
 extern heap _gst_heap_create (int size);
 
 /* Terminate access to a heap managed region by unmapping all memory pages
@@ -56,17 +50,12 @@ extern heap _gst_heap_create (int size);
    Returns NULL on success.
   
    Returns the heap descriptor on failure, which can subsequently be used
-   for further action, such as obtaining more information about the nature of
-   the failure by examining the preserved errno value.
-  
-   Note that the heap descriptor that we are using is currently located in
-   region we are about to unmap, so we first make a local copy of it on the
-   stack and use the copy. */
+   for further action.  */
 extern heap _gst_heap_destroy (heap hd);
 
-/* Get core for the memory region specified by MDP, using SIZE as the
+/* Get core for the memory region specified by HD, using SIZE as the
    amount to either add to or subtract from the existing region.  Works
-   like sbrk(), but using mmap(). */
+   like sbrk(), but using mmap() if HD is not NULL.  */
 extern PTR _gst_heap_sbrk (heap hd,
 			   size_t size);
 
