@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 2001, 2002 Free Software Foundation, Inc.
+ * Copyright 2001, 2002, 2003 Free Software Foundation, Inc.
  * Written by Paolo Bonzini.
  *
  * This file is part of GNU Smalltalk.
@@ -31,8 +31,14 @@
 #include "gst.h"
 #include "gstpriv.h"
 
+#include <sys/types.h>
 #include <poll.h>
 #include <errno.h>
+#include <stdio.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #ifdef STDC_HEADERS
 # include <string.h>
@@ -46,22 +52,13 @@
 # include <readline/history.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_IO_H
-#include <io.h>
-#endif
-
-#include <stdio.h>
-
 #define EMACS_PROCESS_MARKER	'\001'	/* ^A as marker -- random
 					   choice */
 
 typedef struct string_stream
 {
   char *strBase;		/* base of asciz string */
-  char *str;			/* pointer into asciz string */
+  const char *str;		/* pointer into asciz string */
 }
 string_stream;
 
@@ -69,8 +66,8 @@ typedef struct unix_file_stream
 {
   int fd;
   char *buf;
-  char *ptr;
-  char *end;
+  const char *ptr;
+  const char *end;
 }
 unix_file_stream;
 
@@ -78,19 +75,19 @@ typedef struct oop_stream
 {
   OOP  oop;
   char *buf;
-  char *ptr;
-  char *end;
+  const char *ptr;
+  const char *end;
 }
 oop_stream;
 
 typedef struct input_stream
 {
   stream_type type;
-  int pushedBackChars[2];	/* the 2 buffered characters */
+  char pushedBackChars[3];	/* the 3 buffered characters */
   int pushedBackCount;		/* number of chars pushed back */
   int line;
   int column;
-  char *fileName;
+  const char *fileName;
   int fileOffset;
   OOP fileNameOOP;		/* the full path name for file */
   mst_Boolean prompt;
@@ -113,7 +110,7 @@ typedef struct input_stream
 static int my_getc (input_stream stream);
 
 /* Print a line indicator in front of an error message. */
-static void line_stamp (void);
+static void line_stamp (int line);
 
 /* Close the given stream */
 static void my_close (input_stream stream);
@@ -126,9 +123,6 @@ static input_stream push_new_stream (stream_type type);
    that implements the stack. */
 static input_stream in_stream = NULL;
 
-/* The function used as a callback for rl_getc. */
-static int readline_getc (FILE * file);
-
 /* Controls the use of the changes file, for recording source text.
    If NULL, no recording */
 char *_gst_change_file_name = NULL;
@@ -177,12 +171,14 @@ static int compare_strings (const PTR a,
 			    const PTR b);
 
 /* Readline callbacks */
-static char *readline_quote_filename (char *s,
+static int readline_getc (FILE * file);
+
+static char *readline_quote_filename (const char *s,
 				      int rtype,
-				      char *qcp);
-static char *readline_dequote_filename (char *s,
+				      const char *qcp);
+static char *readline_dequote_filename (const char *s,
 					char qc);
-static char *symbol_generator (char *text,
+static char *symbol_generator (const char *text,
 				 int state);
 static char **readline_match_symbols (char *text,
 				      int start,
@@ -220,7 +216,7 @@ _gst_pop_stream (mst_Boolean closeIt)
 
 void
 _gst_push_unix_file (int fd,
-		     char *fileName)
+		     const char *fileName)
 {
   input_stream newStream;
 
@@ -263,13 +259,13 @@ _gst_push_smalltalk_string (OOP stringOOP)
 }
 
 void
-_gst_push_cstring (char *string)
+_gst_push_cstring (const char *string)
 {
   input_stream newStream;
 
   newStream = push_new_stream (STREAM_STRING);
 
-  newStream->st_str.strBase = string;
+  newStream->st_str.strBase = xstrdup (string);
   newStream->st_str.str = newStream->st_str.strBase;
   newStream->fileName = "a C string";
   newStream->prompt = false;
@@ -291,13 +287,11 @@ _gst_push_stdin_string (void)
 
   newStream = push_new_stream (STREAM_READLINE);
 
-  newStream->st_str.strBase = 0;	/* force readline() but no
-					   free() */
-  newStream->st_str.str = 0;
-  newStream->fileName = "stdin";	/* that's where we get input
-					   from */
-  newStream->prompt = false;	/* prompt is shown by readline() */
-#endif /* HAVE_READLINE */
+  newStream->st_str.strBase = NULL;	/* force readline() but no free() */
+  newStream->st_str.str = NULL;
+  newStream->fileName = "stdin";	/* that's where we get input from */
+  newStream->prompt = true;
+#endif
 }
 
 input_stream
@@ -322,10 +316,11 @@ push_new_stream (stream_type type)
 
 void
 _gst_set_stream_info (int line,
-		      char *fileName,
+		      const char *fileName,
 		      int fileOffset)
 {
   in_stream->line = line;
+  in_stream->column = 0;
   if (in_stream->type == STREAM_FILE)
     {
       in_stream->fileName = fileName;
@@ -346,7 +341,7 @@ my_getc (input_stream stream)
 
     case STREAM_OOP:
       /* Refill the buffer... */
-      if UNCOMMON (stream->st_oop.ptr == stream->st_oop.end)
+      if (stream->st_oop.ptr == stream->st_oop.end)
 	{
 	  if (stream->st_oop.buf)
 	    {
@@ -366,14 +361,23 @@ my_getc (input_stream stream)
 	? EOF : (unsigned char) *stream->st_oop.ptr++;
 
     case STREAM_FILE:
+      if (in_stream->column == 0 && in_stream->prompt)
+	{
+	  if (_gst_emacs_process)
+	    printf ("%c", EMACS_PROCESS_MARKER);
+
+	  printf ("st> ");
+	  fflush(stdout);
+	}
+
       /* Refill the buffer... */
-      if UNCOMMON (stream->st_file.ptr == stream->st_file.end)
+      if (stream->st_file.ptr == stream->st_file.end)
 	{
 	  int n;
 	  do
 	    {
 	      errno = 0;
-	      n = read (stream->st_file.fd, stream->st_file.buf, 1024);
+	      n = _gst_read (stream->st_file.fd, stream->st_file.buf, 1024);
 	    }
 	  while ((n == -1) && (errno == EINTR));
 	  if (n < 0)
@@ -485,6 +489,12 @@ my_close (input_stream stream)
 }
 
 
+mst_Boolean
+_gst_get_cur_stream_prompt (void)
+{
+  return (in_stream && in_stream->prompt);
+}
+
 stream_type
 _gst_get_cur_stream_type (void)
 {
@@ -544,8 +554,48 @@ _gst_get_cur_readline (void)
 
 
 void
-_gst_errorf (char *str,
-	     ...)
+_gst_warningf_at (int line,
+		const char *str,
+	        ...)
+{
+  va_list ap;
+
+  va_start (ap, str);
+
+  if (!_gst_report_errors)
+    return;
+
+  fflush (stdout);
+  line_stamp (line);
+  vfprintf (stderr, str, ap);
+  fprintf (stderr, "\n");
+  fflush (stderr);
+  va_end (ap);
+}
+
+void
+_gst_warningf (const char *str,
+	       ...)
+{
+  va_list ap;
+
+  va_start (ap, str);
+
+  if (!_gst_report_errors)
+    return;
+
+  fflush (stdout);
+  line_stamp (0);
+  vfprintf (stderr, str, ap);
+  fprintf (stderr, "\n");
+  fflush (stderr);
+  va_end (ap);
+}
+
+void
+_gst_errorf_at (int line,
+		const char *str,
+	        ...)
 {
   va_list ap;
 
@@ -554,19 +604,10 @@ _gst_errorf (char *str,
   if (_gst_report_errors)
     fflush (stdout);
 
-  line_stamp ();
+  line_stamp (line);
   if (_gst_report_errors)
     {
-#ifndef HAVE_VPRINTF
-# ifndef HAVE_DOPRNT
-      fputs(str, stderr);	/* not great, but perhaps better than nothing... */
-# else /* HAVE_DOPRNT */
-      _doprnt(str, &ap, stderr);
-# endif /* HAVE_DOPRNT */
-#else /* HAVE_VPRINTF */
       vfprintf (stderr, str, ap);
-#endif /* HAVE_VPRINTF */
-
       fprintf (stderr, "\n");
       fflush (stderr);
     }
@@ -580,14 +621,51 @@ _gst_errorf (char *str,
 }
 
 void
-_gst_yyerror (char *s)
+_gst_errorf (const char *str,
+	     ...)
+{
+  va_list ap;
+
+  va_start (ap, str);
+
+  if (_gst_report_errors)
+    fflush (stdout);
+
+  line_stamp (0);
+  if (_gst_report_errors)
+    {
+      vfprintf (stderr, str, ap);
+      fprintf (stderr, "\n");
+      fflush (stderr);
+    }
+  else
+    {
+      if (_gst_first_error_str == NULL)
+	vasprintf (&_gst_first_error_str, str, ap);
+    }
+
+  va_end (ap);
+}
+
+void
+_gst_yyerror (const char *s)
 {
   _gst_errorf ("%s", s);
 }
 
 void
-line_stamp (void)
+_gst_get_location (int *x, int *y)
 {
+  *x = in_stream->column;
+  *y = in_stream->line;
+}
+
+void
+line_stamp (int line)
+{
+  if (line <= 0 && in_stream)
+    line = in_stream->line;
+
   if (_gst_report_errors)
     {
       if (in_stream)
@@ -595,7 +673,7 @@ line_stamp (void)
 	  if (in_stream->fileName)
 	    fprintf (stderr, "%s:", in_stream->fileName);
 
-	  fprintf (stderr, "%d: ", in_stream->line);
+	  fprintf (stderr, "%d: ", line);
 	}
       else
 	fprintf (stderr, "gst: ");
@@ -611,7 +689,7 @@ line_stamp (void)
 		_gst_first_error_file = strdup (in_stream->fileName);
 	    }
 	  if (_gst_first_error_str == NULL)
-	    _gst_first_error_line = in_stream->line;
+	    _gst_first_error_line = line;
 	}
       else
 	{
@@ -636,8 +714,8 @@ _gst_get_method_source_from_cur_file (void)
   /* The second -1 removes the terminating '!' */
   length = endPos - startPos - 1;
   buf = (char *) xmalloc (length + 1);
-  lseek (in_stream->st_file.fd, startPos, 0);
-  read (in_stream->st_file.fd, buf, length);
+  lseek (in_stream->st_file.fd, startPos, SEEK_SET);
+  _gst_read (in_stream->st_file.fd, buf, length);
   buf[length] = '\0';
 
   /* Restore old file position, past the bytes that are still in the
@@ -708,14 +786,6 @@ _gst_next_char (void)
     }
   else
     {
-      if (in_stream->column == 0 && in_stream->prompt)
-	{
-	  if (_gst_emacs_process)
-	    printf ("%c", EMACS_PROCESS_MARKER);
-
-	  printf ("st> ");
-	  fflush(stdout);
-	}
       ic = my_getc (in_stream);
 
       if (change_str != -1)
@@ -783,12 +853,12 @@ _gst_disable_completion (void)
 #ifdef HAVE_READLINE
 /* Find apostrophes and double them */
 char *
-readline_quote_filename (char *s,
+readline_quote_filename (const char *s,
 			 int rtype,
-			 char *qcp)
+			 const char *qcp)
 {
-  char *base = alloca (strlen (s) * 2 + 2);
-  char *p, *r;
+  char *r, *base = alloca (strlen (s) * 2 + 2);
+  const char *p;
 
   int quote;
 
@@ -811,11 +881,11 @@ readline_quote_filename (char *s,
 
 /* Find double apostrophes and turn them to single ones */
 char *
-readline_dequote_filename (char *s,
+readline_dequote_filename (const char *s,
 			   char qc)
 {
-  char *base = alloca (strlen (s) + 2);
-  char *p, *r;
+  char *r, *base = alloca (strlen (s) + 2);
+  const char *p;
 
   if (!qc)
     return strdup (s);
@@ -951,7 +1021,7 @@ compare_strings (const PTR a,
 static int matches_left, current_index;
 
 char *
-symbol_generator (char *text,
+symbol_generator (const char *text,
 		  int state)
 {
   if (matches_left == 0)
@@ -1050,7 +1120,7 @@ readline_getc (FILE * file)
   do
     {
       errno = 0;
-      result = poll (&pfd, 1, INFTIM);
+      result = poll (&pfd, 1, -1); /* Infinite wait */
     }
   while ((result == -1) && (errno == EINTR));
 
@@ -1059,7 +1129,7 @@ readline_getc (FILE * file)
       do
 	{
 	  errno = 0;
-	  result = read (fileno (file), &ch, 1);
+	  result = _gst_read (fileno (file), &ch, 1);
 	}
       while ((result == -1) && (errno == EINTR));
       return (result < 1) ? EOF : (int) ch;
