@@ -33,19 +33,41 @@
 #ifndef GST_INTERP_H
 #define GST_INTERP_H
 
-#include "comp.h"
-
 /* The number of available process priorities */
-#define NUM_PRIORITIES		9
+#define NUM_PRIORITIES		 9
+#define USER_SCHEDULING_PRIORITY 4
+
+/* Ordering of file operations must match that used in
+   FileStream.st */
+enum
+{
+  PRIM_OPEN_FILE,               /* open:mode: */
+  PRIM_CLOSE_FILE,              /* no args */
+  PRIM_PUT_CHARS,               /* data:from:to: */
+  PRIM_GET_CHARS,               /* data:from:to: */
+  PRIM_FSEEK_SET,               /* position: */
+  PRIM_FTELL,                   /* no args */
+  PRIM_FEOF,                    /* no args */
+  PRIM_OPEN_PIPE,               /* open:mode: */
+  PRIM_FSEEK_CUR,               /* skip: */
+  PRIM_FSIZE,                   /* no args */
+  PRIM_FTRUNCATE,               /* no args */
+  PRIM_FILEIN,                  /* no args */
+  PRIM_FILEIN_AT,               /* line:file:charPos: */
+  PRIM_SYNC_POLL,               /* read/write/exception */
+  PRIM_ASYNC_POLL,              /* operation:semaphore: */
+  PRIM_IS_PIPE,                 /* no args */
+  PRIM_MK_TEMP                  /* base: */
+};
 
 /* These macros are used to quickly compute the number of words needed
    for a context with a maximum allowable stack depth of DEPTH.  */
-#define FIXED_CTX_SIZE	(sizeof(struct gst_method_context) / sizeof(long) - 1)
+#define FIXED_CTX_SIZE	(sizeof(struct gst_method_context) / sizeof(PTR) - 1)
 #define CTX_SIZE(depth) (((depth) << DEPTH_SCALE) + FIXED_CTX_SIZE)
 
 #define DUMMY_NATIVE_IP FROM_INT(0)
 
-/* The structure of execution context objects. */
+/* The structure of execution context objects.  */
 typedef struct gst_context_part
 {
   OBJ_HEADER;
@@ -71,7 +93,7 @@ typedef struct gst_method_context
 				   stack */
   OOP receiver;			/* the receiver OOP */
   OOP method;			/* the method that we're executing */
-  long flags;			/* must be an int to distinguish
+  intptr_t flags;		/* must be an int to distinguish
 				   gst_compiled_block/gst_method_context 
 				 */
   OOP contextStack[1];
@@ -99,7 +121,7 @@ typedef struct gst_method_context
 /* Answer whether execution started from this context (this kind
    of MethodContext is used to mark call-ins from C to Smalltalk
    and is placed on top of the context that was executing at the
-   time of the call-in, and is the parent of the called-in method). */
+   time of the call-in, and is the parent of the called-in method).  */
 #define MCF_IS_EXECUTION_ENVIRONMENT  8
 
 
@@ -119,29 +141,41 @@ typedef struct gst_block_context
 }
  *gst_block_context;
 
-/* The structure of various objects related to the process system. */
+/* The structure of various objects related to the process system.  */
 typedef struct gst_semaphore
 {
   OBJ_HEADER;
   OOP firstLink;
   OOP lastLink;
   OOP signals;
+  OOP name;
 }
  *gst_semaphore;
 
+#define PROCESS_HEADER \
+  OBJ_HEADER; \
+  OOP nextLink; \
+  OOP suspendedContext; \
+  OOP priority; \
+  OOP myList; \
+  OOP name; \
+  OOP unwindPoints; \
+  OOP interrupts; \
+  OOP interruptLock
+
 typedef struct gst_process
 {
-  OBJ_HEADER;
-  OOP nextLink;
-  OOP suspendedContext;
-  OOP priority;
-  OOP myList;
-  OOP name;
-  OOP exceptionHandlers;
-  OOP unwindPoints;
+  PROCESS_HEADER;
 }
  *gst_process;
 
+typedef struct gst_callin_process
+{
+  PROCESS_HEADER;
+  OOP returnedValue;
+}
+ *gst_callin_process;
+
 typedef struct gst_processor_scheduler
 {
   OBJ_HEADER;
@@ -149,41 +183,20 @@ typedef struct gst_processor_scheduler
   OOP activeProcess;
   OOP idleTasks;
   OOP processTimeslice;
+  OOP gcSemaphore;
+  OOP gcArray;
 }
  *gst_processor_scheduler;
 
-/* A data structure used to hold various profiling information. */
-#ifdef PROFBLOCK
-struct prof_struct
-{
-  int numMinorGCs;		/* #times minorGC called */
-  int numMajorGCs;		/* #times majorGC called */
-  int numThisContexts;
-  int numMethodAllocs;
-  int numMethodFrees;
-  int numMessageSends;
-  int numValues;		/* number of times a block was
-				   evaluated */
-  int numCacheCollisions;	/* collisions in the method cache */
-  int stackSizeSum;		/* sum at GC time of the saved stack
-				   pointers */
-  int stack_depth;		/* num records (blocks+methods) on
-				   stack now */
-  int max_stack_depth;		/* the max it reaches ever */
-};
-
-extern struct prof_struct ps;
-#endif
-
 /* Some performance counters from the interpreter: these
-   count the number of special returns. */
+   count the number of special returns.  */
 extern unsigned long _gst_literal_returns, _gst_inst_var_returns,
   _gst_self_returns;
 
-/* The number of primitives executed. */
+/* The number of primitives executed.  */
 extern unsigned long _gst_primitives_executed;
 
-/* The number of bytecodes executed. */
+/* The number of bytecodes executed.  */
 extern unsigned long _gst_bytecode_counter;
 
 /* The number of method cache misses */
@@ -194,35 +207,35 @@ extern unsigned long _gst_sample_counter;
 
 /* If this is true, for each byte code that is executed, we print on
    stdout the byte index within the current gst_compiled_method and a
-   decoded interpretation of the byte code. */
+   decoded interpretation of the byte code.  */
 extern mst_Boolean _gst_execution_tracing;
 
 /* When this is true, and an interrupt occurs (such as SIGSEGV),
    Smalltalk will terminate itself by making a core dump (normally it
-   produces a backtrace). */
+   produces a backtrace).  */
 extern mst_Boolean _gst_make_core_file;
 
 /* When true, this indicates that there is no top level loop for
-   control to return to, so it causes the system to exit. */
+   control to return to, so it causes the system to exit.  */
 extern mst_Boolean _gst_non_interactive;
 
 /* The OOP for a gst_compiled_method or gst_compiled_block that is the
-   currently executing method. */
+   currently executing method.  */
 extern OOP _gst_this_method;
 
 /* Physical address of the base of the method temporary variables.
    Typically a small number of bytes (multiple of 4 since it points to
-   OOPs) lower than sp. */
+   OOPs) lower than sp.  */
 extern OOP *_gst_temporaries;
 
-/* Physical address of the base of the method literals. */
+/* Physical address of the base of the method literals.  */
 extern OOP *_gst_literals;
 
-/* An OOP that is the current receiver of the current message. */
+/* An OOP that is the current receiver of the current message.  */
 extern OOP _gst_self;
 
 /* A gst_block_context or gst_method_context that indicates the
-   context that the interpreter is currently running in. */
+   context that the interpreter is currently running in.  */
 extern OOP _gst_this_context_oop;
 
 /* The type used to hold the instruction pointer.  For the JIT, this
@@ -231,8 +244,8 @@ extern OOP _gst_this_context_oop;
    SmallInteger and can be stored in the returnIP field of the context
    objects); for the interpreter, this is the physical address of the
    next executed bytecode (note: the global is usually synchronized at
-   sequence points only). */
-#ifdef USE_JIT_TRANSLATION
+   sequence points only).  */
+#ifdef ENABLE_JIT_TRANSLATION
 typedef int ip_type;
 extern char *native_ip;
 #else /* plain bytecode interpreter */
@@ -243,23 +256,28 @@ typedef gst_uchar *ip_type;
 extern ip_type ip;
 
 /* Set to true when some special action must be done at the next
-   sequence point. */
+   sequence point.  */
 extern volatile mst_Boolean _gst_except_flag;
 
-/* Put a fake context on the top of the stack, which is specially
+/* Create a new Process on the top of the stack, which is specially
    marked so that it stops the interpreter's execution.  This kind
    of MethodContext is used to mark call-ins from C to Smalltalk
-   and is the parent of the called-in method. */
-extern void _gst_prepare_execution_environment (void);
+   and is the parent of the called-in method.  Return the Process.  */
+extern OOP _gst_prepare_execution_environment (void);
 
-/* Remove from the context chain a context (which is the environment
-   context put there by _gst_prepare_execution_environment) and answer
-   that context's stack top. */
-extern OOP _gst_finish_execution_environment (void);
+/* Sends SELECTOR (which should be a Symbol, otherwise _gst_nil_oop is
+   returned) to RECEIVER.  The message arguments should also be OOPs
+   (otherwise, an access violation exception is pretty likely) and are
+   passed in an array ARGS of size NARGS.  The value returned from the
+   method is passed back as an OOP to the C program as the result of
+   the function, or _gst_nil_oop if the number of arguments is wrong.  */
+extern OOP _gst_nvmsg_send (OOP receiver,
+			    OOP selector,
+			    OOP *args,
+			    int nArgs);
 
-/* Start the interpreter, and go on until we fall off an environment
-   context. */
-extern void _gst_interpret (void);
+/* Start the interpreter, and go on until we terminate PROCESSOOP.  */
+extern OOP _gst_interpret (OOP processOOP);
 
 /* Internal function for SEND_MESSAGE and for sends to super; send
    SENDSELECTOR, with SENDARGS arguments, to RECEIVER.  Start looking
@@ -280,7 +298,7 @@ extern void _gst_send_message_internal (OOP sendSelector,
 			      OOP receiver,
 			      OOP method_class);
 
-/* Prepare the data structures held by the interpreter. */
+/* Prepare the data structures held by the interpreter.  */
 extern void _gst_init_interpreter (void);
 
 /* Reset the fast allocator for context objects, telling it that
@@ -293,7 +311,7 @@ extern void _gst_empty_context_pool (void);
    operation, the correct way to signal a semaphore is not to send
    #signal to the object but, rather, to use this function.  The
    signal request will be processed as soon as the next sequence point
-   is reached. */
+   is reached.  */
 extern void _gst_async_signal (OOP semaphoreOOP);
 
 /* Signal SEMAPHOREOOP so that one of the processes waiting on that
@@ -301,7 +319,7 @@ extern void _gst_async_signal (OOP semaphoreOOP);
    Smalltalk call-in is not an atomic operation, the correct way to
    signal a semaphore is not to send #signal to the object but,
    rather, to use this function.  The signal request will be processed
-   as soon as the next sequence point is reached. */
+   as soon as the next sequence point is reached.  */
 extern void _gst_async_signal_and_unregister (OOP semaphoreOOP);
 
 /* Invalidate all the cached CompiledMethod lookups.  This does NOT
@@ -313,7 +331,7 @@ extern void _gst_invalidate_method_cache (void);
 extern void _gst_show_backtrace (void);
 
 /* Trap the signals that we care about, basically SIGBUS and
-   SIGSEGV. */
+   SIGSEGV.  */
 extern void _gst_init_signals (void);
 
 /* Store the context of the VM registers into the currently executing
@@ -329,15 +347,15 @@ extern void _gst_fixup_object_pointers (void);
 /* Complementary to _gst_fixup_object_pointers, this function picks
    the relative addresses stored in the current context and uses
    them to adjust the VM registers after the heap is compacted or
-   grown. */
+   grown.  */
 extern void _gst_restore_object_pointers (void);
 
 /* This runs before every evaluation (_gst_execute_statements) and
    before GC turned on.  It creates an initial process if no process
-   is ready to run or if no process has been created yet. */
+   is ready to run or if no process has been created yet.  */
 extern void _gst_init_process_system (void);
 
-/* This function marks all the objects that the interpreter keeps in
+/* These function mark or copy all the objects that the interpreter keeps in
    the root set.  These are the semaphores that are held to be
    signaled by an asynchronous event (note that they *are* in the root
    set because they could be the only way from which we can get to the
@@ -351,13 +369,18 @@ extern void _gst_init_process_system (void);
    processes).  Processor itself is reachable from the Smalltalk
    dictionary.  */
 extern void _gst_mark_processor_registers (void);
+extern void _gst_copy_processor_registers (void);
 
 /* Print the current state of the lists of ready to run processes for
-   each priority, for debugging purposes. */
+   each priority, for debugging purposes.  */
 extern void _gst_print_process_state (void);
 
+/* Sanity check the process lists that the sole instance of ProcessorScheduler
+   holds.  */
+extern void _gst_check_process_state (void);
+
 /* Print the objects currently on the stack, for debugging
-   purposes. */
+   purposes.  */
 extern void _gst_show_stack_contents (void);
 
 /* Called after the mark phase, but before the sweep phase, so that if
@@ -368,9 +391,12 @@ extern void _gst_show_stack_contents (void);
    also if the translation to native code for the method is garbage
    collected.  In particular, this function is called *after* the
    unused method translations are marked as such, and *before* they
-   are actually freed. */
+   are actually freed.  */
 extern void _gst_validate_method_cache_entries (void);
 
+/* Terminate execution of the given PROCESSOOP.  */
+extern void _gst_terminate_process (OOP processOOP);
+
 /* Similar to _gst_send_message_internal, but forces the specified
    CompiledMethod to be sent.  If it is not valid for the current
    receiver, well, you are looking for trouble and well deserve it.
@@ -383,9 +409,13 @@ extern void _gst_send_method (OOP methodOOP);
    correct way to signal a semaphore is not to send the wait method to
    the object but, rather, to use _gst_sync_wait.  The `sync' in the
    name of this function distinguishes it from _gst_async_signal, in
-   that it cannot be called from within a signal handler. */
+   that it cannot be called from within a signal handler.  */
 extern void _gst_sync_wait (OOP semaphoreOOP);
 
+/* Take a CompiledBlock and turn it into a BlockClosure that references
+   thisContext as its static link.  */
+extern OOP _gst_make_block_closure (OOP blockOOP);
+
 /************************************************* PRIMITIVES ****************/
 
 /* The type for a routine providing the definition for one or more
@@ -395,23 +425,16 @@ extern void _gst_sync_wait (OOP semaphoreOOP);
    and the routine returns true (-1 for the JIT), indicating failure
    to invoke the primitive.  Such a function must execute a primitive,
    aided in the choice of which by the user-defined parameter ID,
-   popping NUMARGS methods off the stack if they succeed.  They also
-   can receive in METHODOOP the OOP for the currently executing
-   method: this allows primitives to poke around in the method itself,
-   to get at pieces that they need; normally, this is only used by the
-   C callout routine to get at the compiled-in descriptor for the
-   called C function.  Note that a primitive must be prepared to fail
-   if it needs METHODOOP and yet it is NULL.  */
-typedef long (*primitive_func) (int primitive, 
-				volatile int numArgs,
-				OOP methodOOP);
+   popping NUMARGS methods off the stack if they succeed.  */
+typedef intptr_t (*primitive_func) (int primitive, 
+				    volatile int numArgs);
 
-/* Table of primitives, including a primitive and its attributes. */
+/* Table of primitives, including a primitive and its attributes.  */
 typedef struct prim_table_entry
 {
-  char *name;
-  long attributes;
+  const char *name;
   primitive_func func;
+  int attributes;
   int id;
 }
 prim_table_entry;
@@ -430,15 +453,14 @@ prim_table_entry;
    system.  This function invokes the proper primitive_func with the
    correct id and the same NUMARGS and METHODOOP with which it was
    invoked.  */
-extern long _gst_execute_primitive_operation (int primitive,
-					      volatile int numArgs,
-					      OOP methodOOP);
+extern intptr_t _gst_execute_primitive_operation (int primitive,
+					          volatile int numArgs);
 
 /* This can be used to obtain information on a particular primitive
    operations in the GNU Smalltalk system.  */
-extern prim_table_entry * _gst_get_primitive_attributes (int primitive) FN_PURE;
+extern prim_table_entry * _gst_get_primitive_attributes (int primitive) ATTRIBUTE_PURE;
 
-/* Initialize the table of primitives. */
+/* Initialize the table of primitives.  */
 extern void _gst_init_primitives ();
 
 #endif /* GST_INTERP_H */
