@@ -30,27 +30,12 @@
  ***********************************************************************/
 
 
-#include "gst.h"
 #include "gstpriv.h"
-#include <obstack.h>
 
-#if defined(MSDOS) || defined (__OS2__)
-#include "gst_tab.h"
-#else
-#include "gst.tab.h"
-#endif
-
-#include <stdio.h>
-#include <ctype.h>
-#include <math.h>
 #if defined(__FreeBSD__)
 #include <floatingpoint.h>
 #endif
 
-#if STDC_HEADERS
-#  include <string.h>
-#endif /* STDC_HEADERS */
-
 /* Define this if you want the lexer to print all the tokens that it scans,
  * before passing them to the parser.
  */
@@ -76,44 +61,39 @@
    compile mode */
 mst_Boolean _gst_compile_code = false;
 
-/* The obstack containing parse tree nodes. */
+/* The obstack containing parse tree nodes.  */
 struct obstack *_gst_compilation_obstack;
 
 /* True if errors must be reported to the standard error, false if
    errors should instead stored so that they are passed to Smalltalk
-   code. */
+   code.  */
 mst_Boolean _gst_report_errors = true;
 
 /* The location of the first error reported, stored here so that
-   compilation primitives can pass them to Smalltalk code. */
+   compilation primitives can pass them to Smalltalk code.  */
 char *_gst_first_error_str = NULL;
 char *_gst_first_error_file = NULL;
-long _gst_first_error_line;
+int _gst_first_error_line;
 
 /* The starting-position state.  If this is <0, as soon as another
    token is lexed, the flag is reinitialized to the starting position
-   of the next token. */
+   of the next token.  */
 static int method_start_pos = -1;
 
-/* Answer true if IC is a valid base-10 digit. */
+/* Answer true if IC is a valid base-10 digit.  */
 static mst_Boolean is_digit (int ic);
 
-/* Answer true if C is a valid base-BASE digit. */
-static mst_Boolean is_base_digit (char c,
+/* Answer true if C is a valid base-BASE digit.  */
+static mst_Boolean is_base_digit (int c,
 				  int base);
 
-/* Answer true if the first thing available in the stream is
-   the string "primitive:" */
-static mst_Boolean parse_primitive (char c,
-				    YYSTYPE * lvalp);
-
 /* Parse the fractional part of a Float constant.  Store it in
    NUMPTR.  Read numbers in base-BASE, the first one being C. 
    Answer the scale (number of digits in numPtr).  If LARGEINTEGER
    is not NULL, the digits are stored in the buffer maintained
    by str.c, and LARGEINTEGER is set to true if numPtr does
    not have sufficient precision.  */
-static int parse_fraction (char c,
+static int parse_fraction (int c,
 			   int base,
 			   long double *numPtr,
 			   mst_Boolean *largeInteger);
@@ -124,8 +104,8 @@ static int parse_fraction (char c,
    changed accordingly.  If LARGEINTEGER is not NULL, the
    digits are stored in the buffer maintained by str.c,
    and LARGEINTEGER is set to true if the return value does
-   not have sufficient precision. */
-static long double parse_digits (char c,
+   not have sufficient precision.  */
+static long double parse_digits (int c,
 			    mst_Boolean negative,
 			    int base,
 			    mst_Boolean * largeInteger);
@@ -133,49 +113,49 @@ static long double parse_digits (char c,
 /* Parse the large integer constant stored as base-BASE
    digits in the buffer maintained by str.c, adjusting
    the sign if NEGATIVE is true.  Return an embryo of the
-   LargeInteger object as a byte_object structure. */
+   LargeInteger object as a byte_object structure.  */
 static byte_object parse_large_integer (mst_Boolean negative,
 					int base);
 
-/* Raise an error. */
-static int invalid (char c,
+/* Raise an error.  */
+static int invalid (int c,
 		    YYSTYPE * lvalp);
 
 /* Parse a comment.  C is '"'.  Return 0 to indicate the lexer
-   that this lexeme must be ignored. */
-static int comment (char c,
+   that this lexeme must be ignored.  */
+static int comment (int c,
 		    YYSTYPE * lvalp);
 
 /* Parse a character literal.  C is '$' */
-static int char_literal (char c,
+static int char_literal (int c,
 			 YYSTYPE * lvalp);
 
 /* Parse a binary operator.  C is the first symbol in the selector */
-static int parse_bin_op (char c,
+static int parse_bin_op (int c,
 			 YYSTYPE * lvalp);
 
 /* Parse a string literal.  C is '\'' */
-static int string_literal (char c,
+static int string_literal (int c,
 			   YYSTYPE * lvalp);
 
-/* Parse a number.  C is the first digit. */
-static int parse_number (char c,
+/* Parse a number.  C is the first digit.  */
+static int parse_number (int c,
 			 YYSTYPE * lvalp);
 
-/* Parse an identifier.  C is the first letter. */
-static int parse_ident (char c,
+/* Parse an identifier.  C is the first letter.  */
+static int parse_ident (int c,
 			YYSTYPE * lvalp);
 
-/* Try to parse an assignment operator.  C is ':'. */
-static int parse_colon (char c,
+/* Try to parse an assignment operator.  C is ':'.  */
+static int parse_colon (int c,
 			YYSTYPE * lvalp);
 
 /* Convert the digit C (if it is a valid base-BASE digit) to its
-   value.  Raise an error if it is invalid. */
-static int digit_to_int (char c,
+   value.  Raise an error if it is invalid.  */
+static int digit_to_int (int c,
 			 int base);
 
-/* Raise BASE to the N-th power. */
+/* Raise BASE to the N-th power.  */
 static inline long double ipowl (long double base,
 			         int n);
 
@@ -186,7 +166,7 @@ static void print_token (int token
 
 typedef struct
 {
-  int (*lexFunc) (char,
+  int (*lexFunc) (int,
 		  YYSTYPE *);
   int retToken;
   int _gst_char_class;
@@ -344,7 +324,7 @@ int _gst_yylex (PTR lvalp);
 #endif /* LEXDEBUG */
 
 int
-_gst_yylex (PTR lvalp)
+_gst_yylex (PTR lvalp, YYLTYPE *llocp)
 {
   int ic, result, tokenStartPos = 0;
   const lex_tab_elt *ct;
@@ -362,22 +342,25 @@ _gst_yylex (PTR lvalp)
          grammar use INTERNAL_TOKEN as the first token of an internal
          method. The same happens when you have a call to methodsFor:
          - INTERNAL_TOKEN is pushed so that the grammar switches to
-         the method_list production. */
+         the method_list production.  */
       _gst_compile_code = false;
       return (INTERNAL_TOKEN);
     }
 
   while ((ic = _gst_next_char ()) != EOF)
     {
+
       ct = CHAR_TAB (ic);
       if ((ct->_gst_char_class & WHITE_SPACE) == 0)
 	{
+	  _gst_get_location (&llocp->first_column, &llocp->first_line);
 	  if (method_start_pos < 0)
 	    tokenStartPos = _gst_get_cur_file_pos ();
 
 	  if (ct->lexFunc)
 	    {
 	      result = (*ct->lexFunc) (ic, (YYSTYPE *) lvalp);
+	      _gst_get_location (&llocp->last_column, &llocp->last_line);
 	      if (result)
 		{
 		  if (method_start_pos < 0)
@@ -388,7 +371,11 @@ _gst_yylex (PTR lvalp)
 		}
 	    }
 	  else if (ct->retToken)
-	    return (ct->retToken);
+	    {
+	      llocp->last_line = llocp->first_line;
+	      llocp->last_column = llocp->first_column;
+	      return (ct->retToken);
+	    }
 
 	  else
 	    {
@@ -405,7 +392,7 @@ _gst_yylex (PTR lvalp)
 
 
 int
-invalid (char c,
+invalid (int c,
 	 YYSTYPE * lvalp)
 {
   char charName[3], *cp;
@@ -430,7 +417,7 @@ invalid (char c,
 
 
 int
-comment (char c,
+comment (int c,
 	 YYSTYPE * lvalp)
 {
   int ic;
@@ -450,7 +437,7 @@ comment (char c,
 }
 
 int
-char_literal (char c,
+char_literal (int c,
 	      YYSTYPE * lvalp)
 {
   int ic;
@@ -472,7 +459,7 @@ char_literal (char c,
 }
 
 int
-parse_colon (char c,
+parse_colon (int c,
 	     YYSTYPE * lvalp)
 {
   int ic;
@@ -480,6 +467,8 @@ parse_colon (char c,
   ic = _gst_next_char ();
   if (ic == '=')
     return (ASSIGNMENT);
+  else if (ic == ':')
+    return (SCOPE_SEPARATOR);
   else
     _gst_unread_char (ic);
 
@@ -488,7 +477,7 @@ parse_colon (char c,
 
 
 int
-parse_bin_op (char c,
+parse_bin_op (int c,
 	      YYSTYPE * lvalp)
 {
   char buf[3], *bp;
@@ -498,15 +487,6 @@ parse_bin_op (char c,
   *bp++ = c;
 
   ic = _gst_next_char ();
-  if (c == '<')
-    {
-      if (ic == 'p')
-	{
-	  if (parse_primitive (ic, lvalp))
-	    return (PRIMITIVE_START);
-	}
-    }
-
   if (ic != EOF && (CHAR_TAB (ic)->_gst_char_class & SPECIAL_CHAR))
     *bp++ = ic;			/* accumulate next char */
   else
@@ -518,17 +498,18 @@ parse_bin_op (char c,
 
   *bp = '\0';
 
-  lvalp->sval = strdup (buf);
+  lvalp->sval = xstrdup (buf);
 
-  if (strcmp (buf, "|") == 0)
-    return ('|');
+  if ((buf[0] == '|' || buf[0] == '<' || buf[0] == '>')
+      && buf[1] == '\0')
+    return (buf[0]);
 
   else
     return (BINOP);
 }
 
 int
-string_literal (char c,
+string_literal (int c,
 		YYSTYPE * lvalp)
 {
   int ic;
@@ -560,19 +541,8 @@ string_literal (char c,
   return (STRING_LITERAL);
 }
 
-mst_Boolean
-parse_primitive (char c,
-		 YYSTYPE * lvalp)
-{
-  mst_Boolean result;
-
-  parse_ident (c, lvalp);
-  result = (strcmp (lvalp->sval, "primitive:") == 0);
-  return (result);
-}
-
 int
-parse_ident (char c,
+parse_ident (int c,
 	     YYSTYPE * lvalp)
 {
   int ic, identType;
@@ -588,13 +558,34 @@ parse_ident (char c,
 	     && (CHAR_TAB (ic)->_gst_char_class & ID_CHAR))
 	_gst_add_str_buf_char (ic);
 
-      if (ic == ':')
-	{			/* we have a keyword if id ends with : */
+      /* Read a dot as '::' if followed by a letter.  */
+      if (ic == '.')
+	{
 	  ic = _gst_next_char ();
 	  _gst_unread_char (ic);
+	  if (CHAR_TAB (ic)->_gst_char_class == ID_CHAR)
+	    {
+	      _gst_unread_char (':');
+	      _gst_unread_char (':');
+            }
+	  else
+	    _gst_unread_char ('.');
+
+	  break;
+        }
+
+      if (ic == ':')
+	{
+	  ic = _gst_next_char ();
+	  _gst_unread_char (ic);
+	  if (ic == ':' && identType == IDENTIFIER) /* Namespace if we have foo:: */
+	    {
+	      _gst_unread_char (':');
+              break;
+            }
+
 	  if (ic == '=')
-	    {			/* except if we have 'foo:=' => 'foo
-				   :=' */
+	    {			/* if we have 'foo:=' => 'foo :=' */
 	      if (identType != IDENTIFIER)
 		{
 		  _gst_errorf ("Malformed symbol literal");
@@ -634,23 +625,23 @@ long double
 ipowl (long double base, 
        int n)
 {
-    int k = 1;
-    long double result = 1.0;
-    while (n)
+  int k = 1;
+  long double result = 1.0;
+  while (n)
     {
-        if (n & k)
+      if (n & k)
         {
-            result *= base;
-            n ^= k;
+          result *= base;
+          n ^= k;
         }
-        base *= base;
-        k <<= 1;
+      base *= base;
+      k <<= 1;
     }
-    return result;
+  return result;
 }
 
 int
-parse_number (char c,
+parse_number (int c,
 	      YYSTYPE * lvalp)
 {
   OOP numOOP;
@@ -709,7 +700,7 @@ parse_number (char c,
 	  /* OOPS...we gobbled the '.' by mistake...it was a statement
 	     boundary delimiter.  We have an integer that we need to
 	     return, and need to push back both the . and the character 
-	     that we just read. */
+	     that we just read.  */
 	  _gst_unread_char (ic);
 	  ic = '.';
 	}
@@ -725,7 +716,7 @@ parse_number (char c,
     do
       {
         /* By default the same as the number of decimal points
-	   we used. */
+	   we used.  */
 	floatExponent = -exponent;
 
 	ic = _gst_next_char ();
@@ -779,6 +770,8 @@ parse_number (char c,
 
   if (ic == 'e' || ic == 'd' || ic == 'q')
     {
+      int exp_char = ic;
+
       switch (ic) {
 	case 'e': float_type = FLOATE_LITERAL; break;
 	case 'd': float_type = FLOATD_LITERAL; break;
@@ -792,11 +785,19 @@ parse_number (char c,
 	    parse_digits (_gst_next_char (), true, 10, NULL);
 	  exponent -= (int) floatExponent;
 	}
-      else
+      else if (CHAR_TAB (ic)->_gst_char_class & DIGIT)
 	{
 	  floatExponent = parse_digits (ic, false, 10, NULL);
 	  exponent += (int) floatExponent;
 	}
+      else if (CHAR_TAB (ic)->_gst_char_class & ID_CHAR)
+	{
+	  /* 123def sends #def to 123!!! */
+	  _gst_unread_char (ic);
+	  ic = exp_char;
+	}
+      else
+	_gst_unread_char (ic);
 
     }
   else
@@ -826,18 +827,18 @@ parse_number (char c,
     }
   else if (num >= 0 && num <= 255)
     {
-      lvalp->ival = (long) num;
+      lvalp->ival = (intptr_t) num;
       return (BYTE_LITERAL);
     }
   else
     {
-      lvalp->ival = (long) num;
+      lvalp->ival = (intptr_t) num;
       return (INTEGER_LITERAL);
     }
 }
 
 long double
-parse_digits (char c,
+parse_digits (int c,
 	      mst_Boolean negative,
 	      int base,
 	      mst_Boolean * largeInteger)
@@ -881,7 +882,7 @@ parse_digits (char c,
 }
 
 int
-parse_fraction (char c,
+parse_fraction (int c,
 		int base,
 		long double *numPtr,
 		mst_Boolean *largeInteger)
@@ -920,7 +921,7 @@ parse_fraction (char c,
 
 
 int
-digit_to_int (char c,
+digit_to_int (int c,
 	      int base)
 {
   if (c < '0' || (c > '9' && c < 'A') || c > 'Z')
@@ -947,7 +948,7 @@ digit_to_int (char c,
 }
 
 mst_Boolean
-is_base_digit (char c,
+is_base_digit (int c,
 	       int base)
 {
   if (c < '0' || (c > '9' && c < 'A') || c > 'Z')
@@ -1108,6 +1109,9 @@ print_token (token,
     case '^':
       printf ("^\n");
       break;
+    case SCOPE_SEPARATOR:
+      printf ("::\n");
+      break;
     case ASSIGNMENT:
       printf (":=\n");
       break;
