@@ -33,9 +33,6 @@
 #ifndef GST_LEX_H
 #define GST_LEX_H
 
-#include <obstack.h>
-#include <stdio.h>
-
 /* True if errors must be reported to the standard error, false if
    errors should instead stored so that they are passed to Smalltalk
    code. */
@@ -57,7 +54,7 @@ extern struct obstack *_gst_compilation_obstack;
 extern void _gst_parse_stream (void);
 
 /* Lexer interface to the lexer. */
-extern int _gst_yylex (PTR lvalp);
+extern int _gst_yylex (PTR lvalp, YYLTYPE *llocp);
 
 /* Print on file FILE a description of TOKEN, with auxiliary data
    stored in LVAL. */
