@@ -31,46 +31,17 @@
 
 
 
-#include "gst.h"
 #include "gstpriv.h"
 #include "getopt.h"
 
-#include <stdio.h>
-#include <sys/types.h>
-#ifdef HAVE_IO_H
-#include <io.h>
-#else
-#include <sys/file.h>
-#endif
-
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#if STDC_HEADERS
-#include <string.h>
-#endif /* STDC_HEADERS */
-
 #ifndef MAXPATHLEN
-#define MAXPATHLEN		1024	/* max length of a file and
-					   path */
+#define MAXPATHLEN		1024	/* max length of a file and path */
 #endif
 
 
-/* Define to _gst_debug the getopt code. */
+/* Define to debug the getopt code.  */
 /* #define DEBUG_GETOPT */
 
-#ifdef NAMES83
-#define MAP_FILE(name83, name14)	name83
-#else
-#define MAP_FILE(name83, name14)	name14
-#endif
-
-
 #ifdef atarist
 #define INIT_FILE_NAME		".gstinit"
 #define PRE_IMAGE_FILE_NAME	".gstpre"	/* hope this is ok */
@@ -100,7 +71,7 @@ static const char help_text[] =
   "\n   -c --core-dump\t\t Dump core on fatal signal"
   "\n   -d --declaration-trace-user\t Trace compilation of files on the command line"
   "\n   -D --declaration-trace-all\t Trace compilation of all loaded files"
-#ifndef USE_JIT_TRANSLATION
+#ifndef ENABLE_JIT_TRANSLATION
   "\n   -e --execution-trace-user\t Trace execution of files on the command line"
   "\n   -E --execution-trace-all\t Trace execution of all loaded files"
 #endif
@@ -109,8 +80,6 @@ static const char help_text[] =
   "\n   -i --rebuild-image\t\t Ignore the image file; rebuild it from scratch"
   "\n   -I --image-file file\t\t Instead of `gst.im', use `file' as the image\n\t\t\t\t file, and ignore the kernel files' timestamps\n"
   "\n   -K --kernel-file file\t Make file's path relative to the image path."
-  "\n   -l --log-changes\t\t Log changes in `change-log.st'"
-  "\n   -L --log-file file\t\t Log changes in file `file'"
   "\n   -p --emacs-mode\t\t Execute as a `process' (from within Emacs)"
   "\n   -q --quiet --silent\t\t Do not print execution information"
   "\n   -Q --no-messages\t\t Run Smalltalk with -q -g and no startup banner"
@@ -118,7 +87,7 @@ static const char help_text[] =
   "\n   -s --store-no-source\t\t Don't store source code as Strings for\n\t\t\t\t methods outside the kernel directory\n"
   "\n   -S --snapshot\t\t Save a snapshot just before exiting"
   "\n   -v --version\t\t\t Print the Smalltalk version number and exit"
-  "\n   -V --verbose\t\t\t Print names of loaded files"
+  "\n   -V --verbose\t\t\t Print names of loaded files and execution stats"
   "\n   -y --yacc-debug\t\t Turn on debugging in the parser"
   "\n   -\t\t\t\t Read input from standard input explicitly"
   "\n"
@@ -145,7 +114,7 @@ static const struct option long_options[] = {
   {"core-dump", 0, 0, 'c'},
   {"declaration-trace-user", 0, 0, 'd'},
   {"declaration-trace-all", 0, 0, 'D'},
-#ifndef USE_JIT_TRANSLATION
+#ifndef ENABLE_JIT_TRANSLATION
   {"execution-trace-user", 0, 0, 'e'},
   {"execution-trace-all", 0, 0, 'E'},
 #endif
@@ -154,8 +123,6 @@ static const struct option long_options[] = {
   {"rebuild-image", 0, 0, 'i'},
   {"image-file", 1, 0, 'I'},
   {"kernel-file", 1, 0, 'K'},
-  {"log-changes", 0, 0, 'l'},
-  {"log-file", 1, 0, 'L'},
   {"emacs-mode", 0, 0, 'p'},
   {"quiet", 0, 0, 'q'},
   {"no-messages", 0, 0, 'Q'},
@@ -176,24 +143,28 @@ static const struct option long_options[] = {
  */
 mst_Boolean _gst_quiet_execution;
 
+/* Set by cmd line flag.  If true, Smalltalk is more verbose about
+   what it's doing.  */
+mst_Boolean _gst_verbose = false;
+
 /* These contain the default path that was picked (after looking at the
-   environment variables) for the kernel files and the image. */
-char *_gst_kernel_file_default_path, *_gst_image_file_default_path;
+   environment variables) for the kernel files and the image.  */
+const char *_gst_kernel_file_path, *_gst_image_file_path;
 
 /* This is the name of the binary image to load.  If it is not NULL after the
    command line is parsed, the checking of the dates of the kernel source files
    against the image file date is overridden.  If it is NULL, it is set to
-   default_image_name. */
-char *_gst_binary_image_name = NULL;
+   default_image_name.  */
+const char *_gst_binary_image_name = NULL;
 
 /* This is used by the callin functions to auto-initialize Smalltalk.
    When it's not true, initialization needs to be performed.  It's set
-   to true by gst_init_smalltalk(). */
+   to true by gst_init_smalltalk().  */
 mst_Boolean _gst_smalltalk_initialized = false;
 
 /* This is used to avoid doing complicated things (currently, this
    includes call-ins before and after _gst_execute_statements) before
-   the system is ready to do them. */
+   the system is ready to do them.  */
 mst_Boolean _gst_kernel_initialized = false;
 
 /* If true, even both kernel and user method definitions are shown as
@@ -201,7 +172,7 @@ mst_Boolean _gst_kernel_initialized = false;
 mst_Boolean _gst_trace_kernel_declarations;
 
 /* If true, execution tracing is performed when loading kernel method
-   definitions. */
+   definitions.  */
 mst_Boolean _gst_trace_kernel_execution;
 
 /* This is TRUE if we are doing regression testing, and causes
@@ -220,14 +191,16 @@ mst_Boolean _gst_regression_testing;
 /* Parse the Smalltalk source code contained in FILENAME.  If QUIET
    is true, for no reason show messages about the execution state (this
    is turned on while loading kernel files).  */
-static mst_Boolean process_file (char *fileName,
-				 mst_Boolean quiet);
+static mst_Boolean process_file (const char *fileName);
+
+/* Parse the Smalltalk source code read from stdin.  */
+static void process_stdin ();
 
 /* Answer whether it is ok to load the binary image pointed to by
    _gst_binary_image_name.  This is good is the image file is local
    and newer than all of the kernel files, or if the image file is
    global, newer than all of the global kernel files, and no local
-   kernel file is found. */
+   kernel file is found.  */
 static mst_Boolean ok_to_load_binary (void);
 
 /* Attempts to find a viable kernel Smalltalk file (.st file).  First
@@ -258,68 +231,62 @@ static mst_Boolean find_kernel_file (const char *fileName,
    that is used in preference to one in the default location.  The
    default location can be overridden at runtime by setting the
    SMALLTALK_KERNEL environment variable.  */
-static void load_standard_files (void);
+static int load_standard_files (void);
 
 /* Sets up the paths for the kernel source directory and for where the
    saved Smalltalk binary image lives.  Uses environment variables
    SMALLTALK_KERNEL and SMALLTALK_IMAGE if they are set, otherwise
-   uses the paths passed by the makefiles. */
+   uses the paths passed by the makefiles.  */
 static void init_paths (void);
 
 /* This routine scans the command line arguments, accumulating
-   information and setting flags. */
-static void parse_args (int argc,
-			char **argv);
+   information and setting flags.  */
+static int parse_args (int argc,
+		       const char **argv);
 
 /* These functions load the per-user customization files, respectively
    .stinit (loaded at every startup) and .stpre (loaded before a local
-   image is saved. */
+   image is saved.  */
 static void load_user_init_file (void);
 static void load_user_pre_image_file (void);
 
 /* Set by command line flag.  When true, Smalltalk saves a snapshot after
-   loading the files on the command line, before exiting. */
+   loading the files on the command line, before exiting.  */
 static mst_Boolean snapshot_after_load = false;
 
 /* Whether SMALLTALK_IMAGE is set (only if it is set, .stpre is loaded) */
 static mst_Boolean is_local_image;
 
-/* The _gst_image_file_default_path followed by /gst.im */
+/* Usually the same as _gst_image_file_path.  */
+static char *default_image_path;
+
+/* The default_image_path followed by /gst.im */
 static char *default_image_name;
 
 /* Set by command line flag.  When true, Smalltalk does not print any
    sort of version banner at the startup of the interpreter.  This
    makes Smalltalk usable as a filter, or as a pure producer of
-   content (such as the main program in a cgi-bin script). */
-static mst_Boolean no_banner = false;
-
-/* Set by cmd line flag.  If true, Smalltalk is more verbose about
-   what it's doing. */
-static mst_Boolean verbose = false;
+   content (such as the main program in a cgi-bin script).  */
+static mst_Boolean no_errors = false;
 
 /* If true, skip date checking of kernel files vs. binary image; pretend
-   that binary image does not exist. */
+   that binary image does not exist.  */
 static mst_Boolean ignore_image;
 
-/* Set by command line flag.  When this is true, the interpreter does
-   not print out things like "execution begins" or information about
-   the number of byte codes executed. */
-static mst_Boolean run_quietly = false;
-
-
 /* The complete list of "kernel" class and method definitions.  Each
    of these files is loaded, in the order given below.  Their last
    modification dates are compared against that of the image file; if
    any are newer, the image file is ignored, these files are loaded,
-   and a new image file is created. */
+   and a new image file is created.  */
 static const char *standard_files[] = {
   "Builtins.st",
   "Object.st",
   "Message.st",
+  "DirMessage.st",
   "Boolean.st",
   "False.st",
   "True.st",
-  MAP_FILE ("Magnitud.st", "Magnitude.st"),
+  "Magnitude.st",
   "Integer.st",
   "Date.st",
   "Time.st",
@@ -331,64 +298,81 @@ static const char *standard_files[] = {
   "Fraction.st",
   "LargeInt.st",
   "SmallInt.st",
-  MAP_FILE ("Charactr.st", "Character.st"),
-  MAP_FILE ("LookupK.st", "LookupKey.st"),
-  MAP_FILE ("Assoc.st", "Association.st"),
+  "Character.st",
+  "LookupKey.st",
+  "Association.st",
+  "HomedAssoc.st",
+  "VarBinding.st",
   "Link.st",
   "Process.st",
-  MAP_FILE ("CompCode.st", "CompildCode.st"),
-  MAP_FILE ("CompMeth.st", "CompildMeth.st"),
-  MAP_FILE ("CompBlk.st", "CompiledBlk.st"),
-  MAP_FILE ("Collectn.st", "Collection.st"),
-  MAP_FILE ("SeqColl.st", "SeqCollect.st"),
-  MAP_FILE ("LnkList.st", "LinkedList.st"),
-  MAP_FILE ("Semaphor.st", "Semaphore.st"),
-  MAP_FILE ("ArrdColl.st", "ArrayColl.st"),
+  "CallinProcess.st",
+  "CompildCode.st",
+  "CompildMeth.st",
+  "CompiledBlk.st",
+  "Collection.st",
+  "SeqCollect.st",
+  "LinkedList.st",
+  "Semaphore.st",
+  "ArrayColl.st",
   "Array.st",
-  MAP_FILE ("ByteArr.st", "ByteArray.st"),
-  MAP_FILE ("CharArr.st", "CharArray.st"),
+  "ByteArray.st",
+  "CharArray.st",
   "String.st",
   "Symbol.st",
   "Interval.st",
-  MAP_FILE ("OrdColl.st", "OrderColl.st"),
-  MAP_FILE ("SortColl.st", "SortCollect.st"),
+  "OrderColl.st",
+  "SortCollect.st",
   "Bag.st",
-  MAP_FILE ("MapColl.st", "MappedColl.st"),
-  MAP_FILE ("HashColl.st", "HashedColl.st"),
+  "MappedColl.st",
+  "HashedColl.st",
   "Set.st",
-  MAP_FILE ("IdentSet.st", "IdentitySet.st"),
-  MAP_FILE ("Dict.st", "Dictionary.st"),
-  MAP_FILE ("LookupT.st", "LookupTable.st"),
-  MAP_FILE ("IdDict.st", "IdentDict.st"),
-  MAP_FILE ("MthDict.st", "MethodDict.st"),
-  MAP_FILE ("RootNS.st", "RootNamespc.st"),
-  MAP_FILE ("Namespc.st", "Namespace.st"),
-  MAP_FILE ("SysDict.st", "SysDict.st"),
+  "IdentitySet.st",
+  "Dictionary.st",
+  "LookupTable.st",
+  "IdentDict.st",
+  "MethodDict.st",
+  "BindingDict.st",
+  "AbstNamespc.st",
+  "RootNamespc.st",
+  "Namespace.st",
+  "SysDict.st",
   "Stream.st",
-  MAP_FILE ("PosStrm.st", "PosStream.st"),
-  MAP_FILE ("RdStream.st", "ReadStream.st"),
-  MAP_FILE ("WrStream.st", "WriteStream.st"),
+  "PosStream.st",
+  "ReadStream.st",
+  "WriteStream.st",
   "RWStream.st",
-  MAP_FILE ("ByteStrm.st", "ByteStream.st"),
-  MAP_FILE ("TokStrm.st", "TokenStream.st"),
+  "ByteStream.st",
+  "TokenStream.st",
   "Random.st",
-  MAP_FILE ("UndefObj.st", "UndefObject.st"),
-  MAP_FILE ("ProcSch.st", "ProcSched.st"),
+  "UndefObject.st",
+  "ProcSched.st",
   "Delay.st",
-  MAP_FILE ("ShQueue.st", "SharedQueue.st"),
+  "SharedQueue.st",
   "Behavior.st",
-  MAP_FILE ("ClassDsc.st", "ClassDesc.st"),
+  "ClassDesc.st",
   "Class.st",
-  MAP_FILE ("Metaclas.st", "Metaclass.st"),
-  MAP_FILE ("Context.st", "ContextPart.st"),
-  MAP_FILE ("MthCtxt.st", "MthContext.st"),
-  MAP_FILE ("BlkCtxt.st", "BlkContext.st"),
-  MAP_FILE ("BlkClsr.st", "BlkClosure.st"),
+  "Metaclass.st",
+  "ContextPart.st",
+  "MthContext.st",
+  "BlkContext.st",
+  "BlkClosure.st",
   "Memory.st",
-  MAP_FILE ("MthInfo.st", "MethodInfo.st"),
-  MAP_FILE ("FileSeg.st", "FileSegment.st"),
+  "MethodInfo.st",
+  "FileSegment.st",
+  "FileDescr.st",
   "SymLink.st",
-  MAP_FILE ("ObjMem.st", "ObjMemory.st"),
+  "ObjMemory.st",
+
+  /* More core classes */
+  "WeakObjects.st",
+  "RecursionLock.st",
+  "Point.st",
+  "Rectangle.st",
+  "RunArray.st",
+  "OtherArrays.st",
+  "AnsiDates.st",
+  "ScaledDec.st",
+  "ValueAdapt.st",
 
   /* C call-out facilities */
   "CObject.st",
@@ -396,72 +380,62 @@ static const char *standard_files[] = {
   "CFuncs.st",
   "CStruct.st",
 
-  /* More core classes */
-  MAP_FILE ("WeakObjs.st", "WeakObjects.st"),
-  "Point.st",
-  MAP_FILE ("Rect.st", "Rectangle.st"),
-  "RunArray.st",
-  MAP_FILE ("Arrays.st", "OtherArrays.st"),
-  MAP_FILE ("DirMsg.st", "DirMessage.st"),
-  MAP_FILE ("AnsiDate.st", "AnsiDates.st"),
-  MAP_FILE ("Scaled.st", "ScaledDec.st"),
+  /* Exception handling */
+  "ExcHandling.st",
+  "AnsiExcept.st",
 
   /* Virtual filesystem layer */
   "File.st",
-  MAP_FILE ("Dir.st", "Directory.st"),
+  "Directory.st",
   "VFS.st",
-  MAP_FILE ("FDescr.st", "FileDescr.st"),
-  MAP_FILE ("FStream.st", "FileStream.st"),
-  MAP_FILE ("Trnscrpt.st", "Transcript.st"),	/* yuck! no vocals! */
-
-  /* Exception handling */
-  MAP_FILE ("ExcHandl.st", "ExcHandling.st"),
-  MAP_FILE ("AnsiExc.st", "AnsiExcept.st"),
+  "URL.st",
+  "FileStream.st",
+  "Transcript.st",
 
   /* Goodies */
   "Autoload.st",
-  "Browser.st",
-  MAP_FILE ("Adaptors.st", "ValueAdapt.st"),
-  MAP_FILE ("ObjDump.st", "ObjDumper.st"),
-  MAP_FILE ("PkgLoad.st", "PkgLoader.st"),
+  "ObjDumper.st",
+  "PkgLoader.st",
   "DLD.st",
   NULL
 };
 
 /* The argc and argv that are passed to libgst via gst_smalltalk_args. 
-   The default is passing no parameters. */
-static char *smalltalk_arg_vec[] = { "gst", NULL };
+   The default is passing no parameters.  */
+static const char *smalltalk_arg_vec[] = { "gst", NULL };
 static int smalltalk_argc = 0;
-static char **smalltalk_argv = smalltalk_arg_vec;
+static const char **smalltalk_argv = smalltalk_arg_vec;
 
 /* The argc and argv that are made available to Smalltalk programs
-   through the -a option. */
+   through the -a option.  */
 int _gst_smalltalk_passed_argc = 0;
-char **_gst_smalltalk_passed_argv = NULL;
+const char **_gst_smalltalk_passed_argv = NULL;
 
 
 
 void
 gst_smalltalk_args (int argc,
-		    char **argv)
+		    const char **argv)
 {
   smalltalk_argc = argc;
   smalltalk_argv = argv;
 }
 
 
-void
+int
 gst_init_smalltalk (void)
 {
-  mst_Boolean loadBinary, traceUserDeclarations, traceUserExecution;
+  mst_Boolean loadBinary, abortOnFailure;
+  mst_Boolean traceUserDeclarations, traceUserExecution;
+  int result;
   char *p;
 
   _gst_yydebug = 0;
   _gst_trace_kernel_declarations = _gst_declare_tracing = false;
   _gst_trace_kernel_execution = _gst_execution_tracing = false;
   _gst_regression_testing = false;
+  _gst_verbose = false;
   ignore_image = false;
-  verbose = false;
 
   /* Even though we're nowhere near through initialization, we set this
      to make sure we don't invoke a callin function which would recursively
@@ -471,11 +445,12 @@ gst_init_smalltalk (void)
   init_paths ();
   _gst_init_snprintfv ();
 
-  parse_args (smalltalk_argc, smalltalk_argv);
+  result = parse_args (smalltalk_argc, smalltalk_argv);
+  if (result)
+    return result;
 
   _gst_init_sysdep ();
   _gst_init_signals ();
-  _gst_init_mem ();
   _gst_init_cfuncs ();
   _gst_init_primitives ();
 
@@ -483,7 +458,8 @@ gst_init_smalltalk (void)
     {
       _gst_trace_kernel_declarations = _gst_declare_tracing = false;
       _gst_trace_kernel_execution = _gst_execution_tracing = false;
-      verbose = false;
+      _gst_verbose = false;
+      setvbuf (stdout, NULL, _IOLBF, 1024);
     }
 
   traceUserDeclarations = _gst_declare_tracing;
@@ -491,37 +467,35 @@ gst_init_smalltalk (void)
   _gst_declare_tracing = _gst_trace_kernel_declarations;
   _gst_execution_tracing = _gst_trace_kernel_execution;
 
-  if (_gst_binary_image_name
-      && _gst_file_is_readable (_gst_binary_image_name))
-    loadBinary = true;
+  if (_gst_binary_image_name)
+    loadBinary = abortOnFailure = true;
   else
     {
       _gst_binary_image_name = default_image_name;
       loadBinary = !ignore_image && ok_to_load_binary();
-    }
+      abortOnFailure = false;
 
-  /* If we must create a new non-local image, but the directory is
-     not writeable, we must resort to the current directory.  In
-     practice this is what is used when working in the build directory.  */
+      /* If we must create a new non-local image, but the directory is
+         not writeable, we must resort to the current directory.  In
+         practice this is what is used when working in the build directory.  */
 
-  if (!loadBinary
-      && !is_local_image
-      && !_gst_file_is_writeable (_gst_image_file_default_path))
-    {
-      xfree (_gst_binary_image_name);
-
-      is_local_image = true;
-      _gst_image_file_default_path = _gst_get_cur_dir_name ();
-      _gst_binary_image_name = "gst.im";
-      loadBinary = !ignore_image && ok_to_load_binary();
+      if (!loadBinary
+          && !is_local_image
+          && !_gst_file_is_writeable (_gst_image_file_path))
+        {
+          is_local_image = true;
+          xfree (default_image_path);
+          _gst_image_file_path = default_image_path = _gst_get_cur_dir_name ();
+          _gst_binary_image_name = "gst.im";
+          loadBinary = !ignore_image && ok_to_load_binary();
+        }
     }
 
   /* Compute the actual path of the image file */
-  xfree (_gst_image_file_default_path);
-
-  p = _gst_image_file_default_path =
+  _gst_image_file_path = p =
     _gst_get_full_file_name (_gst_binary_image_name);
-  p += strlen (_gst_image_file_default_path);
+  
+  p += strlen (_gst_image_file_path);
 #if defined(MSDOS) || defined(WIN32) || defined(__OS2__)
   while (*--p != '/' && *p != '\\');
 #else
@@ -531,27 +505,40 @@ gst_init_smalltalk (void)
 
   if (loadBinary && _gst_load_from_file (_gst_binary_image_name))
     {
-      _gst_init_changes_stream ();
-      _gst_init_compiler ();
       _gst_init_interpreter ();
+      _gst_init_compiler ();
       _gst_init_vmproxy ();
     }
   else
     {
-      _gst_init_ooptable (INITIAL_OOP_TABLE_SIZE);
+      mst_Boolean willRegressTest = _gst_regression_testing;
+      _gst_regression_testing = false;
+
+      if (abortOnFailure)
+	{
+	  _gst_errorf ("Couldn't load image file %s", _gst_binary_image_name);
+	  return 1;
+	}
+
+      _gst_init_oop_table (INITIAL_OOP_TABLE_SIZE);
+      _gst_init_mem_default ();
       _gst_init_dictionary ();
-      _gst_init_compiler ();
       _gst_init_interpreter ();
+      _gst_init_compiler ();
       _gst_init_vmproxy ();
 
-      _gst_reset_changes_file ();
       _gst_install_initial_methods ();
-      _gst_init_changes_stream ();
 
-      load_standard_files ();
+      result = load_standard_files ();
+      if (!result)
+	{
+          if (is_local_image)
+	    load_user_pre_image_file ();
+	}
 
-      if (is_local_image)
-	load_user_pre_image_file ();
+      _gst_regression_testing = willRegressTest;
+      if (result)
+	return result;
 
       if (!_gst_save_to_file (_gst_binary_image_name))
 	_gst_errorf ("Couldn't open file %s", _gst_binary_image_name);
@@ -567,6 +554,8 @@ gst_init_smalltalk (void)
 #ifdef HAVE_READLINE
   _gst_initialize_readline ();
 #endif /* HAVE_READLINE */
+
+  return 0;
 }
 
 void
@@ -574,24 +563,15 @@ gst_top_level_loop (void)
 {
   int filesProcessed;
 
-  if (!no_banner)
-    printf ("GNU Smalltalk ready\n\n");
-
   for (filesProcessed = 0; *++smalltalk_argv;)
     {
       if (smalltalk_argv[0][0] == '-')
-	{
-	  /* - by itself indicates standard input */
-	  _gst_non_interactive = false;
-	  _gst_push_stdin_string ();
-	  _gst_parse_stream ();
-	  _gst_pop_stream (true);
-	  _gst_non_interactive = true;
-	}
+        /* - by itself indicates standard input */
+        process_stdin ();
       else
 	{
 	  _gst_use_undeclared++;
-	  if (!process_file (smalltalk_argv[0], false) && !no_banner)
+	  if (!process_file (smalltalk_argv[0]) && !no_errors)
 	    _gst_errorf ("Couldn't open file %s", smalltalk_argv[0]);
 
 	  _gst_use_undeclared--;
@@ -600,12 +580,7 @@ gst_top_level_loop (void)
     }
 
   if (filesProcessed == 0)
-    {
-      /* didn't do any from cmd line, so read stdin */
-      _gst_non_interactive = false;
-      _gst_push_stdin_string ();
-      _gst_parse_stream ();
-    }
+    process_stdin ();
 
   if (snapshot_after_load)
     _gst_save_to_file (_gst_binary_image_name);
@@ -619,39 +594,49 @@ void
 init_paths (void)
 {
   char *currentDirectory = _gst_get_cur_dir_name ();
+  const char *kernel_path, *image_path;
 
-  _gst_kernel_file_default_path = (char *) getenv ("SMALLTALK_KERNEL");
-  _gst_image_file_default_path = (char *) getenv ("SMALLTALK_IMAGE");
+  kernel_path = (char *) getenv ("SMALLTALK_KERNEL");
+  image_path = (char *) getenv ("SMALLTALK_IMAGE");
   is_local_image = true;
 
-  if (!_gst_kernel_file_default_path
-      || !_gst_file_is_readable (_gst_kernel_file_default_path))
-    asprintf (&_gst_kernel_file_default_path, "%s/kernel",
-	      currentDirectory);
-
-  if (!_gst_image_file_default_path
-      || !_gst_file_is_readable (_gst_image_file_default_path))
+  if (!kernel_path
+      || !_gst_file_is_readable (kernel_path))
     {
-      _gst_image_file_default_path = IMAGE_PATH;
+      char *kernel_file_path;
+      asprintf (&kernel_file_path, "%s/kernel",
+	        currentDirectory);
 
-      if (_gst_file_is_readable (_gst_image_file_default_path))
+      _gst_kernel_file_path = kernel_file_path;
+    }
+  else
+    _gst_kernel_file_path = xstrdup (kernel_path);
+
+  if (!image_path
+      || !_gst_file_is_readable (image_path))
+    {
+      image_path = IMAGE_PATH;
+
+      if (_gst_file_is_readable (image_path))
 	/* Found in the standard image path.  Apply this kludge so
 	   that OSes such as Windows and MS-DOS which have no concept 
-	   of home directories always load the .stpre file. */
+	   of home directories always load the .stpre file.  */
 	is_local_image = (((char *) getenv ("HOME")) == NULL);
+      else
+	image_path = currentDirectory;
     }
 
   asprintf (&default_image_name, "%s/gst.im",
-	    _gst_image_file_default_path);
+	    image_path);
 
-  _gst_image_file_default_path = strdup (_gst_image_file_default_path);
+  _gst_image_file_path = default_image_path = xstrdup (image_path);
   xfree (currentDirectory);
 }
 
 mst_Boolean
 ok_to_load_binary (void)
 {
-  long imageFileTime;
+  time_t imageFileTime;
   const char **fileNames;
   char fullFileName[MAXPATHLEN], *home, preImageFileName[MAXPATHLEN];
 
@@ -667,7 +652,7 @@ ok_to_load_binary (void)
 	{
 	  /* file lives locally but the image doesn't -- bad semantics.
 	     Note that if SOME of the files are local and the image file
-	     is local, it is good. */
+	     is local, it is good.  */
 	  return (false);
 	}
       if (imageFileTime < _gst_get_file_modify_time (fullFileName))
@@ -688,7 +673,7 @@ ok_to_load_binary (void)
   return (true);
 }
 
-void
+int
 load_standard_files (void)
 {
   const char **fileNames;
@@ -702,17 +687,18 @@ load_standard_files (void)
   for (fileNames = standard_files; *fileNames; fileNames++)
     {
       find_kernel_file (*fileNames, fullFileName);
-      if (!process_file (fullFileName, true))
+      if (!process_file (fullFileName))
 	{
 	  _gst_errorf ("can't find system file '%s'", fullFileName);
 	  _gst_errorf
 	    ("image bootstrap failed, set SMALLTALK_KERNEL to the directory name");
-	  exit (1);
+	  return 1;
 	}
     }
   _gst_use_undeclared--;
 
   _gst_store_no_source = willStoreNoSource;
+  return 0;
 }
 
 
@@ -726,14 +712,14 @@ find_kernel_file (const char *fileName,
       return (true);
     }
 
-  sprintf (fullFileName, "%s/%s", _gst_kernel_file_default_path,
+  sprintf (fullFileName, "%s/%s", _gst_kernel_file_path,
 	   fileName);
   if (_gst_file_is_readable (fullFileName))
     {
       char systemFileName[256];
       sprintf (systemFileName, KERNEL_PATH "/%s", fileName);
       /* If this file and the system file are the same, consider the
-         file as a system file instead. */
+         file as a system file instead.  */
       return (!_gst_file_is_readable (systemFileName) ||
 	      _gst_get_file_modify_time (fullFileName) !=
 	      _gst_get_file_modify_time (systemFileName));
@@ -754,7 +740,7 @@ load_user_pre_image_file (void)
   else
     strcpy (fileName, PRE_IMAGE_FILE_NAME);
 
-  process_file (fileName, false);
+  process_file (fileName);
 }
 
 void
@@ -762,17 +748,35 @@ load_user_init_file (void)
 {
   char fileName[MAXPATHLEN], *home;
 
+  if (_gst_regression_testing)
+    return;
+
   if ((home = (char *) getenv ("HOME")) != NULL)
     sprintf (fileName, "%s/%s", home, INIT_FILE_NAME);
   else
     strcpy (fileName, INIT_FILE_NAME);
 
-  process_file (fileName, false);
+  process_file (fileName);
+}
+
+void
+process_stdin ()
+{
+ if (!no_errors)
+    {
+      printf ("GNU Smalltalk ready\n\n");
+      fflush (stdout);
+    }
+
+  _gst_non_interactive = false;
+  _gst_push_stdin_string ();
+  _gst_parse_stream ();
+  _gst_pop_stream (true);
+  _gst_non_interactive = true;
 }
 
 mst_Boolean
-process_file (char *fileName,
-	      mst_Boolean quiet)
+process_file (const char *fileName)
 {
   int fd;
 
@@ -780,26 +784,24 @@ process_file (char *fileName,
   if (fd == -1)
     return (false);
 
-  if (verbose)
+  if (_gst_verbose)
     printf ("Processing %s\n", fileName);
 
-  _gst_quiet_execution = _gst_quiet_execution || quiet;
   _gst_push_unix_file (fd, fileName);
   _gst_parse_stream ();
   _gst_pop_stream (true);
-  _gst_quiet_execution = run_quietly || _gst_emacs_process;
   return (true);
 }
 
 
-void
+int
 parse_args (int argc,
-	    char **argv)
+	    const char **argv)
 {
-  char **av = argv;
+  const char **av = argv;
   int ch, prev = 0, prevOptind = -1;
 
-#ifndef USE_DYNAMIC_TRANSLATION
+#ifndef ENABLE_DYNAMIC_TRANSLATION
 # define OPTIONS "-acdDeEghHiI:K:lL:pQqrsSvVy"
 #else
 # define OPTIONS "-acdDghHiI:K:lL:pQqrsSvVy"
@@ -809,7 +811,7 @@ parse_args (int argc,
   opterr = 1;
 
   while ((ch =
-	  getopt_long (argc, argv, OPTIONS, long_options, NULL)) != -1)
+	  getopt_long (argc, (char **) argv, OPTIONS, long_options, NULL)) != -1)
     {
 #undef OPTIONS
 
@@ -823,7 +825,7 @@ parse_args (int argc,
 	  /* The first option after -a was not introduced by a minus,
 	     so getopt_long passed it to us as '\1'.  Undo the effect
 	     of this last getopt_long, so that this option is actually
-	     passed to Smalltalk. */
+	     passed to Smalltalk.  */
 	  optind--;
 	  break;
 	}
@@ -842,7 +844,7 @@ parse_args (int argc,
 	case 'd':
 	  _gst_declare_tracing = true;
 	  break;
-#ifndef USE_JIT_TRANSLATION
+#ifndef ENABLE_JIT_TRANSLATION
 	case 'E':
 	  _gst_trace_kernel_execution = true;		/* fall thru */
 	case 'e':
@@ -860,9 +862,9 @@ parse_args (int argc,
 	  break;
 	case 'Q':
 	  _gst_gc_message = false;
-	  no_banner = true;	/* fall thru */
+	  no_errors = true;	/* fall thru */
 	case 'q':
-	  run_quietly = true;
+          _gst_quiet_execution = true;
 	  break;
 	case 'r':
 	  _gst_regression_testing = true;
@@ -874,53 +876,42 @@ parse_args (int argc,
 	  snapshot_after_load = true;
 	  break;
 	case 'V':
-	  verbose = true;
+	  _gst_verbose = true;
 	  break;
 	case 'y':
 	  _gst_yydebug = 1;
 	  break;
 
-	  /* ??? Is there some better place for this to be? Canonical
-	     places seem to lose in that canonical places are often
-	     write-protected, which is ok for extant changes (like in a 
-	     saved system), but not for new changes by the user.
-	     Again, the notion of "the" user's home area is kind of
-	     random if there are multiple snapshots, since each
-	     snapshot should be associated with a particular changes
-	     file for which it is relevant. We use the current
-	     directory and allow to override with the -L option. */
-	case 'l':
-	  _gst_change_file_name = "change-log.st";
-	  break;
-
 	case 'a':
 	  /* "Officially", the C command line ends here.  The Smalltalk 
 	     command line, instead, starts right after the parameter
 	     containing -a. -a is handled specially by the code that
 	     tests the prev and prevOptind variables, so that ./gst -aI 
 	     xxx yyy for example interprets xxx as the image to be
-	     loaded. */
+	     loaded.  */
 	  break;
 
 	case 'I':
 	  _gst_binary_image_name = optarg;
+          if (!_gst_file_is_readable (_gst_binary_image_name))
+	    {
+	      _gst_errorf ("Couldn't open image file %s", _gst_binary_image_name);
+	      return 1;
+	    }
 	  break;
 
 	case 'K':
 	  {
-	    asprintf (++av, "%s/%s", _gst_image_file_default_path,
-		      optarg);
+	    char *file;
+	    asprintf (&file, "%s/%s", _gst_image_file_path, optarg);
+	    *++av = file;
 	    break;
 	  }
 
-	case 'L':
-	  _gst_change_file_name = optarg;
-	  break;
-
 	case 'v':
 	  printf (copyright_and_legal_stuff_text, VERSION, KERNEL_PATH,
 		  IMAGE_PATH);
-	  exit (0);
+	  return -1;
 
 	default:
 	  /* Fall through and show help message */
@@ -929,13 +920,13 @@ parse_args (int argc,
 	case 'H':
 	case '?':
 	  printf (help_text);
-	  exit (0);
+	  return -1;
 	}
 
       if (prev != 'a')
 	{
 	  /* Track whether we have found a '-a' argument, by updating
-	     prev only until we find it. */
+	     prev only until we find it.  */
 	  prev = ch;
 
 	}
@@ -943,18 +934,18 @@ parse_args (int argc,
 	{
 	  /* We had found a '-a' argument and we ate all the remaining
 	     options in the element of argv that contained -a.  All the 
-	     remaining arguments are passed to Smalltalk. */
+	     remaining arguments are passed to Smalltalk.  */
 	  break;
 	}
       prevOptind = optind;
     }
 
-  _gst_quiet_execution = run_quietly || _gst_emacs_process;
-
   *++av = NULL;
   if (prev == 'a')
     {
       _gst_smalltalk_passed_argc = argc - optind;
       _gst_smalltalk_passed_argv = argv + optind;
     }
+
+  return 0;
 }
