@@ -34,19 +34,19 @@
 #define GST_LIB_H
 
 /* These contain the default path that was picked (after looking at the
-   environment variables) for the kernel files and the image. */
-extern char *_gst_kernel_file_default_path;
-extern char *_gst_image_file_default_path;
+   environment variables) for the kernel files and the image.  */
+extern const char *_gst_kernel_file_path;
+extern const char *_gst_image_file_path;
 
 /* This is the name of the binary image to load.  If it is not NULL after the
    command line is parsed, the checking of the dates of the kernel source files
    against the image file date is overridden.  If it is NULL, it is set to
-   default_image_name. */
-extern char *_gst_binary_image_name;
+   default_image_name.  */
+extern const char *_gst_binary_image_name;
 
 /* This is used by the callin functions to auto-initialize Smalltalk.
    When it's not true, initialization needs to be performed.  It's set
-   to true by gst_init_smalltalk(). */
+   to true by gst_init_smalltalk().  */
 extern mst_Boolean _gst_smalltalk_initialized;
 
 /* This is TRUE if we are doing regression testing, and causes
@@ -56,17 +56,17 @@ extern mst_Boolean _gst_regression_testing;
 
 /* When true, this flag suppresses the printing of execution-related
    messages, such as the number of byte codes executed by the
-   last expression, etc. */
+   last expression, etc.  */
 extern mst_Boolean _gst_quiet_execution;
 
 /* The argc and argv that are made available to Smalltalk programs
-   through the -a option. */
+   through the -a option.  */
 extern int _gst_smalltalk_passed_argc;
-extern char **_gst_smalltalk_passed_argv;
+extern const char **_gst_smalltalk_passed_argv;
 
 /* This is used to avoid doing complicated things (currently, this
    includes call-ins before and after _gst_execute_statements) before
-   the system is ready to do them. */
+   the system is ready to do them.  */
 extern mst_Boolean _gst_kernel_initialized;
 
 /* If true, even both kernel and user method definitions are shown as
@@ -74,7 +74,11 @@ extern mst_Boolean _gst_kernel_initialized;
 extern mst_Boolean _gst_trace_kernel_declarations;
 
 /* If true, execution tracing is performed when loading kernel method
-   definitions. */
+   definitions.  */
 extern mst_Boolean _gst_trace_kernel_execution;
 
+/* Set by cmd line flag.  If true, Smalltalk is more verbose about
+   what it's doing.  */
+extern mst_Boolean _gst_verbose;
+
 #endif /* GST_LIB_H */
