@@ -1629,8 +1629,10 @@ _gst_mpz_from_oop(gst_mpz *mpz, OOP srcOOP)
     }
 #endif
 
-  /* DEST now points past the limbs in the new gst_mpz */
-  dest[-1] &= adjust;
+  /* DEST now points past the limbs in the new gst_mpz.  Don't do
+     gratuitous writes to avoid over-GCing. */
+  if (dest[-1] & ~adjust)
+    dest[-1] &= adjust;
 
   /* A few functions require that the most significant limb be >0.  */
   if (mpz->size < 0 && mpz->d[-mpz->size-1] == 0)
