@@ -55,22 +55,14 @@ extern void _gst_compute_stack_positions (gst_uchar * bp,
 					  PTR * base,
 					  PTR ** pos);
 
-/* For now, this could fill a table that says which bytecodes leave a 
-   SmallInteger as the stack-top (but it is disabled because it is
-   not finished yet).  In the future it will prepare a full CodeTree
-   because CodeTree generation will not be done on the fly, and will
-   annotate it with SSA information. 
-
-   METHODOOP is the OOP for a CompiledMethod or CompiledBlock to be
+/* METHODOOP is the OOP for a CompiledMethod or CompiledBlock to be
    analyzed, having SIZE bytecodes.  DEST is an array of SIZE items,
    which on output has non-zero items for jump destinations only: in
    particular, it is positive for a forward jump and negative for a
-   backward jump.  Temporary data is allocated into OBSTACK. */
+   backward jump.  */
 extern void _gst_analyze_bytecodes (OOP methodOOP,
 				    int size,
-				    char *dest,
-				    char *inIntTab,
-				    struct obstack *obstack);
+				    char *dest);
 
 /* This looks at BYTECODES and checks if they could be replaced
    with an optimized return of self, of an instance variable or of
