@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -34,13 +34,9 @@
 #include "gstpriv.h"
 #include "snprintfv/mem.h"
 
+#include <float.h>
+
 
-/* Show information about the contents of the object OBJ. */
-static void display_object (mst_Object obj);
-
-/* Show information about the contents of the indirect object pointer OOP. */
-static void display_oop (OOP oop);
-
 /* Print a String OOP to a snprintfv stream, STREAM.  */
 static void print_string_to_stream (STREAM *stream,
 				    OOP string);
@@ -62,12 +58,13 @@ static void print_oop_constructor_to_stream (STREAM *stream,
    quotes, while %+O expects that an Association is passed and prints
    its key.  */
 static void printf_oop (STREAM *stream,
-                        struct printf_info *info,
-                        const void *const *args);
+		        struct printf_info *info,
+        		const void *const *args);
 
 static int printf_oop_arginfo (struct printf_info *info,
-                               size_t n,
+			       size_t n,
                                int *argtypes);
+
 
 
 void
@@ -150,7 +147,7 @@ print_oop_constructor_to_stream (STREAM *stream, OOP oop)
 
 void
 printf_oop (STREAM *stream,
-            struct printf_info *info,
+	    struct printf_info *info,
             const void *const *args)
 {
   OOP oop = (OOP) (args[0]);
@@ -179,25 +176,46 @@ printf_oop (STREAM *stream,
   else if (OOP_CLASS (oop) == _gst_floatd_class)
     {
       double f = FLOATD_OOP_VALUE (oop);
-      stream_printf (stream, "%#.6g", f);
+      char buf[100], *p;
+      p = buf + sprintf (buf, "%#.*g",
+			 (_gst_regression_testing ? 6 : DBL_DIG), f);
+      for (; p >= buf; p--)
+        if (*p == 'e')
+	  {
+	    *p = 'd';
+	    break;
+	  }
+
+      stream_puts (buf, stream);
     }
 
   else if (OOP_CLASS (oop) == _gst_floate_class)
     {
       double f = FLOATE_OOP_VALUE (oop);
-      stream_printf (stream, "%#.6g", f);
+      stream_printf (stream, "%#.*g",
+		     (_gst_regression_testing ? 6 : FLT_DIG), f);
     }
 
   else if (OOP_CLASS (oop) == _gst_floatq_class)
     {
       long double f = FLOATQ_OOP_VALUE (oop);
-      stream_printf (stream, "%#.6Lg", f);
+      char buf[100], *p;
+      p = buf + sprintf (buf, "%#.*Lg",
+			 (_gst_regression_testing ? 6 : LDBL_DIG), f);
+      for (; p >= buf; p--)
+        if (*p == 'e')
+	  {
+	    *p = 'q';
+	    break;
+	  }
+
+      stream_puts (buf, stream);
     }
 
   else if (OOP_CLASS (oop) == _gst_symbol_class)
     {
       if (!info->alt)
-        stream_printf (stream, "#");
+	stream_printf (stream, "#");
       print_string_to_stream (stream, oop);
     }
 
@@ -217,10 +235,10 @@ printf_oop (STREAM *stream,
       if (IS_NIL (class_oop))
         print_oop_constructor_to_stream (stream, oop);
       else
-        {
-          print_class_name_to_stream (stream, class_oop);
-          stream_printf (stream, " class");
-        }
+	{
+	  print_class_name_to_stream (stream, class_oop);
+	  stream_printf (stream, " class");
+	}
     }
 
   else if (IS_A_CLASS (oop))
@@ -234,8 +252,8 @@ printf_oop (STREAM *stream,
 
 int
 printf_oop_arginfo (struct printf_info *info,
-                    size_t n,
-                    int *argtypes)
+		    size_t n,
+		    int *argtypes)
 {
   /* We always take exactly one argument and this is a pointer to the
      structure.. */
@@ -244,70 +262,100 @@ printf_oop_arginfo (struct printf_info *info,
   return 1;
 }
 
-
 
 void
 _gst_classify_addr (void *addr)
 {
-  if (IS_OOP_ADDR (addr))
-    display_oop (addr);
-
-  else if (IS_OBJ_ADDR (addr))
-    display_object (addr);
-
-  else if (IS_INT (addr))
+  if (IS_INT (addr))
     printf ("Smalltalk SmallInteger %ld\n", TO_INT (addr));
 
+  else if (IS_OOP_ADDR (addr))
+    _gst_display_oop (addr);
+
   else
-    printf ("Address %p is not a Smalltalk entity\n", addr);
+    _gst_display_object (addr);
 
   fflush (stdout);
 }
 
 void
-display_oop (OOP oop)
+_gst_display_oop_short (OOP oop)
 {
-  mst_Boolean isBuiltin;
+  if (oop->flags & F_FREE)
+    printf ("%-10p   Free\n", oop);
+  else
+    {
+      printf ("%-10p   %-10p  %-10s %-10s %-10s\n", oop, oop->object,
+            oop->flags & F_CONTEXT ? "Context" :
+            oop->flags & F_WEAK ? "Weak" :
+            oop->flags & F_EPHEMERON ? "Ephemeron" : "",
 
+            oop->flags & _gst_mem.active_flag ? "To-space" :
+            oop->flags & F_FIXED ? "Fixed" :
+            oop->flags & F_OLD ? "Old" : "From-space",
+
+	    IS_EDEN_ADDR (oop->object) ? "Eden" :
+	    IS_SURVIVOR_ADDR (oop->object, 0) ? "Surv (Even)" :
+	    IS_SURVIVOR_ADDR (oop->object, 1) ? "Surv (Odd)" :
+            oop->flags & F_POOLED ? "Pooled" :
+	    oop->flags & F_REACHABLE ? "Old/marked" : "Old");
+    }
+}
+
+void
+_gst_display_oop (OOP oop)
+{
   if (!IS_OOP_ADDR (oop))
     {
       printf ("Parameter %p does not appear to be an OOP!\n", oop);
       return;
     }
 
-  isBuiltin = (oop < _gst_oop_table);
-
-  if (!isBuiltin)
-    printf ("OOP %p [%ld]\n", oop,
-	    (unsigned long) (oop - _gst_oop_table));
-
   if (oop->flags & F_FREE)
-    printf ("Free ");
+    printf ("%-10p   Free\n", oop);
+  else
+    {
+      printf ("%-10p   %-10p  %-10s %-10s %-10s", oop, oop->object,
+            oop->flags & F_CONTEXT ? "Context" :
+            oop->flags & F_WEAK ? "Weak" :
+            oop->flags & F_EPHEMERON ? "Ephemeron" : "",
 
-  if (oop->flags & F_REACHABLE)
-    printf ("Reachable ");
+            oop->flags & _gst_mem.active_flag ? "To-space" :
+            oop->flags & F_FIXED ? "Fixed" :
+            oop->flags & F_OLD ? "Old" : "From-space",
 
-  printf ("   Empty bytes = %ld\n", (oop->flags & EMPTY_BYTES));
-  if (!(oop->flags & F_FREE))
-    _gst_print_object (oop);
+	    IS_EDEN_ADDR (oop->object) ? "Eden" :
+	    IS_SURVIVOR_ADDR (oop->object, 0) ? "Surv (Even)" :
+	    IS_SURVIVOR_ADDR (oop->object, 1) ? "Surv (Odd)" :
+            oop->flags & F_POOLED ? "Pooled" :
+	    oop->flags & F_REACHABLE ? "Old/marked" : "Old");
 
-  printf ("\n");
+      if (IS_OOP_ADDR (oop->object->objClass))
+        printf ("   %O (%O)\n", oop->object->objClass, oop->object->objSize);
+      else
+        printf ("   (invalid class)\n");
+    }
 }
 
-
 void
-display_object (mst_Object obj)
+_gst_display_object (mst_Object obj)
 {
-  if (!IS_OBJ_ADDR (obj))
+  if (IS_OOP_ADDR (obj))
     {
-      printf ("Parameter %p does not appear to be an object!\n", obj);
+      printf ("Parameter %p appears to be an OOP!\n", obj);
       return;
     }
 
-  printf ("Object at %p, ", obj);
-  printf ("Size %ld\n", NUM_OOPS (obj));
-  printf ("class %O", obj->objClass);
-  printf ("\n");
+  printf ("Object at %p (%s)",
+	  IS_EDEN_ADDR (obj) ? "Eden" :
+	  IS_SURVIVOR_ADDR (obj, 0) ? "Even" :
+	  IS_SURVIVOR_ADDR (obj, 1) ? "Odd" : "Old");
+
+  if (IS_OOP_ADDR (obj->objClass))
+    printf (", size %O (%ld OOPs), class %O\n",
+	    obj->objSize, NUM_OOPS (obj), obj->objClass);
+  else
+    printf (", contains invalid data\n");
 }
 
 
@@ -320,7 +368,7 @@ void _gst_init_snprintfv ()
   snv_realloc = xrealloc;
   snv_free = xfree;
   spec = register_printf_function ('O', printf_generic,
-                                   printf_oop_arginfo);
+				   printf_oop_arginfo);
 
   spec->user = printf_oop;
 }
