@@ -1,5 +1,5 @@
 /******************************** -*- C -*- ****************************
- *
+*
  *	Symbol Table module.
  *
  *
@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -30,16 +30,8 @@
  ***********************************************************************/
 
 
-#include "gst.h"
 #include "gstpriv.h"
 
-#include <stdio.h>
-#include <ctype.h>
-#if STDC_HEADERS
-#include <stdlib.h>
-#include <string.h>
-#endif /* STDC_HEADERS */
-
 typedef struct
 {
   OBJ_HEADER;
@@ -61,7 +53,7 @@ struct symbol_list
 /* Represents all the identifiers, both arguments and temporaries,
    which are declared in a given scope.  Nested scopes result in
    nested instances of the scope struct, with the current scope always
-   being the innermost at any point during the compilation. */
+   being the innermost at any point during the compilation.  */
 typedef struct scope *scope;
 struct scope
 {
@@ -75,7 +67,7 @@ struct scope
 typedef struct symbol_info
 {
   OOP *symbolVar;
-  char *value;
+  const char *value;
 }
 symbol_info;
 
@@ -83,7 +75,7 @@ symbol_info;
 OOP _gst_and_colon_symbol, _gst_at_colon_put_colon_symbol,
   _gst_at_colon_symbol, _gst_at_sign_symbol, _gst_at_end_symbol,
   _gst_bit_and_colon_symbol, _gst_bit_or_colon_symbol,
-  _gst_bit_shift_colon_symbol, _gst_block_copy_colon_symbol,
+  _gst_bit_shift_colon_symbol, _gst_bit_xor_colon_symbol,
   _gst_class_symbol, _gst_is_nil_symbol, _gst_not_nil_symbol,
   _gst_divide_symbol, _gst_do_colon_symbol, _gst_equal_symbol,
   _gst_greater_equal_symbol, _gst_greater_than_symbol,
@@ -95,11 +87,12 @@ OOP _gst_and_colon_symbol, _gst_at_colon_put_colon_symbol,
   _gst_not_equal_symbol, _gst_not_same_object_symbol,
   _gst_or_colon_symbol, _gst_plus_symbol, _gst_remainder_symbol,
   _gst_same_object_symbol, _gst_size_symbol, _gst_this_context_symbol,
-  _gst_finalize_symbol, _gst_times_symbol,
+  _gst_times_symbol, _gst_permission_colon_symbol,
   _gst_to_colon_do_colon_symbol, _gst_times_repeat_colon_symbol,
-  _gst_to_colon_by_colon_do_colon_symbol, _gst_uppercase_super_symbol,
+  _gst_to_colon_by_colon_do_colon_symbol,
   _gst_value_colon_symbol, _gst_value_colon_value_colon_symbol,
   _gst_value_colon_value_colon_value_colon_symbol,
+  _gst_value_with_rec_colon_with_args_colon_symbol,
   _gst_value_with_arguments_colon_symbol, _gst_while_false_symbol,
   _gst_value_symbol, _gst_yourself_symbol, _gst_while_true_symbol,
   _gst_while_false_colon_symbol, _gst_while_true_colon_symbol,
@@ -114,25 +107,29 @@ OOP _gst_and_colon_symbol, _gst_at_colon_put_colon_symbol,
   _gst_boolean_symbol, _gst_void_symbol, _gst_variadic_symbol,
   _gst_c_object_symbol, _gst_c_object_ptr_symbol, _gst_smalltalk_symbol,
   _gst_byte_array_symbol, _gst_byte_array_out_symbol,
-  _gst_undeclared_symbol, _gst_self_smalltalk_symbol,
+  _gst_undeclared_symbol, _gst_self_smalltalk_symbol, _gst_long_double_symbol,
   _gst_vm_primitives_symbol, _gst_variadic_smalltalk_symbol,
+  _gst_start_execution_colon_symbol, _gst_primitive_colon_symbol,
   _gst_symbol_table, _gst_current_namespace;
 
+/* The list of selectors for the send immediate bytecode.  */
+struct builtin_selector *_gst_builtin_selectors[256];
+
 /* True if undeclared globals can be considered forward references.  */
-long _gst_use_undeclared;
+int _gst_use_undeclared;
 
 /* Answer whether OOP is a Smalltalk String LEN characters long and
    these characters match the first LEN characters of STR (which must
-   not have embedded NULs). */
+   not have embedded NULs).  */
 static mst_Boolean is_same_string (const char *str,
 				   OOP oop,
 				   int len);
 
 /* Answer whether C is considered a white space character in Smalltalk
-   programs. */
-static mst_Boolean is_white_space (gst_uchar c);
+   programs.  */
+static mst_Boolean is_white_space (char c);
 
-/* Free the list of symbols declared in the given SCOPE. */
+/* Free the list of symbols declared in the given SCOPE.  */
 static void free_scope_symbols (scope scope);
 
 /* Scans a variable name (letters and digits, initial letter), and
@@ -141,11 +138,11 @@ static void free_scope_symbols (scope scope);
    whitespace or start of variable.  At end, it points to the first
    character after the initial whitespace, if any.  ENDP instead is
    initialized by the function to point to first character after the
-   parsed variable name, which may be NUL. */
-static void parse_variable_name (gst_uchar ** pp,
-				 gst_uchar ** endp);
+   parsed variable name, which may be NUL.  */
+static void parse_variable_name (const char ** pp,
+				 const char ** endp);
 
-/* This fills ENT's fields with the contents of its parameters. */
+/* This fills ENT's fields with the contents of its parameters.  */
 static void fill_symbol_entry (symbol_entry * ent,
 			       scope_type scope,
 			       mst_Boolean readOnly,
@@ -158,35 +155,41 @@ static void fill_symbol_entry (symbol_entry * ent,
    of the string to be scanned.  May be pointing at either whitespace
    or start of variable.  At end, points to first character after the
    parsed variable name, which may be NUL.  The output is a Smalltalk
-   Symbol, _gst_nil_oop if no variable name is found. */
-static OOP scan_name (gst_uchar ** pp);
+   Symbol, _gst_nil_oop if no variable name is found.  */
+static OOP scan_name (const char ** pp);
 
 /* This creates a Symbol containing LEN bytes starting at STR and puts
    it in the symbol list, or returns an existing one if it is
-   found. */
+   found.  */
 static OOP intern_counted_string (const char *str,
 				  int len);
 
 /* This looks for SYMBOL among the instance variables that the current
    class declares, and returns the index of the variable if one is
-   found. */
+   found.  */
 static int instance_variable_index (OOP symbol);
 
+/* This checks if the INDEX-th instance variable among those that the
+   current class declares is read-only.  Read-only index variables are
+   those that are declared by a trusted super-class of an untrusted
+   subclass.  */
+static mst_Boolean is_instance_variable_read_only (int index);
+
 /* This looks for SYMBOL among the arguments and temporary variables
    that the current scope sees, and returns the entry in the symbol
-   list for the variable if it is found. */
+   list for the variable if it is found.  */
 static symbol_list find_local_var (scope scope,
 				   OOP symbol);
 
 /* This looks for SYMBOL among the global variables that the current
    scope sees, including superspaces if any, and returns the entry in
-   the symbol list for the variable if it is found. */
+   the symbol list for the variable if it is found.  */
 static OOP find_class_variable (OOP varName);
 
 static scope cur_scope = NULL;
 
 /* This is an array of symbols which the virtual machine knows about,
-   and is used to restore the global variables upon image load. */
+   and is used to restore the global variables upon image load.  */
 static const symbol_info sym_info[] = {
   {&_gst_and_colon_symbol, "and:"},
   {&_gst_as_scaled_decimal_scale_symbol, "asScaledDecimal:scale:"},
@@ -197,8 +200,8 @@ static const symbol_info sym_info[] = {
   {&_gst_bad_return_error_symbol, "badReturnError"},
   {&_gst_bit_and_colon_symbol, "bitAnd:"},
   {&_gst_bit_or_colon_symbol, "bitOr:"},
+  {&_gst_bit_xor_colon_symbol, "bitXor:"},
   {&_gst_bit_shift_colon_symbol, "bitShift:"},
-  {&_gst_block_copy_colon_symbol, "blockCopy:"},
   {&_gst_byte_array_symbol, "byteArray"},
   {&_gst_byte_array_out_symbol, "byteArrayOut"},
   {&_gst_boolean_symbol, "boolean"},
@@ -213,7 +216,6 @@ static const symbol_info sym_info[] = {
   {&_gst_double_symbol, "double"},
   {&_gst_equal_symbol, "="},
   {&_gst_false_symbol, "false"},
-  {&_gst_finalize_symbol, "finalize"},
   {&_gst_greater_equal_symbol, ">="},
   {&_gst_greater_than_symbol, ">"},
   {&_gst_if_false_colon_if_true_colon_symbol, "ifFalse:ifTrue:"},
@@ -226,6 +228,7 @@ static const symbol_info sym_info[] = {
   {&_gst_is_nil_symbol, "isNil"},
   {&_gst_less_equal_symbol, "<="},
   {&_gst_less_than_symbol, "<"},
+  {&_gst_long_double_symbol, "longDouble"},
   {&_gst_long_symbol, "long"},
   {&_gst_ulong_symbol, "uLong"},
   {&_gst_minus_symbol, "-"},
@@ -239,14 +242,17 @@ static const symbol_info sym_info[] = {
   {&_gst_not_nil_symbol, "notNil"},
   {&_gst_not_same_object_symbol, "~~"},
   {&_gst_or_colon_symbol, "or:"},
+  {&_gst_permission_colon_symbol, "permission:"},
   {&_gst_plus_symbol, "+"},
-  {&_gst_remainder_symbol, "\\"},
+  {&_gst_primitive_colon_symbol, "primitive:"},
+  {&_gst_remainder_symbol, "\\\\"},
   {&_gst_repeat_symbol, "repeat"},
   {&_gst_same_object_symbol, "=="},
   {&_gst_self_symbol, "self"},
   {&_gst_self_smalltalk_symbol, "selfSmalltalk"},
   {&_gst_size_symbol, "size"},
   {&_gst_smalltalk_symbol, "smalltalk"},
+  {&_gst_start_execution_colon_symbol, "startExecution:"},
   {&_gst_string_out_symbol, "stringOut"},
   {&_gst_string_symbol, "string"},
   {&_gst_super_symbol, "super"},
@@ -260,11 +266,12 @@ static const symbol_info sym_info[] = {
   {&_gst_true_symbol, "true"},
   {&_gst_undeclared_symbol, "Undeclared"},
   {&_gst_unknown_symbol, "unknown"},
-  {&_gst_uppercase_super_symbol, "Super"},
   {&_gst_value_colon_symbol, "value:"},
   {&_gst_value_colon_value_colon_symbol, "value:value:"},
   {&_gst_value_colon_value_colon_value_colon_symbol,
    "value:value:value:"},
+  {&_gst_value_with_rec_colon_with_args_colon_symbol,
+   "valueWithReceiver:withArguments:"},
   {&_gst_value_symbol, "value"},
   {&_gst_value_with_arguments_colon_symbol, "valueWithArguments:"},
   {&_gst_variadic_symbol, "variadic"},
@@ -280,7 +287,18 @@ static const symbol_info sym_info[] = {
 };
 
 
-
+const char *
+_gst_get_scope_kind (scope_type scope)
+{
+  switch (scope)
+    {
+    case SCOPE_TEMPORARY: return "argument";
+    case SCOPE_RECEIVER: return "instance variable";
+    case SCOPE_GLOBAL: return "global variable";
+    case SCOPE_SPECIAL: return "special variable";
+    default: abort ();
+    }
+}
 
 int
 _gst_get_arg_count (void)
@@ -377,12 +395,12 @@ _gst_undeclare_name (void)
 
   oldList = cur_scope->symbols;
   cur_scope->symbols = cur_scope->symbols->prevSymbol;
-  free (oldList);
+  xfree (oldList);
 }
 
 
 int
-_gst_declare_name (char *name,
+_gst_declare_name (const char *name,
 		   mst_Boolean writeable)
 {
   symbol_list newList;
@@ -395,7 +413,7 @@ _gst_declare_name (char *name,
 
   /* Arguments are always declared first, so we can assume it is a
      temporary -- if it is not, _gst_declare_arguments and
-     _gst_declare_block_arguments will fix it. */
+     _gst_declare_block_arguments will fix it.  */
   cur_scope->numTemporaries++;
   cur_scope->symbols = newList;
   return (newList->index);
@@ -416,6 +434,19 @@ free_scope_symbols (scope scope)
 }
 
 
+OOP
+_gst_get_class_object (OOP classOOP)
+{
+  if (OOP_CLASS (classOOP) == _gst_metaclass_class)
+    classOOP = METACLASS_INSTANCE (classOOP);
+
+  while (OOP_CLASS (classOOP) == _gst_behavior_class
+	 || OOP_CLASS (classOOP) == _gst_class_description_class)
+    classOOP = SUPERCLASS (classOOP);
+
+  return classOOP;
+}
+
 OOP
 find_class_variable (OOP varName)
 {
@@ -424,20 +455,7 @@ find_class_variable (OOP varName)
   int numPools, i;
   gst_class class;
 
-  myClass = _gst_this_class;
-  if (OOP_CLASS (myClass) == _gst_metaclass_class)
-    {
-      /* pretend that metaclasses have the class variables and shared
-         pools that their instance classes do */
-      myClass = METACLASS_INSTANCE (myClass);
-    }
-  while (OOP_CLASS (myClass) == _gst_behavior_class
-	 || OOP_CLASS (myClass) == _gst_class_description_class)
-    {
-      /* pretend that lightweight classes have the class variables and
-         shared pools that their superclasses have */
-      myClass = SUPERCLASS (myClass);
-    }
+  myClass = _gst_get_class_object (_gst_this_class);
 
   /* Now search in the class pools */
   for (class_oop = myClass; !IS_NIL (class_oop);
@@ -485,7 +503,7 @@ find_class_variable (OOP varName)
 OOP
 _gst_find_variable_binding (tree_node list)
 {
-  OOP symbol, root, assocOOP;
+  OOP symbol, root, assocOOP, undeclaredDictionary;
 
   symbol = _gst_intern_string (list->v_list.name);
   assocOOP = find_class_variable (symbol);
@@ -497,20 +515,47 @@ _gst_find_variable_binding (tree_node list)
       assocOOP = _gst_find_shared_pool_variable (root, symbol);
     }
 
+  if (IS_NIL (assocOOP) && !(list->v_list.next))
+    {
+      char *varName;
+
+      varName = STRING_OOP_CHARS (symbol);
+      if (!isupper (*varName) || !_gst_use_undeclared)
+	return (assocOOP);
+
+      undeclaredDictionary =
+	dictionary_at (_gst_smalltalk_dictionary,
+		       _gst_undeclared_symbol);
+
+      assocOOP =
+	dictionary_association_at (undeclaredDictionary, symbol);
+
+      if (IS_NIL (assocOOP))
+	{
+	  assocOOP =
+	    NAMESPACE_AT_PUT (undeclaredDictionary, symbol,
+			      _gst_nil_oop);
+
+	  MAKE_OOP_UNTRUSTED (assocOOP, IS_OOP_UNTRUSTED (_gst_this_class));
+	}
+    }
+
   return (assocOOP);
 }
 
 
 mst_Boolean
 _gst_find_variable (symbol_entry * se,
-		    OOP symbol)
+		    tree_node list)
 {
-  OOP varAssoc, undeclaredDictionary;
+  OOP varAssoc;
   int index;
   unsigned int scopeDistance;
   scope scope;
   symbol_list s;
+  OOP symbol;
 
+  symbol = _gst_intern_string (list->v_list.name);
   if (symbol == _gst_self_symbol || symbol == _gst_super_symbol)
     {
       fill_symbol_entry (se, SCOPE_SPECIAL, true, symbol, RECEIVER_INDEX,
@@ -556,54 +601,54 @@ _gst_find_variable (symbol_entry * se,
   index = instance_variable_index (symbol);
   if (index >= 0)
     {
-      fill_symbol_entry (se, SCOPE_RECEIVER, false, symbol, index, 0);
+      fill_symbol_entry (se, SCOPE_RECEIVER, 
+			 is_instance_variable_read_only (index),
+			 symbol, index, 0);
       return (true);
     }
 
-  varAssoc = find_class_variable (symbol);
+  varAssoc = _gst_find_variable_binding (list);
   if (IS_NIL (varAssoc))
-    {
-      gst_uchar *varName;
-
-      varName = STRING_OOP_CHARS (symbol);
-      if (!isupper (*varName) || !_gst_use_undeclared)
-	return (false);
-
-      undeclaredDictionary =
-	dictionary_at (_gst_smalltalk_dictionary,
-		       _gst_undeclared_symbol);
-
-      varAssoc =
-	dictionary_association_at (undeclaredDictionary, symbol);
-      if (IS_NIL (varAssoc))
-	varAssoc =
-	  DICTIONARY_AT_PUT (undeclaredDictionary, symbol,
-			     _gst_nil_oop);
-    }
+    return (false);
 
   index = _gst_add_forced_object (varAssoc);
 
-  fill_symbol_entry (se, SCOPE_GLOBAL, false, varAssoc, index, 0);
+  fill_symbol_entry (se, SCOPE_GLOBAL, 
+		     ((IS_OOP_UNTRUSTED (_gst_this_class)
+		       || IS_OOP_UNTRUSTED (_gst_this_context_oop))
+		      && !IS_OOP_UNTRUSTED (varAssoc)),
+		     varAssoc, index, 0);
   return (true);
 }
 
+static mst_Boolean
+is_instance_variable_read_only (int index)
+{
+  int numVars;
+  OOP class_oop;
+
+  if (!IS_OOP_UNTRUSTED (_gst_this_class)
+      && !IS_OOP_UNTRUSTED (_gst_this_context_oop))
+    return (false);
+
+  for (class_oop = _gst_this_class; IS_OOP_UNTRUSTED (class_oop);
+       class_oop = SUPERCLASS (class_oop))
+    ;
+
+  numVars = CLASS_FIXED_FIELDS (class_oop);
+  return index + 1 <= numVars;
+}
+
 static int
 instance_variable_index (OOP symbol)
 {
   OOP arrayOOP;
   int index, numVars;
 
-
-  if (OOP_CLASS (_gst_this_class) == _gst_behavior_class)
-    {
-      /* behaviors have no instance variables */
-      return (-1);
-    }
-
   arrayOOP = _gst_instance_variable_array (_gst_this_class);
   numVars = NUM_OOPS (OOP_TO_OBJ (arrayOOP));
 
-  for (index = 1; index <= numVars; index++)
+  for (index = numVars; index >= 1; index--)
     if (ARRAY_AT (arrayOOP, index) == symbol)
       return (index - 1);
 
@@ -669,16 +714,16 @@ _gst_print_symbol_entry (symbol_entry * ent)
 
 OOP
 _gst_make_instance_variable_array (OOP superclassOOP,
-				   gst_uchar * variableString)
+				   const char * variableString)
 {
   OOP arrayOOP, superArrayOOP, name;
   int index, numInstanceVars, superInstanceVars;
-  gst_uchar *p;
+  const char *p;
   inc_ptr incPtr;
   mst_Object array;
 
   if (variableString == NULL)
-    variableString = (gst_uchar *) "";
+    variableString = "";
 
   if (IS_NIL (superclassOOP))
     {
@@ -730,10 +775,11 @@ _gst_make_instance_variable_array (OOP superclassOOP,
 }
 
 OOP
-_gst_make_class_variable_dictionary (gst_uchar * variableNames)
+_gst_make_class_variable_dictionary (const char *variableNames,
+				     OOP classOOP)
 {
   OOP dictionaryOOP, name;
-  gst_uchar *p;
+  const char *p;
   inc_ptr incPtr;
 
   if (variableNames == NULL)
@@ -741,39 +787,40 @@ _gst_make_class_variable_dictionary (gst_uchar * variableNames)
 
   incPtr = INC_SAVE_POINTER ();
 
-  dictionaryOOP = _gst_dictionary_new (13);
-  INC_ADD_OOP (dictionaryOOP);
-
+  dictionaryOOP = _gst_nil_oop;
   for (p = variableNames; *p;)
     {
       name = scan_name (&p);
       if (!IS_NIL (name))
 	{
+	  if (IS_NIL (dictionaryOOP))
+	    {
+	      dictionaryOOP = _gst_binding_dictionary_new (8, classOOP);
+	      INC_ADD_OOP (dictionaryOOP);
+	    }
+
 	  /* ### error if already exists */
 	  /* don't need to add name to incubator -- it's a symbol so
 	     it's already held onto */
-	  DICTIONARY_AT_PUT (dictionaryOOP, name, _gst_nil_oop);
+	  NAMESPACE_AT_PUT (dictionaryOOP, name, _gst_nil_oop);
 	}
     }
 
-  if (DICTIONARY_SIZE (dictionaryOOP) == 0)
-    dictionaryOOP = _gst_nil_oop;
-
   INC_RESTORE_POINTER (incPtr);
   return (dictionaryOOP);
 }
 
 OOP
-_gst_make_pool_array (gst_uchar * poolNames)
+_gst_make_pool_array (const char * poolNames)
 {
   OOP poolsOOP, name;
   mst_Object pools;
   int numPools, i;
-  gst_uchar *p, *e;
+  const char *p, *e;
   inc_ptr incPtr;
 
   if (poolNames == NULL)
-    poolNames = (gst_uchar *) "";
+    poolNames = (char *) "";
 
   /* count the number of new pool names */
   for (p = poolNames, numPools = 0; *p;)
@@ -797,7 +844,7 @@ _gst_make_pool_array (gst_uchar * poolNames)
       if (!IS_NIL (name))
 	{
 	  /* don't need to add name to incubator -- it's a symbol so
-	     it's already held onto. */
+	     it's already held onto.  */
 
 	  /* ### error if already exists in parent?, or if value isn't
 	     a dictionary */
@@ -824,17 +871,18 @@ _gst_make_pool_array (gst_uchar * poolNames)
 
 
 static OOP
-scan_name (gst_uchar ** pp)
+scan_name (const char ** pp)
 {
-  gst_uchar *end, *str;
-  long len;
+  const char *end;
+  char *str;
+  size_t len;
 
   parse_variable_name (pp, &end);
   len = end - *pp;
   if (len == 0)
     return (_gst_nil_oop);
 
-  str = (gst_uchar *) alloca (len + 1);
+  str = (char *) alloca (len + 1);
   strncpy (str, *pp, len);
   str[len] = '\0';
 
@@ -844,10 +892,10 @@ scan_name (gst_uchar ** pp)
 }
 
 static void
-parse_variable_name (gst_uchar ** pp,
-		     gst_uchar ** endp)
+parse_variable_name (const char ** pp,
+		     const char ** endp)
 {
-  gst_uchar *p, *e;
+  const char *p, *e;
 
   p = *pp;
   while (is_white_space (*p))
@@ -869,7 +917,7 @@ parse_variable_name (gst_uchar ** pp,
 }
 
 static mst_Boolean
-is_white_space (gst_uchar c)
+is_white_space (char c)
 {
   return (c == ' ' || c == '\r' || c == '\t' || c == '\n' || c == '\f');
 }
@@ -891,7 +939,7 @@ _gst_intern_string_oop (OOP stringOOP)
      of the stringOOP.  intern_counted_string can do allocations.  If
      it allocates, and the gc runs, stringOOP can move, meaning the
      dereferenced set of chars becomes invalid.  So instead we make a
-     non-moving copy and use that. */
+     non-moving copy and use that.  */
   if (len < sizeof (copyBuf))
     copyPtr = copyBuf;
   else
@@ -920,7 +968,7 @@ static OOP
 intern_counted_string (const char *str,
 		       int len)
 {
-  unsigned long index;
+  uintptr_t index;
   sym_link link;
   gst_symbol symbol;
   OOP symbolOOP, linkOOP;
@@ -944,7 +992,7 @@ intern_counted_string (const char *str,
 
   incPtr = INC_SAVE_POINTER ();
   symbol = (gst_symbol) new_instance_with (_gst_symbol_class, 
-					   (long) len, &symbolOOP);
+					   len, &symbolOOP);
 
   memcpy (symbol->symString, str, len);
   symbolOOP->flags |= F_READONLY;
@@ -956,7 +1004,6 @@ intern_counted_string (const char *str,
   ARRAY_AT_PUT (_gst_symbol_table, index, linkOOP);
 
   INC_RESTORE_POINTER (incPtr);
-  /* printf ("%d %x %O\n", index, OOP_INDEX(symbolOOP), symbolOOP); */
   return (symbolOOP);
 }
 
@@ -980,11 +1027,11 @@ _gst_string_oop_len (OOP oop)
   return (OOP_SIZE_BYTES (oop) - (oop->flags & EMPTY_BYTES));
 }
 
-unsigned long
+uintptr_t
 _gst_hash_string (const char *str,
 		  int len)
 {
-  unsigned long hashVal = 1497032417;    /* arbitrary value */
+  uintptr_t hashVal = 1497032417;    /* arbitrary value */
 
   while (len--)
     {
@@ -1021,7 +1068,7 @@ _gst_symbol_as_string (OOP symbolOOP)
 void
 _gst_check_symbol_chain (void)
 {
-  unsigned long i;
+  int i;
 
   for (i = 1; i <= SYMBOL_TABLE_SIZE; i++)
     {
@@ -1035,7 +1082,7 @@ _gst_check_symbol_chain (void)
 	      OOP_CLASS (link->symbol) != _gst_symbol_class)
 	    {
 	      printf ("Bad symbol %p\n", linkOOP);
-	      _gst_debug ();
+	      abort ();
 	    }
 	}
     }
@@ -1045,7 +1092,7 @@ _gst_check_symbol_chain (void)
 void
 _gst_add_all_symbol_completions (void)
 {
-  unsigned long i;
+  int i;
 
   for (i = 1; i <= SYMBOL_TABLE_SIZE; i++)
     {
@@ -1060,7 +1107,7 @@ _gst_add_all_symbol_completions (void)
 	  string = _gst_to_cstring (link->symbol);
 	  len = _gst_string_oop_len (link->symbol);
 	  _gst_add_symbol_completion (string, len);
-	  free (string);
+	  xfree (string);
 	}
     }
 }
@@ -1071,11 +1118,11 @@ _gst_add_all_symbol_completions (void)
 int
 _gst_selector_num_args (OOP symbolOOP)
 {
-  gst_uchar *bytes;
+  char *bytes;
   int numArgs, len;
 
   len = _gst_string_oop_len (symbolOOP);
-  bytes = (gst_uchar *) (OOP_TO_OBJ (symbolOOP)->data);
+  bytes = (char *) (OOP_TO_OBJ (symbolOOP)->data);
   if (bytes[0] < 'A' || bytes[0] > 'z')
     return (1);
 
@@ -1092,13 +1139,27 @@ _gst_selector_num_args (OOP symbolOOP)
   return (numArgs);
 }
 
+#include "builtins.inl"
 
 void
 _gst_init_symbols (void)
 {
   const symbol_info *si;
+  struct builtin_selector *bs;
 
   for (si = sym_info; si->symbolVar; si++)
     *si->symbolVar = _gst_intern_string (si->value);
-}
 
+  /* Complete gperf's generated table with each symbol's OOP,
+     and prepare a kind of reverse mapping from the 256 bytecodes
+     to the hash table entries.  */
+  for (bs = _gst_builtin_selectors_hash;
+       bs - _gst_builtin_selectors_hash <
+	 sizeof (_gst_builtin_selectors_hash) / sizeof (_gst_builtin_selectors_hash[0]);
+       bs++)
+    if (bs->name)
+      {
+	bs->symbol = _gst_intern_string (bs->name);
+        _gst_builtin_selectors[bs->bytecode] = bs;
+      }
+}
