@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -80,9 +80,9 @@ extern OOP _gst_and_colon_symbol, _gst_at_colon_put_colon_symbol,
   _gst_not_equal_symbol, _gst_not_same_object_symbol,
   _gst_or_colon_symbol, _gst_plus_symbol, _gst_remainder_symbol,
   _gst_same_object_symbol, _gst_size_symbol, _gst_this_context_symbol,
-  _gst_finalize_symbol, _gst_times_symbol,
+  _gst_mourn_symbol, _gst_times_symbol,
   _gst_to_colon_do_colon_symbol, _gst_times_repeat_colon_symbol,
-  _gst_to_colon_by_colon_do_colon_symbol, _gst_uppercase_super_symbol,
+  _gst_to_colon_by_colon_do_colon_symbol,
   _gst_value_colon_symbol, _gst_value_colon_value_colon_symbol,
   _gst_value_colon_value_colon_value_colon_symbol,
   _gst_value_with_arguments_colon_symbol, _gst_while_false_symbol,
@@ -101,13 +101,16 @@ extern OOP _gst_and_colon_symbol, _gst_at_colon_put_colon_symbol,
   _gst_byte_array_symbol, _gst_byte_array_out_symbol,
   _gst_undeclared_symbol, _gst_self_smalltalk_symbol,
   _gst_vm_primitives_symbol, _gst_variadic_smalltalk_symbol,
-  _gst_symbol_table, _gst_current_namespace;
+  _gst_start_execution_colon_symbol, _gst_symbol_table,
+  _gst_current_namespace;
 
 /* This walks the list of scopes and of symbols defined for each scope,
-   looking for a variable named SYMBOL.  Then it looks in the instance
-   variables, then in the class variables, and then in the pool dictionaries
-   (starting from those declared in the class and going up in the hierarchy).
-   If the SYMBOL is not found anywhere but starts with an uppercase letter,
+   looking for a variable represented by the tree LIST.  Then it looks in
+   the instance variables, then in the class variables, and then in the
+   pool dictionaries (starting from those declared in the class and going
+   up in the hierarchy).
+
+   If the variable is not found anywhere but starts with an uppercase letter,
    it is declared in the Undeclared dictionary and the methods will be fixed
    automatically as soon as it is defined (if it is).
 
@@ -115,7 +118,7 @@ extern OOP _gst_and_colon_symbol, _gst_at_colon_put_colon_symbol,
    variable if it is found or it is deemed part of Undeclared.  Else,
    SE is untouched and FALSE is returned. */
 extern mst_Boolean _gst_find_variable (symbol_entry * se,
-				       OOP symbol) FN_PURE;
+				       tree_node list) FN_PURE;
 
 /* This converts a C string to a symbol and stores it in the symbol
    table. */
@@ -127,9 +130,11 @@ extern OOP _gst_intern_string (const char *str) FN_PURE;
 extern OOP _gst_make_instance_variable_array (OOP superclassOOP,
 					      gst_uchar * variableString);
 
-/* This makes a Dictionary whose keys are the class variables declared
-   in VARIABLENAMES. */
-extern OOP _gst_make_class_variable_dictionary (gst_uchar * variableNames);
+/* This makes a BindingDictionary whose keys are the class variables
+   declared in VARIABLENAMES.  The environment of the dictionary is
+   classOOP.  */
+extern OOP _gst_make_class_variable_dictionary (gst_uchar * variableNames,
+						OOP classOOP);
 
 /* This makes an Array whose elements are the pool dictionaries
    declared in POOLNAMES. */
@@ -190,7 +195,7 @@ extern int _gst_declare_block_arguments (tree_node args);
    is created and links a new entry inside the symbol list for the
    currently active scope.  Returns the index of the variable into the
    stack frame. */
-extern int _gst_declare_name (char *name,
+extern int _gst_declare_name (const char *name,
 			      mst_Boolean writeable);
 
 /* Computes the number of selectors that a message named SYMBOLOOP
