@@ -37,37 +37,15 @@
 # define _BSD_SIGNALS
 #endif
 
-#include <stdio.h>
-#include <signal.h>
-#include <fcntl.h>
-#include <unistd.h>
-
 #ifndef SEEK_SET
 #define SEEK_SET                0
 #define SEEK_CUR                1
 #define SEEK_END                2
 #endif
 
-#ifndef SIGALRM
-#define SIGALRM		SIGUSR1	/* hmmm... */
-#endif
-#ifndef SIGVTALRM
-#define SIGVTALRM	SIGUSR2	/* hmmm... */
-#endif
-
 #define TIMER_REAL       SIGALRM
 #define TIMER_PROCESS    SIGVTALRM
 
-#ifdef SIG_BLOCK
-typedef sigset_t *int_state;
-#else
-#ifdef INTERRUPT_STATE_TYPE
-typedef INTERRUPT_STATE_TYPE int_state;
-#else
-typedef int int_state;
-#endif
-#endif
-
 #ifndef  O_NONBLOCK
 #ifdef  O_NDELAY
 #define O_NONBLOCK O_NDELAY
@@ -81,6 +59,13 @@ typedef int int_state;
 #endif
 #endif
 
+#ifndef HAVE_SYS_MMAN_H
+#define PROT_READ       0x1             /* Page can be read.  */
+#define PROT_WRITE      0x2             /* Page can be written.  */
+#define PROT_EXEC       0x4             /* Page can be executed.  */
+#define PROT_NONE       0x0             /* Page can not be accessed.  */
+#endif
+
 /* These nice tests are simple, guaranteed and independent of byte order. */
 #define IS_FINITE(dbl)		(!IS_NAN((dbl) * 0.0))
 #define IS_NAN(dbl)		 ((dbl) != (dbl))
@@ -88,14 +73,12 @@ typedef int int_state;
 typedef RETSIGTYPE (*SigHandler) ();
 
 /* Saves and returns the current state of the software interrupt
-   system.  Disables all interrupts.  Returns the old state of the
-   interrupt system, presumably for saving for a later call to
-   _gst_enable_interrupts. */
-extern int_state _gst_disable_interrupts (void);
+   system.  Disables all interrupts. */
+extern void _gst_disable_interrupts (void);
 
 /* Restores the state of the interrupt system to that which it had when
-   "mask" was created by _gst_disable_interrupts. */
-extern void _gst_enable_interrupts (int_state newSet);
+   _gst_disable_interrupts was last invoked. */
+extern void _gst_enable_interrupts (void);
 
 /* Set the signal handler for signal SIGNUM to be HANDLERFUNC. */
 extern SigHandler _gst_set_signal_handler (int signum,
@@ -155,6 +138,12 @@ extern char *_gst_get_cur_dir_name (void);
 /* Returns the name of the current time zone. */
 extern char *_gst_current_time_zone_name (void);
 
+/* Portable implementation of mprotect.  The implementation does not
+   necessarily support PROT_EXEC or PROT_WRITE without PROT_READ.  The
+   only guarantees are that no writing will be allowed without
+   PROT_WRITE and no access will be allowed for PROT_NONE. */
+extern int _gst_mem_protect (PTR addr, unsigned long len, int prot);
+
 /* Returns true if the file named FILENAME exists has the given
    permission for the current user.  Returns false otherwise. */
 extern mst_Boolean _gst_file_is_readable (const char *fileName);
@@ -174,13 +163,55 @@ extern int _gst_open_pipe (const char *command,
 extern int _gst_open_file (const char *filename,
 			   const char *mode);
 
+/* Read SIZE bytes into BUFFER from the file descriptor, FD. */
+extern int _gst_read (int fd,
+		      PTR buffer,
+		      int size);
+
 /* Ensures that exactly SIZE bytes of BUFFER are written into the
    file descriptor, FD. */
 extern int _gst_full_write (int fd,
-			    PTR buffer,
-			    int size);
+		            PTR buffer,
+		            int size);
+
+/* Write SIZE bytes of BUFFER into the file descriptor, FD. */
+extern int _gst_write (int fd,
+		       PTR buffer,
+		       int size);
+
+/* Read SIZE bytes into BUFFER from the file descriptor, FD. */
+extern int _gst_recv (int fd,
+		      PTR buffer,
+		      int size,
+		      int flags);
+
+/* Write SIZE bytes of BUFFER into the socket descriptor, FD. */
+extern int _gst_send (int fd,
+		      PTR buffer,
+		      int size,
+		      int flags);
 
 /* Writes a debug message with the given formatting. */
-extern void _gst_debugf (const char *, ...) FN_PRINTF;
+extern void _gst_debugf (const char *, ...) FN_PRINTF (1);
+
+/* Do an anonymous memory map of SIZE bytes.  */
+extern PTR _gst_osmem_alloc (size_t size);
+
+/* Unmap the SIZE bytes at PTR.  */
+extern void _gst_osmem_free (PTR ptr, size_t size);
+
+/* Reserve SIZE bytes of the address space without allocating them.  */
+extern PTR _gst_osmem_reserve (size_t size);
+
+/* Release SIZE bytes of the address space starting from BASE.  */
+extern void _gst_osmem_release (PTR base, size_t size);
+
+/* Actually allocate SIZE bytes of address space, starting from BASE,
+   that have already been reserved.  */
+extern PTR _gst_osmem_commit (PTR base, size_t size);
+
+/* Free the memory used by SIZE bytes of address space, starting from BASE,
+   but keep the addresses reserved.  */
+extern void _gst_osmem_decommit (PTR base, size_t size);
 
 #endif /* GST_SYSDEP_H */
