@@ -40,20 +40,23 @@ char *_gst_nil_name = "(nil)";	/* how to print nil */
 
 /* Make a tree_node made up of the NODETYPE type-tag and a list_node
    representing the head of the list, for NAME and VALUE.  */
-static tree_node make_list_node (node_type nodeType,
+static tree_node make_list_node (YYLTYPE *location,
+				 node_type nodeType,
 				 char *name,
 				 tree_node value);
 
 /* Make a expr_node made up of the NODETYPE type-tag and an expr_node
    with given RECEIVER, SELECTOR and EXPRESSION.  */
-static tree_node make_expr_node (node_type nodeType,
+static tree_node make_expr_node (YYLTYPE *location,
+				 node_type nodeType,
 				 tree_node receiver,
 				 OOP selector,
 				 tree_node expression);
 
 /* Allocate from the compilation obstack a node and assign it the
    NODETYPE type-tag.  */
-static tree_node make_tree_node (node_type nodeType);
+static tree_node make_tree_node (YYLTYPE *location,
+				 node_type nodeType);
 
 /* Print the NODE method_node with LEVEL spaces of indentation. */
 static void print_method_node (tree_node node,
@@ -81,10 +84,6 @@ static void print_const_node (tree_node node,
 static void print_array_constructor_node (tree_node node,
 					  int level);
 
-/* Print the node type of the NODE, with LEVEL spaces of indentation. */
-static void print_node_type (tree_node node,
-			     int level);
-
 /* Print LEVEL spaces of indentation. */
 static void indent (int level);
 
@@ -97,37 +96,40 @@ static char *node_type_names[] = {
   "TREE_KEYWORD_EXPR",		/* TREE_KEYWORD_EXPR */
   "TREE_VARIABLE_NODE",		/* TREE_VARIABLE_NODE */
   "TREE_KEYWORD_LIST",		/* TREE_KEYWORD_LIST */
-  "TREE_VARIABLE_LIST",		/* TREE_VARIABLE_LIST */
-  "TREE_STATEMENT_LIST",		/* TREE_STATEMENT_LIST */
+  "TREE_VAR_DECL_LIST",		/* TREE_VAR_DECL_LIST */
+  "TREE_VAR_ASSIGN_LIST",	/* TREE_VAR_ASSIGN_LIST */
+  "TREE_STATEMENT_LIST",	/* TREE_STATEMENT_LIST */
   "TREE_RETURN_EXPR",		/* TREE_RETURN_EXPR */
   "TREE_ASSIGN_EXPR",		/* TREE_ASSIGN_EXPR */
   "TREE_CONST_EXPR",		/* TREE_CONST_EXPR */
   "TREE_SYMBOL_NODE",		/* TREE_SYMBOL_NODE */
-  "TREE_ARRAY_ELT_LIST",		/* TREE_ARRAY_ELT_LIST */
+  "TREE_ARRAY_ELT_LIST",	/* TREE_ARRAY_ELT_LIST */
   "TREE_BLOCK_NODE",		/* TREE_BLOCK_NODE */
-  "TREE_CASCADE_EXPR",	/* TREE_CASCADE_EXPR */
-  "TREE_MESSAGE_LIST"		/* TREE_MESSAGE_LIST */
-    "TREE_ARRAY_CONSTRUCTOR"	/* TREE_ARRAY_CONSTRUCTOR */
+  "TREE_CASCADE_EXPR",		/* TREE_CASCADE_EXPR */
+  "TREE_MESSAGE_LIST",		/* TREE_MESSAGE_LIST */
+  "TREE_ARRAY_CONSTRUCTOR"	/* TREE_ARRAY_CONSTRUCTOR */
 };
 
 
 
 tree_node
-_gst_make_array_elt (tree_node elt)
+_gst_make_array_elt (YYLTYPE *location,
+		     tree_node elt)
 {
-  return (make_list_node (TREE_ARRAY_ELT_LIST, NULL, elt));
+  return (make_list_node (location, TREE_ARRAY_ELT_LIST, NULL, elt));
 }
 
 
 tree_node
-_gst_make_method (tree_node selectorExpr,
+_gst_make_method (YYLTYPE *location,
+		     tree_node selectorExpr,
 		  tree_node temporaries,
 		  char *primitiveName,
 		  tree_node statements)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_METHOD_NODE);
+  result = make_tree_node (location, TREE_METHOD_NODE);
   result->v_method.selectorExpr = selectorExpr;
   result->v_method.temporaries = temporaries;
   result->v_method.primitiveName = primitiveName;
@@ -137,16 +139,18 @@ _gst_make_method (tree_node selectorExpr,
 
 
 tree_node
-_gst_make_cascaded_message (tree_node messageExpr,
+_gst_make_cascaded_message (YYLTYPE *location,
+		     tree_node messageExpr,
 			    tree_node cascadedMessages)
 {
-  return (make_expr_node (TREE_CASCADE_EXPR, messageExpr, NULL,
+  return (make_expr_node (location, TREE_CASCADE_EXPR, messageExpr, NULL,
 			  cascadedMessages));
 }
 
 
 tree_node
-_gst_make_unary_expr (tree_node receiver,
+_gst_make_unary_expr (YYLTYPE *location,
+		     tree_node receiver,
 		      char *unarySelectorExpr)
 {
   OOP selector;
@@ -154,59 +158,83 @@ _gst_make_unary_expr (tree_node receiver,
   /* selectors, being interned symbols, don't need to be incubated --
      symbols once created are always referenced */
   selector = _gst_intern_string (unarySelectorExpr);
-  return (make_expr_node (TREE_UNARY_EXPR, receiver, selector, NULL));
+  return (make_expr_node (location, TREE_UNARY_EXPR, receiver, selector, NULL));
 }
 
 tree_node
-_gst_intern_ident (char *ident)
+_gst_intern_ident (YYLTYPE *location,
+		     char *ident)
 {
   return (make_expr_node
-	  (TREE_SYMBOL_NODE, NULL, _gst_intern_string (ident), NULL));
+	  (location, TREE_SYMBOL_NODE, NULL, _gst_intern_string (ident), NULL));
 }
 
 tree_node
-_gst_make_return (tree_node expression)
+_gst_make_return (YYLTYPE *location,
+		     tree_node expression)
 {
   return (make_expr_node
-	  (TREE_RETURN_EXPR, expression, _gst_nil_oop, NULL));
+	  (location, TREE_RETURN_EXPR, expression, _gst_nil_oop, NULL));
 }
 
 tree_node
-_gst_make_keyword_expr (tree_node receiver,
+_gst_make_keyword_expr (YYLTYPE *location,
+		     tree_node receiver,
 			tree_node keywordMessage)
 {
   return (make_expr_node
-	  (TREE_KEYWORD_EXPR, receiver, _gst_nil_oop, keywordMessage));
+	  (location, TREE_KEYWORD_EXPR, receiver, _gst_nil_oop, keywordMessage));
 }
 
 tree_node
-_gst_make_assign (tree_node variables,
+_gst_make_assign (YYLTYPE *location,
+		     tree_node variables,
 		  tree_node expression)
 {
   return (make_expr_node
-	  (TREE_ASSIGN_EXPR, variables, _gst_nil_oop, expression));
+	  (location, TREE_ASSIGN_EXPR, variables, _gst_nil_oop, expression));
 }
 
 tree_node
-_gst_make_keyword_list (char *keyword,
+_gst_make_statement_list (YYLTYPE *location,
+			  tree_node expression)
+{
+  return (make_list_node (location, TREE_STATEMENT_LIST, NULL, expression));
+}
+
+tree_node
+_gst_make_keyword_list (YYLTYPE *location,
+		     char *keyword,
 			tree_node expression)
 {
-  return (make_list_node (TREE_KEYWORD_LIST, keyword, expression));
+  return (make_list_node (location, TREE_KEYWORD_LIST, keyword, expression));
 }
 
 tree_node
-_gst_make_variable_list (tree_node variable)
+_gst_make_variable_list (YYLTYPE *location,
+		     tree_node variable)
 {
   /* Actually, we rely on the fact that a variable is represented as a
      tree node of type list_node, so all we do is change the node tag
-     to TREE_VARIABLE_LIST. */
-  variable->nodeType = TREE_VARIABLE_LIST;
+     to TREE_VAR_DECL_LIST. */
+  variable->nodeType = TREE_VAR_DECL_LIST;
   return (variable);
 }
 
+tree_node
+_gst_make_assignment_list (YYLTYPE *location,
+		     tree_node variable)
+{
+  /* Actually, we rely on the fact that a variable is represented as a
+     tree node of type list_node, so all we do is change the node tag
+     to TREE_VAR_DECL_LIST. */
+  return (make_list_node (location, TREE_VAR_ASSIGN_LIST, NULL, variable));
+}
+
 
 tree_node
-_gst_make_binary_expr (tree_node receiver,
+_gst_make_binary_expr (YYLTYPE *location,
+		     tree_node receiver,
 		       char *binaryOp,
 		       tree_node argument)
 {
@@ -214,29 +242,25 @@ _gst_make_binary_expr (tree_node receiver,
 
   selector = _gst_intern_string (binaryOp);
   return (make_expr_node
-	  (TREE_BINARY_EXPR, receiver, selector, argument));
+	  (location, TREE_BINARY_EXPR, receiver, selector, argument));
 }
 
 tree_node
-_gst_make_statement_list (tree_node expression)
+_gst_make_message_list (YYLTYPE *location,
+		     tree_node messageElt)
 {
-  return (make_list_node (TREE_STATEMENT_LIST, NULL, expression));
+  return (make_list_node (location, TREE_MESSAGE_LIST, NULL, messageElt));
 }
 
 tree_node
-_gst_make_message_list (tree_node messageElt)
-{
-  return (make_list_node (TREE_MESSAGE_LIST, NULL, messageElt));
-}
-
-tree_node
-_gst_make_block (tree_node arguments,
+_gst_make_block (YYLTYPE *location,
+		     tree_node arguments,
 		 tree_node temporaries,
 		 tree_node statements)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_BLOCK_NODE);
+  result = make_tree_node (location, TREE_BLOCK_NODE);
   result->v_block.arguments = arguments;
   result->v_block.temporaries = temporaries;
   result->v_block.statements = statements;
@@ -244,18 +268,20 @@ _gst_make_block (tree_node arguments,
 }
 
 tree_node
-_gst_make_variable (char *name)
+_gst_make_variable (YYLTYPE *location,
+		     char *name)
 {
-  return (make_list_node (TREE_VARIABLE_NODE, name, NULL));
+  return (make_list_node (location, TREE_VARIABLE_NODE, name, NULL));
 }
 
 
 tree_node
-_gst_make_int_constant (long int ival)
+_gst_make_int_constant (YYLTYPE *location,
+		     long int ival)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = CONST_INT;
   result->v_const.val.iVal = ival;
 
@@ -263,11 +289,12 @@ _gst_make_int_constant (long int ival)
 }
 
 tree_node
-_gst_make_byte_object_constant (byte_object boval)
+_gst_make_byte_object_constant (YYLTYPE *location,
+		     byte_object boval)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = CONST_BYTE_OBJECT;
   result->v_const.val.boVal = boval;
 
@@ -275,11 +302,12 @@ _gst_make_byte_object_constant (byte_object boval)
 }
 
 tree_node
-_gst_make_float_constant (long double fval, int type)
+_gst_make_float_constant (YYLTYPE *location,
+		     long double fval, int type)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = type;
   result->v_const.val.fVal = fval;
 
@@ -287,11 +315,12 @@ _gst_make_float_constant (long double fval, int type)
 }
 
 tree_node
-_gst_make_string_constant (char *sval)
+_gst_make_string_constant (YYLTYPE *location,
+		     char *sval)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = CONST_STRING;
   result->v_const.val.sVal = sval;
 
@@ -299,23 +328,26 @@ _gst_make_string_constant (char *sval)
 }
 
 tree_node
-_gst_make_oop_constant (OOP oval)
+_gst_make_oop_constant (YYLTYPE *location,
+		     OOP oval)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = CONST_OOP;
   result->v_const.val.oopVal = oval;
+  INC_ADD_OOP (oval);
 
   return (result);
 }
 
 tree_node
-_gst_make_char_constant (char cval)
+_gst_make_char_constant (YYLTYPE *location,
+		     char cval)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = CONST_OOP;
   result->v_const.val.oopVal = CHAR_OOP_AT (cval);
 
@@ -323,11 +355,12 @@ _gst_make_char_constant (char cval)
 }
 
 tree_node
-_gst_make_symbol_constant (tree_node symbolNode)
+_gst_make_symbol_constant (YYLTYPE *location,
+		     tree_node symbolNode)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = CONST_OOP;
   if (symbolNode)
     result->v_const.val.oopVal = symbolNode->v_expr.selector;
@@ -352,7 +385,8 @@ _gst_make_symbol_constant (tree_node symbolNode)
  *    and ByteArrays are represented exactly the same as LargeIntegers.
  */
 tree_node
-_gst_make_byte_array_constant (tree_node aval)
+_gst_make_byte_array_constant (YYLTYPE *location,
+		     tree_node aval)
 {
   tree_node arrayElt, ival;
   int len;
@@ -377,15 +411,16 @@ _gst_make_byte_array_constant (tree_node aval)
       *data++ = ival->v_const.val.iVal;
     }
 
-  return (_gst_make_byte_object_constant (bo));
+  return (_gst_make_byte_object_constant (location, bo));
 }
 
 tree_node
-_gst_make_array_constant (tree_node aval)
+_gst_make_array_constant (YYLTYPE *location,
+		     tree_node aval)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = CONST_ARRAY;
   result->v_const.val.aVal = aval;
 
@@ -393,11 +428,12 @@ _gst_make_array_constant (tree_node aval)
 }
 
 tree_node
-_gst_make_array_constructor (tree_node statements)
+_gst_make_array_constructor (YYLTYPE *location,
+		     tree_node statements)
 {
   tree_node result;
 
-  result = make_tree_node (TREE_ARRAY_CONSTRUCTOR);
+  result = make_tree_node (location, TREE_ARRAY_CONSTRUCTOR);
   result->v_const.constType = CONST_ARRAY;
   result->v_const.val.aVal = statements;
 
@@ -405,7 +441,8 @@ _gst_make_array_constructor (tree_node statements)
 }
 
 tree_node
-_gst_make_binding_constant (tree_node variables)
+_gst_make_binding_constant (YYLTYPE *location,
+		     tree_node variables)
 {
   tree_node result;
   OOP assocOOP = _gst_find_variable_binding (variables);
@@ -413,7 +450,7 @@ _gst_make_binding_constant (tree_node variables)
   if (IS_NIL (assocOOP))
     return (NULL);
 
-  result = make_tree_node (TREE_CONST_EXPR);
+  result = make_tree_node (location, TREE_CONST_EXPR);
   result->v_const.constType = CONST_OOP;
   result->v_const.val.oopVal = assocOOP;
 
@@ -424,20 +461,16 @@ void
 _gst_add_node (tree_node n1,
 	       tree_node n2)
 {
+  n1->location.last_line = n2->location.last_line;
+  n1->location.last_column = n2->location.last_column;
+
   *(n1->v_list.nextAddr) = n2;
-  n1->v_list.nextAddr = n2->v_list.nextAddr;	/* since they're all
-						   created this way
-						   anyway, we might as
-						   well use it to our
-						   advantage */
+  n1->v_list.nextAddr = n2->v_list.nextAddr;
 }
 
 void
-_gst_free_tree (tree_node tree)
+_gst_free_tree ()
 {
-  if (tree == NULL)
-    return;
-
   obstack_free (_gst_compilation_obstack, NULL);
   obstack_init (_gst_compilation_obstack);
 }
@@ -454,13 +487,14 @@ _gst_free_tree (tree_node tree)
 
 
 static tree_node
-make_list_node (node_type nodeType,
+make_list_node (YYLTYPE *location,
+		     node_type nodeType,
 		char *name,
 		tree_node value)
 {
   tree_node result;
 
-  result = make_tree_node (nodeType);
+  result = make_tree_node (location, nodeType);
   result->v_list.name = name;
   result->v_list.value = value;
   result->v_list.next = NULL;
@@ -469,14 +503,15 @@ make_list_node (node_type nodeType,
 }
 
 static tree_node
-make_expr_node (node_type nodeType,
+make_expr_node (YYLTYPE *location,
+		     node_type nodeType,
 		tree_node receiver,
 		OOP selector,
 		tree_node expression)
 {
   tree_node result;
 
-  result = make_tree_node (nodeType);
+  result = make_tree_node (location, nodeType);
   result->v_expr.receiver = receiver;
   result->v_expr.selector = selector;
   result->v_expr.expression = expression;
@@ -484,7 +519,8 @@ make_expr_node (node_type nodeType,
 }
 
 static tree_node
-make_tree_node (node_type nodeType)
+make_tree_node (YYLTYPE *location,
+		     node_type nodeType)
 {
   tree_node result;
 
@@ -492,6 +528,7 @@ make_tree_node (node_type nodeType)
 				      sizeof (struct tree_node));
 
   result->nodeType = nodeType;
+  result->location = *location;
   return (result);
 }
 
@@ -508,19 +545,26 @@ _gst_print_tree (tree_node node,
 {
   if (node == NULL)
     {
-      indent (level);
       printf ("%s\n", _gst_nil_name);
       return;
     }
 
+  if (node->nodeType < TREE_FIRST || node->nodeType > TREE_LAST)
+    {
+      printf ("Unknown tree node type %d\n", node->nodeType);
+      return;
+    }
+
+  printf ("%s\n", node_type_names[node->nodeType]);
+
   switch (node->nodeType)
     {
     case TREE_METHOD_NODE:
-      print_method_node (node, level);
+      print_method_node (node, level + 2);
       break;
 
     case TREE_BLOCK_NODE:
-      print_block_node (node, level);
+      print_block_node (node, level + 2);
       break;
 
     case TREE_SYMBOL_NODE:
@@ -528,30 +572,31 @@ _gst_print_tree (tree_node node,
     case TREE_BINARY_EXPR:
     case TREE_KEYWORD_EXPR:
     case TREE_CASCADE_EXPR:
-    case TREE_STATEMENT_LIST:
     case TREE_RETURN_EXPR:
     case TREE_ASSIGN_EXPR:
-      print_expr_node (node, level);
+      print_expr_node (node, level + 2);
       break;
 
     case TREE_VARIABLE_NODE:
     case TREE_KEYWORD_LIST:
-    case TREE_VARIABLE_LIST:
     case TREE_ARRAY_ELT_LIST:
     case TREE_MESSAGE_LIST:
-      print_list_node (node, level);
+    case TREE_STATEMENT_LIST:
+    case TREE_VAR_DECL_LIST:
+    case TREE_VAR_ASSIGN_LIST:
+      print_list_node (node, level + 2);
       break;
 
     case TREE_ARRAY_CONSTRUCTOR:
-      print_array_constructor_node (node, level);
+      print_array_constructor_node (node, level + 2);
       break;
 
     case TREE_CONST_EXPR:
-      print_const_node (node, level);
+      print_const_node (node, level + 2);
       break;
 
     default:
-      _gst_errorf ("Unknown tree note type %d\n", node->nodeType);
+      abort ();
     }
 }
 
@@ -559,75 +604,71 @@ static void
 print_array_constructor_node (tree_node node,
 			      int level)
 {
-  print_node_type (node, level);
-  _gst_print_tree (node->v_const.val.aVal, level + 1);
+  indent (level);
+  _gst_print_tree (node->v_const.val.aVal, level);
 }
 
 static void
 print_list_node (tree_node node,
 		 int level)
 {
-  print_node_type (node, level);
-  indent (level + 1);
+  indent (level);
   printf ("name: %s\n",
 	  node->v_list.name ? node->v_list.name : _gst_nil_name);
-  indent (level + 1);
-  printf ("value:\n");
-  _gst_print_tree (node->v_list.value, level + 2);
-  indent (level + 1);
-  printf ("next:\n");
-  _gst_print_tree (node->v_list.next, level);
+  indent (level);
+  printf ("value: ");
+  _gst_print_tree (node->v_list.value, level + 7);
+  if (node->v_list.next)
+    {
+      indent (level - 2);
+      _gst_print_tree (node->v_list.next, level - 2);
+    }
 }
 
 static void
 print_expr_node (tree_node node,
 		 int level)
 {
-  print_node_type (node, level);
-  indent (level + 1);
+  indent (level);
   printf ("selector: %#O\n", node->v_expr.selector);
-  indent (level + 1);
-  printf ("receiver:\n");
-  _gst_print_tree (node->v_expr.receiver, level + 2);
+  indent (level);
+  printf ("receiver: ");
+  _gst_print_tree (node->v_expr.receiver, level + 10);
   /* ??? don't print the expression for unary type things, and don't
      print the receiver for symbol nodes */
-  indent (level + 1);
-  printf ("expression:\n");
-  _gst_print_tree (node->v_expr.expression, level + 2);
+  indent (level);
+  printf ("expression: ");
+  _gst_print_tree (node->v_expr.expression, level + 12);
 }
 
 static void
 print_method_node (tree_node node,
 		   int level)
 {
-  print_node_type (node, level);
-  indent (level + 1);
+  indent (level);
   printf ("selectorExpr: ");
-  _gst_print_tree (node->v_method.selectorExpr, level + 2);
-  indent (level + 1);
-  /* ??? don't print the temporaries label if there are no temporaries */
-  printf ("temporaries:\n");
-  _gst_print_tree (node->v_method.temporaries, level + 2);
-  indent (level + 1);
-  printf ("statements:\n");
-  _gst_print_tree (node->v_method.statements, level + 2);
+  _gst_print_tree (node->v_method.selectorExpr, level + 14);
+  indent (level);
+  printf ("temporaries: ");
+  _gst_print_tree (node->v_method.temporaries, level + 13);
+  indent (level);
+  printf ("statements: ");
+  _gst_print_tree (node->v_method.statements, level + 12);
 }
 
 static void
 print_block_node (tree_node node,
 		  int level)
 {
-  print_node_type (node, level);
-  indent (level + 1);
-  printf ("arguments:\n");
-  _gst_print_tree (node->v_block.arguments, level + 2);
-  indent (level + 1);
-  /* ??? don't print the temporaries label if there are no temporaries */
-  printf ("temporaries:\n");
-  _gst_print_tree (node->v_block.temporaries, level + 2);
-  indent (level + 1);
-  printf ("statements:\n");
-  _gst_print_tree (node->v_block.statements, level + 2);
+  indent (level);
+  printf ("arguments: ");
+  _gst_print_tree (node->v_block.arguments, level + 11);
+  indent (level);
+  printf ("temporaries: ");
+  _gst_print_tree (node->v_block.temporaries, level + 13);
+  indent (level);
+  printf ("statements: ");
+  _gst_print_tree (node->v_block.statements, level + 12);
 }
 
 static void
@@ -662,8 +703,8 @@ print_const_node (tree_node node,
       break;
 
     case CONST_ARRAY:
-      printf ("array:\n");
-      _gst_print_tree (node->v_const.val.aVal, level + 1);
+      printf ("array: ");
+      _gst_print_tree (node->v_const.val.aVal, level + 7);
       break;
 
     default:
@@ -671,18 +712,8 @@ print_const_node (tree_node node,
     }
 }
 
-
-static void
-print_node_type (tree_node node,
-		 int level)
-{
-  indent (level);
-  printf ("%s\n", node_type_names[node->nodeType]);
-}
-
 static void
 indent (int level)
 {
-  for (; level > 0; level--)
-    printf ("  ");
+  printf ("%*s", level, "");
 }
