@@ -92,9 +92,7 @@
  * A newly created Filament object is returned.
  **/
 Filament *
-filnew (init, len)
-     const char *const init;
-     size_t len;
+filnew (const char *const init, size_t len)
 {
   Filament *new;
 
@@ -121,10 +119,7 @@ filnew (init, len)
  * The initialised Filament object is returned.
  **/
 Filament *
-filinit (fil, init, len)
-     Filament *fil;
-     const char *const init;
-     size_t len;
+filinit (Filament *fil, const char *const init, size_t len)
 {
   if (init == NULL || len < 1)
     {
@@ -181,8 +176,7 @@ filinit (fil, init, len)
  * this string including any embedded nulls.
  **/
 char *
-fildelete (fil)
-     Filament *fil;
+fildelete (Filament *fil)
 {
   char *value;
 
@@ -200,133 +194,6 @@ fildelete (fil)
   return value;
 }
 
-#undef filval
-/**
- * filval:
- * @fil: The Filament object being queried.
- *
- * Return value:
- * A pointer to the null terminated string held by the Filament
- * object is returned.  Since the @fil may contain embedded nulls, it
- * is not entirely safe to use the strfoo() API to examine the contents
- * of the return value.
- **/
-char *
-filval (fil)
-     Filament *fil;
-{
-  return _filval (fil);
-}
-
-#undef fillen
-/**
- * fillen:
- * @fil: The Filament object being queried.
- * 
- * Return value:
- * The length of @fil, including any embedded nulls, but excluding the
- * terminating null, is returned.
- **/
-size_t
-fillen (fil)
-     Filament *fil;
-{
-  return _fillen (fil);
-}
-
-#undef filelt
-/**
- * filelt:
- * @fil: The Filament being queried.
- * @n: A zero based index into @fil.
- * 
- * This function looks for the @n'th element of @fil.
- * 
- * Return value:
- * If @n is an index inside the Filament @fil, then the character stored
- * at that index cast to an int is returned, otherwise @n is outside
- * this range and -1 is returned.
- **/
-int
-filelt (fil, n)
-     Filament *fil;
-     size_t n;
-{
-  return _filelt (fil, n);
-}
-
-#undef filcat
-/**
- * filcat:
- * @fil: The destination Filament of the concatenation.
- * @str: The address of the source bytes for concatenation.
- * 
- * The bytes starting at address @str upto and including the first null
- * byte encountered are destructively concatenated to @fil.  If
- * necessary @fil is dynamically reallocated to make room for this
- * operation.
- * 
- * Return value: 
- * A pointer to the (not null terminated) string which is the result
- * of this concatenation is returned.
- **/
-char *
-filcat (fil, str)
-     Filament *fil;
-     const char *str;
-{
-  size_t length = strlen (str);
-  return _filncat (fil, str, length);
-}
-
-#undef filncat
-/**
- * filncat:
- * @fil: The destination Filament of the concatenation.
- * @str: The address of the source bytes for concatenation.
- * @n: The number of bytes to be copied from @str.
- * 
- * @n bytes starting with the byte at address @str are destructively
- * concatenated to @fil.  If necessary, @fil is dynamically reallocated
- * to make room for this operation.
- * 
- * Return value:
- * A pointer to the (not null terminated) string which is the result
- * of this concatenation is returned.
- **/
-char *
-filncat (fil, str, n)
-     Filament *fil;
-     const char *str;
-     size_t n;
-{
-  return _filncat (fil, str, n);
-}
-
-#undef filccat
-/**
- * filccat:
- * @fil: The destination Filament of the concatenation.
- * @c: The character to append to @fil.
- * 
- * @c is destructively concatenated to @fil.  If necessary, @fil is
- * dynamically reallocated to make room for this operation.  When used
- * repeatedly this function is less efficient than %filncat,
- * since it must check whether to extend the filament before each
- * character is appended.
- * 
- * Return value:
- * A pointer to the (not null terminated) string which is the result
- * of this concatenation is returned.
- **/
-char *
-filccat (fil, c)
-     Filament *fil;
-     int c;
-{
-  return _filccat (fil, c);
-}
-
 /**
  * _fil_extend:
  * @fil: The Filament object which may need more string space.
@@ -338,10 +205,7 @@ filccat (fil, c)
  * for the prospective contents.
  */
 void
-_fil_extend (fil, len, copy)
-     Filament *fil;
-     size_t len;
-     boolean copy;
+_fil_extend (Filament *fil, size_t len, boolean copy)
 {
   /* Usually we will simply double the amount of space previously
      allocated, but if the extra data is larger than the current
