@@ -1,3 +1,4 @@
+#line 1 "./filament.in"
 /*  -*- Mode: C -*-  */
 
 /* filament.h --- a bit like a string but different =)O|
@@ -70,7 +71,8 @@ struct filament
  * Return value:
  * A newly created Filament object is returned.
  **/
-extern Filament * filnew PARAMS((const char *const init, size_t len));
+extern Filament * filnew (const char *const init, size_t len);
+
 
 /**
  * filinit:   
@@ -85,7 +87,8 @@ extern Filament * filnew PARAMS((const char *const init, size_t len));
  * Return value:
  * The initialised Filament object is returned.
  **/
-extern Filament * filinit PARAMS((Filament *fil, const char *const init, size_t len));
+extern Filament * filinit (Filament *fil, const char *const init, size_t len);
+
 
 /**
  * fildelete:  destructor 
@@ -99,92 +102,8 @@ extern Filament * filinit PARAMS((Filament *fil, const char *const init, size_t
  * as a normal C string.  The entire Filament contents are copied into
  * this string including any embedded nulls.
  **/
-extern char * fildelete PARAMS((Filament *fil));
+extern char * fildelete (Filament *fil);
 
-/**
- * filval:   
- * @fil: The Filament object being queried.
- *
- * Return value:
- * A pointer to the null terminated string held by the Filament
- * object is returned.  Since the @fil may contain embedded nulls, it
- * is not entirely safe to use the strfoo() API to examine the contents
- * of the return value.
- **/
-extern char * filval PARAMS((Filament *fil));
-
-/**
- * fillen:   
- * @fil: The Filament object being queried.
- * 
- * Return value:
- * The length of @fil, including any embedded nulls, but excluding the
- * terminating null, is returned.
- **/
-extern size_t fillen PARAMS((Filament *fil));
-
-/**
- * filelt:   
- * @fil: The Filament being queried.
- * @n: A zero based index into @fil.
- * 
- * This function looks for the @n'th element of @fil.
- * 
- * Return value:
- * If @n is an index inside the Filament @fil, then the character stored
- * at that index cast to an int is returned, otherwise @n is outside
- * this range and -1 is returned.
- **/
-extern int filelt PARAMS((Filament *fil, size_t n));
-
-/**
- * filcat:   
- * @fil: The destination Filament of the concatenation.
- * @str: The address of the source bytes for concatenation.
- * 
- * The bytes starting at address @str upto and including the first null
- * byte encountered are destructively concatenated to @fil.  If
- * necessary @fil is dynamically reallocated to make room for this
- * operation.
- * 
- * Return value: 
- * A pointer to the (not null terminated) string which is the result
- * of this concatenation is returned.
- **/
-extern char * filcat PARAMS((Filament *fil, const char *str));
-
-/**
- * filncat:   
- * @fil: The destination Filament of the concatenation.
- * @str: The address of the source bytes for concatenation.
- * @n: The number of bytes to be copied from @str.
- * 
- * @n bytes starting with the byte at address @str are destructively
- * concatenated to @fil.  If necessary, @fil is dynamically reallocated
- * to make room for this operation.
- * 
- * Return value:
- * A pointer to the (not null terminated) string which is the result
- * of this concatenation is returned.
- **/
-extern char * filncat PARAMS((Filament *fil, const char *str, size_t n));
-
-/**
- * filccat:   
- * @fil: The destination Filament of the concatenation.
- * @c: The character to append to @fil.
- * 
- * @c is destructively concatenated to @fil.  If necessary, @fil is
- * dynamically reallocated to make room for this operation.  When used
- * repeatedly this function is less efficient than %filncat,
- * since it must check whether to extend the filament before each
- * character is appended.
- * 
- * Return value:
- * A pointer to the (not null terminated) string which is the result
- * of this concatenation is returned.
- **/
-extern char * filccat PARAMS((Filament *fil, int c));
 
 /**
  * _fil_extend:   
@@ -196,77 +115,135 @@ extern char * filccat PARAMS((Filament *fil, int c));
  * considering the space left in @fil and @len, the length required
  * for the prospective contents.
  */
-extern void _fil_extend PARAMS((Filament *fil, size_t len, boolean copy));
+extern void _fil_extend (Filament *fil, size_t len, boolean copy);
 
 
+#line 61 "./filament.in"
+
 /* Save the overhead of a function call in the great majority of cases. */
 #define fil_maybe_extend(fil, len, copy)  \
   (((len)>=(fil)->size) ? _fil_extend((fil), (len), (copy)) : (void)0)
 
-/* These macros evaluate the parameter multiple times.
-   They are used to share the implementation between the
-   macros and the functions.  */
+/**
+ * filval:
+ * @fil: The Filament object being queried.
+ *
+ * Return value:
+ * A pointer to the null terminated string held by the Filament
+ * object is returned.  Since the @fil may contain embedded nulls, it
+ * is not entirely safe to use the strfoo() API to examine the contents
+ * of the return value.
+ **/
+SNV_INLINE char *
+filval (Filament *fil)
+{
+  /* Because we have been careful to ensure there is always at least
+     one spare byte of allocated memory, it is safe to set it here. */
+  fil->value[fil->length] = '\0';
+  return (char *) (fil->value);
+}
 
-/* Because we have been careful to ensure there is always at least
-   one spare byte of allocated memory, it is safe to set it here. */
-#define _filval(fil) \
-  ((char *) (fil->value[fil->length] = '\0', fil->value))
+/**
+ * fillen:
+ * @fil: The Filament object being queried.
+ *
+ * Return value:
+ * The length of @fil, including any embedded nulls, but excluding the
+ * terminating null, is returned.
+ **/
+SNV_INLINE size_t
+fillen (Filament *fil)
+{
+  return fil->length;
+}
 
-#define _fillen(fil) \
-  (fil->length)
+/**
+ * filelt:
+ * @fil: The Filament being queried.
+ * @n: A zero based index into @fil.
+ *
+ * This function looks for the @n'th element of @fil.
+ *
+ * Return value:
+ * If @n is an index inside the Filament @fil, then the character stored
+ * at that index cast to an int is returned, otherwise @n is outside
+ * this range and -1 is returned.
+ **/
+SNV_INLINE int
+filelt (Filament *fil, size_t n)
+{
+  if ((n >= 0) && (n < fil->length))
+    return (int) fil->value[n];
+  else
+    return -1;
+}
 
-#define _filelt(fil, n) \
-  (((n >= 0) && (n < fil->length)) ? (int) fil->value[n] : -1)
+/**
+ * filncat:
+ * @fil: The destination Filament of the concatenation.
+ * @str: The address of the source bytes for concatenation.
+ * @n: The number of bytes to be copied from @str.
+ *
+ * @n bytes starting with the byte at address @str are destructively
+ * concatenated to @fil.  If necessary, @fil is dynamically reallocated
+ * to make room for this operation.
+ *
+ * Return value:
+ * A pointer to the (not null terminated) string which is the result
+ * of this concatenation is returned.
+ **/
+SNV_INLINE char *
+filncat (Filament *fil, const char *str, size_t n)
+{
+  fil_maybe_extend (fil, n + fil->length, TRUE);
+  memcpy (fil->value + fil->length, str, n);
+  fil->length += n;
+  return fil->value;
+}
 
-#define _filncat(fil, str, n) \
-  (fil_maybe_extend (fil, n + fil->length, TRUE), \
-   memcpy (fil->value + fil->length, str, n), \
-   fil->length += n, \
-   fil->value)
+/**
+ * filcat:
+ * @fil: The destination Filament of the concatenation.
+ * @str: The address of the source bytes for concatenation.
+ *
+ * The bytes starting at address @str upto and including the first null
+ * byte encountered are destructively concatenated to @fil.  If
+ * necessary @fil is dynamically reallocated to make room for this
+ * operation.
+ *
+ * Return value:
+ * A pointer to the (not null terminated) string which is the result
+ * of this concatenation is returned.
+ **/
+SNV_INLINE char *
+filcat (Filament *fil, const char *str)
+{
+  size_t length = strlen (str);
+  return filncat (fil, str, length);
+}
 
-#define _filccat(fil, c) \
-  (fil_maybe_extend (fil, 1 + fil->length, TRUE), \
-   fil->value[fil->length++] = c, \
-   fil->value)
-
-#if __GNUC__ >= 2
-#define filval(fil) __extension__ ({ \
-  Filament *_fil = (fil); \
-  _filval (_fil); \
-})
-
-#define fillen(fil) __extension__ ({ \
-  Filament *_fil = (fil); \
-  _fillen (_fil); \
-})
-
-#define filelt(fil, n) __extension__ ({ \
-  Filament *_fil = (fil); \
-  size_t _n = (n); \
-  _filelt (_fil, _n); \
-})
-
-#define filcat(fil, str) __extension__ ({ \
-  Filament *_fil = (fil); \
-  const char *_str = (str); \
-  size_t _n = strlen(_str); \
-  _filncat (_fil, _str, _n); \
-})
-
-#define filncat(fil, str, n) __extension__ ({ \
-  Filament *_fil = (fil); \
-  const char *_str = (str); \
-  size_t _n = (n); \
-  _filncat (_fil, _str, _n); \
-})
-
-#define filccat(fil, c) __extension__ ({ \
-  Filament *_fil = fil; \
-  char _c = (c); \
-  _filccat (_fil, _c); \
-})
-
-#endif /* __GNUC__ */
+/**
+ * filccat:
+ * @fil: The destination Filament of the concatenation.
+ * @c: The character to append to @fil.
+ *
+ * @c is destructively concatenated to @fil.  If necessary, @fil is
+ * dynamically reallocated to make room for this operation.  When used
+ * repeatedly this function is less efficient than %filncat,
+ * since it must check whether to extend the filament before each
+ * character is appended.
+ *
+ * Return value:
+ * A pointer to the (not null terminated) string which is the result
+ * of this concatenation is returned.
+ **/
+SNV_INLINE char *
+filccat (Filament *fil, int c)
+{
+  fil_maybe_extend (fil, 1 + fil->length, TRUE);
+  fil->value[fil->length++] = c;
+  return fil->value;
+}
 
 #ifdef __cplusplus
 #if 0
