@@ -46,119 +46,20 @@
 #include "printf.h"
 
 #ifdef HAVE_LONG_DOUBLE
-extern long double frexpl (long double x, int *exp);
-extern long double ldexpl (long double x, int exp);
+#ifndef HAVE_FREXPL
+static long double frexpl (long double x, int *exp);
+#endif
+#ifndef HAVE_LDEXPL
+static long double ldexpl (long double x, int exp);
+#endif
 #else
 #define frexpl frexp
 #define ldexpl ldexp
 #endif
 
-
-/* This is where the parsing of FORMAT strings is handled:
-
-   Each of these functions should inspect PPARSER for parser
-   state information;  update PPARSER as necessary based on
-   the state discovered;  possibly put some characters in STREAM, in
-   which case that number of characters must be returned.  If the
-   handler detects that parsing (of the current specifier) is complete,
-   then it must set pinfo->state to SNV_STATE_END.  The library will then
-   copy characters from the format string to STREAM until another unescaped
-   SNV_CHAR_SPEC is detected when the handlers will be called again. */
-
-static int printf_numeric_param_info PARAMS ((struct printf_info * const pinfo,
-				   size_t n, int *argtypes));
-static int printf_flag_info PARAMS ((struct printf_info * const pinfo,
-				 size_t n, int *argtypes));
-static int printf_modifier_info PARAMS ((struct printf_info * const pinfo,
-				     size_t n, int *argtypes));
-
-static int printf_flag PARAMS ((STREAM * stream, 
-				   struct printf_info * const pinfo,
-				   union printf_arg const *args));
-static int printf_numeric_param PARAMS ((STREAM * stream, 
-				   struct printf_info * const pinfo,
-				   union printf_arg const *args));
-static int printf_count PARAMS ((STREAM * stream,
-				   struct printf_info * const pinfo,
-				   union printf_arg const *args));
-static int printf_char PARAMS ((STREAM * stream,
-				   struct printf_info * const pinfo,
-				   union printf_arg const *args));
-static int printf_float PARAMS ((STREAM * stream,
-				   struct printf_info * const pinfo,
-				   union printf_arg const *args));
-static int printf_integer PARAMS ((STREAM * stream,
-				   struct printf_info * const pinfo,
-				   union printf_arg const *args));
-static int printf_pointer PARAMS ((STREAM * stream,
-				   struct printf_info * const pinfo,
-				   union printf_arg const *args));
-static int printf_string PARAMS ((STREAM * stream,
-				   struct printf_info * const pinfo,
-				   union printf_arg const *args));
-
-spec_entry snv_default_spec_table[] = {
-  /* ch  type         function */
-  {' ', TRUE, 0, printf_flag, printf_flag_info},
-  {'#', TRUE, 0, printf_flag, printf_flag_info},
-  {'+', TRUE, 0, printf_flag, printf_flag_info},
-  {'-', TRUE, 0, printf_flag, printf_flag_info},
-  {'\'', TRUE, 0, printf_flag, printf_flag_info},
-  {'*', TRUE, PA_INT, printf_numeric_param, printf_numeric_param_info},
-  {'.', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'0', TRUE, 0, printf_flag, printf_flag_info},
-  {'1', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'2', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'3', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'4', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'5', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'6', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'7', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'8', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'9', TRUE, 0, printf_numeric_param, printf_numeric_param_info},
-  {'c', FALSE, PA_CHAR, printf_char, NULL},
-  {'d', FALSE, PA_INT, printf_integer, printf_generic_info, (snv_pointer) 10},
-  {'e', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
-  {'E', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
-  {'f', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
-  {'F', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
-  {'g', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
-  {'G', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
-  {'h', TRUE, 0, printf_flag, printf_modifier_info},
-  {'i', FALSE, PA_INT, printf_integer, printf_generic_info, (snv_pointer) 10},
-  {'j', TRUE, 0, printf_flag, printf_modifier_info},
-  {'l', TRUE, 0, printf_flag, printf_modifier_info},
-  {'L', TRUE, 0, printf_flag, printf_modifier_info},
-  {'n', FALSE, PA_INT | PA_FLAG_PTR, printf_count, printf_generic_info},
-  {'o', FALSE, PA_INT | PA_FLAG_UNSIGNED,
-   printf_integer, printf_generic_info, (snv_pointer) 8},
-  {'p', FALSE, PA_POINTER, printf_pointer, NULL, (snv_pointer) 16},
-  {'q', TRUE, 0, printf_flag, printf_modifier_info},
-  {'s', FALSE, PA_STRING, printf_string, NULL},
-  {'t', TRUE, 0, printf_flag, printf_modifier_info},
-  {'u', FALSE, PA_INT | PA_FLAG_UNSIGNED,
-   printf_integer, printf_generic_info, (snv_pointer) 10},
-  {'x', FALSE, PA_INT | PA_FLAG_UNSIGNED,
-   printf_integer, printf_generic_info, (snv_pointer) 16},
-  {'X', FALSE, PA_INT | PA_FLAG_UNSIGNED,
-   printf_integer, printf_generic_info, (snv_pointer) 16},
-  {'z', TRUE, 0, printf_flag, printf_modifier_info},
-  {'\0', FALSE, PA_LAST}
-};
-
-static intmax_t fetch_intmax PARAMS ((struct printf_info * pinfo, union printf_arg const *arg));
-static uintmax_t fetch_uintmax PARAMS ((struct printf_info * pinfo, union printf_arg const *arg));
-static snv_long_double fetch_double PARAMS ((struct printf_info * pinfo, union printf_arg const *arg));
-
-/* Helper functions to print doubles */
-static inline snv_long_double ipow PARAMS ((snv_long_double base, int n));
-static int print_float PARAMS ((struct printf_info * pinfo, char *buf, snv_long_double n));
-
 
-uintmax_t
-fetch_uintmax (pinfo, arg)
-     struct printf_info *pinfo;
-     union printf_arg const *arg;
+static uintmax_t
+fetch_uintmax (struct printf_info *pinfo, union printf_arg const *arg)
 {
   if (pinfo->is_long_double)
     return (uintmax_t) arg->pa_u_long_long_int;
@@ -175,10 +76,8 @@ fetch_uintmax (pinfo, arg)
   return (uintmax_t) arg->pa_u_int;
 }
 
-intmax_t
-fetch_intmax (pinfo, arg)
-     struct printf_info *pinfo;
-     union printf_arg const *arg;
+static intmax_t
+fetch_intmax (struct printf_info *pinfo, union printf_arg const *arg)
 {
   if (pinfo->is_long_double)
     return (intmax_t) arg->pa_long_long_int;
@@ -195,10 +94,8 @@ fetch_intmax (pinfo, arg)
   return (intmax_t) arg->pa_int;
 }
 
-snv_long_double
-fetch_double (pinfo, arg)
-     struct printf_info *pinfo;
-     union printf_arg const *arg;
+static snv_long_double
+fetch_double (struct printf_info *pinfo, union printf_arg const *arg)
 {
   if (pinfo->is_long_double)
     return arg->pa_long_double;
@@ -209,8 +106,7 @@ fetch_double (pinfo, arg)
 
 #ifndef HAVE_COPYSIGNL
 snv_long_double
-copysignl (x, y)
-     snv_long_double x, y;
+copysignl (snv_long_double x, snv_long_double y)
 {
 #ifdef HAVE_COPYSIGN
   return x * (snv_long_double) copysign (1.0, x * y);
@@ -223,10 +119,8 @@ copysignl (x, y)
 }
 #endif
 
-snv_long_double
-ipow (base, n)
-     snv_long_double base;
-     int n;
+static snv_long_double
+ipow (snv_long_double base, int n)
 {
   int k = 1;
   snv_long_double result = 1.0;
@@ -249,11 +143,8 @@ ipow (base, n)
   return result;
 }
 
-int
-print_float (pinfo, buf, n)
-     struct printf_info *pinfo;
-     char *buf;
-     snv_long_double n;
+static int
+print_float (struct printf_info *pinfo, char *buf, snv_long_double n)
 {
   /* Print value of n in a buffer in the given base.
      Based upon the algorithm outlined in:
@@ -320,7 +211,7 @@ print_float (pinfo, buf, n)
     }
 
   /* Zero and infinity also can have a sign in front of them. */
-  if (copysign (1.0, n) < 0.0)
+  if (copysignl (1.0, n) < 0.0)
     {
       n = -1.0 * n;
       *p++ = '-';
@@ -576,20 +467,14 @@ print_float (pinfo, buf, n)
 }
 
 
-int
-printf_flag (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+static int
+printf_flag (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
   return 0;
 }
 
-int
-printf_flag_info (pinfo, n, argtypes)
-     struct printf_info *const pinfo;
-     size_t n;
-     int *argtypes;
+static int
+printf_flag_info (struct printf_info *const pinfo, size_t n, int *argtypes)
 {
   return_val_if_fail (pinfo != NULL, SNV_ERROR);
 
@@ -649,11 +534,8 @@ printf_flag_info (pinfo, n, argtypes)
   return 0;
 }
 
-int
-printf_numeric_param (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+static int
+printf_numeric_param (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
   /* Called twice if both the width and precision are
      asterisks, in which case we extract the width on
@@ -667,13 +549,10 @@ printf_numeric_param (stream, pinfo, args)
   return 0;
 }
  
-int
-printf_numeric_param_info (pinfo, n, argtypes)
-     struct printf_info *const pinfo;
-     size_t n;
-     int *argtypes;
+static int
+printf_numeric_param_info (struct printf_info *const pinfo, size_t n, int *argtypes)
 {
-  char *pEnd = NULL;
+  const char *pEnd = NULL;
   int found = 0, allowed_states, new_state;
 
   unsigned long value;		/* we use strtoul, and cast back to int. */
@@ -687,12 +566,24 @@ printf_numeric_param_info (pinfo, n, argtypes)
       found |= 1;
     }
 
-  /* Parse the number (or optionally a ``*''). */
-  value = strtoul (pinfo->format, &pEnd, 10);
-  if (pEnd != NULL && pEnd > pinfo->format)
+  /* Parse the ``*''. */
+  if (*pinfo->format == '*')
+    {
+      if (n)
+        argtypes[0] = PA_INT;
+
+      pinfo->format++;
+      found |= 2;
+    }
+
+  /* Parse the number. */
+  for (pEnd = pinfo->format, value = 0; *pEnd >= '0' && *pEnd <= '9'; pEnd++)
+    value = value * 10 + (*pEnd - '0');
+
+  if (pEnd > pinfo->format)
     {
       pinfo->format = pEnd;
-      found |= 2;
+      found |= 4;
     }
 
   if (value > INT_MAX)
@@ -701,15 +592,7 @@ printf_numeric_param_info (pinfo, n, argtypes)
       return -1;
     }
 
-  if (*pinfo->format == '*')
-    {
-      if (n)
-        argtypes[0] = PA_INT;
-
-      pinfo->format++;
-      found |= 4;
-    }
-
+  /* Finally look for a $. */
   if (*pinfo->format == '$')
     {
       pinfo->format++;
@@ -719,27 +602,35 @@ printf_numeric_param_info (pinfo, n, argtypes)
   switch (found)
     {
     /* We must have read a width specification. */
-    case 4:
+    case 2:
       value = INT_MIN;
 
-    case 2:
+    case 4:
       allowed_states = SNV_STATE_BEGIN | SNV_STATE_WIDTH;
       new_state = ~(SNV_STATE_BEGIN | SNV_STATE_FLAG | SNV_STATE_WIDTH);
       pinfo->width = value;
       break;
 
+    case 14:
+      PRINTF_ERROR (pinfo, "position specifications for widths not supported");
+      return -1;
+
     /* We must have read a precision specification. */
-    case 5:
+    case 3:
       value = INT_MIN;
 
-    case 3:
+    case 5:
       allowed_states = SNV_STATE_PRECISION | SNV_STATE_BEGIN;
       new_state = SNV_STATE_MODIFIER | SNV_STATE_SPECIFIER;
       pinfo->prec = value;
       break;
 
+    case 15:
+      PRINTF_ERROR (pinfo, "precision specifications for precisions not supported");
+      return -1;
+
     /* We must have read a position specification. */
-    case 10:
+    case 12:
       allowed_states = SNV_STATE_BEGIN;
       new_state = ~SNV_STATE_BEGIN;
       pinfo->dollar = value;
@@ -761,14 +652,11 @@ printf_numeric_param_info (pinfo, n, argtypes)
   pinfo->format--;
 
   /* Return the number of arguments used. */
-  return found & 4 ? 1 : 0;
+  return (found & 2) != 0;
 }
 
-int
-printf_modifier_info (pinfo, n, argtypes)
-     struct printf_info *const pinfo;
-     size_t n;
-     int *argtypes;
+static int
+printf_modifier_info (struct printf_info *const pinfo, size_t n, int *argtypes)
 {
   return_val_if_fail (pinfo != NULL, SNV_ERROR);
 
@@ -856,10 +744,7 @@ printf_modifier_info (pinfo, n, argtypes)
  * Always 1.
  */
 int
-printf_generic_info (pinfo, n, argtypes)
-     struct printf_info *const pinfo;
-     size_t n;
-     int *argtypes;
+printf_generic_info (struct printf_info *const pinfo, size_t n, int *argtypes)
 {
   int type = pinfo->type;
 
@@ -886,11 +771,8 @@ printf_generic_info (pinfo, n, argtypes)
 }
 
 
-int
-printf_char (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+static int
+printf_char (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
   int count_or_errorcode = SNV_OK;
   char ch = '\0';
@@ -934,14 +816,11 @@ printf_char (stream, pinfo, args)
   return count_or_errorcode;
 }
 
-int
-printf_float (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+static int
+printf_float (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
   snv_long_double value = 0.0;
-  int len, count_or_errorcode = SNV_OK, type = PA_DOUBLE;
+  int len, count_or_errorcode = SNV_OK;
 #ifdef HAVE_LONG_DOUBLE
   char buffer[LDBL_MAX_10_EXP + 20], *p = buffer;
 #else
@@ -1007,14 +886,9 @@ printf_float (stream, pinfo, args)
   return count_or_errorcode;
 }
 
-int
-printf_count (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+static int
+printf_count (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
-  int type = pinfo->type;
-
   if (pinfo->is_char)
     *(char *) (args->pa_pointer) = pinfo->count;
 
@@ -1033,11 +907,8 @@ printf_count (stream, pinfo, args)
   return 0;
 }
 
-int
-printf_integer (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+static int
+printf_integer (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
   static const char digits_lower[] = "0123456789abcdefghijklmnopqrstuvwxyz";
   static const char digits_upper[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
@@ -1177,11 +1048,8 @@ printf_integer (stream, pinfo, args)
   return count_or_errorcode;
 }
 
-int
-printf_pointer (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+static int
+printf_pointer (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
   int count_or_errorcode = SNV_OK;
 
@@ -1233,11 +1101,8 @@ printf_pointer (stream, pinfo, args)
   return count_or_errorcode;
 }
 
-int
-printf_string (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+static int
+printf_string (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
   int len = 0, count_or_errorcode = SNV_OK;
   const char *p = NULL;
@@ -1311,10 +1176,7 @@ printf_string (stream, pinfo, args)
  * The number of characters output.
  **/
 int
-printf_generic (stream, pinfo, args)
-     STREAM *stream;
-     struct printf_info *const pinfo;
-     union printf_arg const *args;
+printf_generic (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args)
 {
   int len = 0, count_or_errorcode = SNV_OK;
   char *p = NULL;
@@ -1339,7 +1201,7 @@ printf_generic (stream, pinfo, args)
 
   /* Print to a stream using a user-supplied function. */
   fil = filnew (NULL, 0);
-  out = stream_new (fil, SNV_UNLIMITED, NULL, filputc);
+  out = stream_new (fil, SNV_UNLIMITED, NULL, snv_filputc);
   user_func (out, pinfo, args);
   stream_delete (out);
   len = fillen (fil);
@@ -1380,4 +1242,154 @@ printf_generic (stream, pinfo, args)
   return count_or_errorcode;
 }
 
+
+
+/* replacements for frexpl and ldexpl follow */
+
+#if defined HAVE_LONG_DOUBLE && !defined HAVE_FREXPL
+
+/* Binary search.  Quite inefficient but portable. */
+long double
+frexpl(long double x, int *exp)
+{
+  long double exponents[20], *next;
+  int exponent, bit;
+
+  /* Check for zero, nan and infinity. */
+  if (x != x || x + x == x )
+    {
+      *exp = 0;
+      return x;
+    }
+
+  if (x < 0)
+    return -frexpl(-x, exp);
+
+  exponent = 0;
+  if (x > 1.0)
+    {
+      for (next = exponents, exponents[0] = 2.0L, bit = 1;
+	   *next <= x + x;
+	   bit <<= 1, next[1] = next[0] * next[0], next++);
+
+      for (; next >= exponents; bit >>= 1, next--)
+	if (x + x >= *next)
+	  {
+	    x /= *next;
+	    exponent |= bit;
+	  }
+
+    }
+
+  else if (x < 0.5)
+    {
+      for (next = exponents, exponents[0] = 0.5L, bit = 1;
+	   *next > x;
+	   bit <<= 1, next[1] = next[0] * next[0], next++);
+
+      for (; next >= exponents; bit >>= 1, next--)
+	if (x < *next)
+	  {
+	    x /= *next;
+	    exponent |= bit;
+	  }
+
+      exponent = -exponent;
+    }
+
+  *exp = exponent;
+  return x;
+}
+#endif
+
+#if defined HAVE_LONG_DOUBLE && !defined HAVE_LDEXPL
+
+long double
+ldexpl(long double x, int exp)
+{
+  long double factor;
+  int bit;
+
+  /* Check for zero, nan and infinity. */
+  if (x != x || x + x == x )
+    return x;
+
+  if (exp < 0)
+    {
+      exp = -exp;
+      factor = 0.5L;
+    }
+  else
+    factor = 2.0L;
+
+  for (bit = 1; bit <= exp; bit <<= 1, factor *= factor)
+    if (exp & bit)
+      x *= factor;
+
+  return x;
+}
+#endif
+
+
+
+/* This is where the parsing of FORMAT strings is handled:
+
+   Each of these functions should inspect PPARSER for parser
+   state information;  update PPARSER as necessary based on
+   the state discovered;  possibly put some characters in STREAM, in
+   which case that number of characters must be returned.  If the
+   handler detects that parsing (of the current specifier) is complete,
+   then it must set pinfo->state to SNV_STATE_END.  The library will then
+   copy characters from the format string to STREAM until another unescaped
+   SNV_CHAR_SPEC is detected when the handlers will be called again. */
+
+spec_entry snv_default_spec_table[] = {
+  /* ch  type         function */
+  {' ', TRUE, 0, printf_flag, printf_flag_info, NULL},
+  {'#', TRUE, 0, printf_flag, printf_flag_info, NULL},
+  {'+', TRUE, 0, printf_flag, printf_flag_info, NULL},
+  {'-', TRUE, 0, printf_flag, printf_flag_info, NULL},
+  {'\'', TRUE, 0, printf_flag, printf_flag_info, NULL},
+  {'*', TRUE, PA_INT, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'.', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'0', TRUE, 0, printf_flag, printf_flag_info, NULL},
+  {'1', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'2', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'3', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'4', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'5', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'6', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'7', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'8', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'9', TRUE, 0, printf_numeric_param, printf_numeric_param_info, NULL},
+  {'c', FALSE, PA_CHAR, printf_char, NULL, NULL},
+  {'d', FALSE, PA_INT, printf_integer, printf_generic_info, (snv_pointer) 10},
+  {'e', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
+  {'E', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
+  {'f', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
+  {'F', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
+  {'g', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
+  {'G', FALSE, PA_DOUBLE, printf_float, printf_generic_info, (snv_pointer) 6},
+  {'h', TRUE, 0, printf_flag, printf_modifier_info, NULL},
+  {'i', FALSE, PA_INT, printf_integer, printf_generic_info, (snv_pointer) 10},
+  {'j', TRUE, 0, printf_flag, printf_modifier_info, NULL},
+  {'l', TRUE, 0, printf_flag, printf_modifier_info, NULL},
+  {'L', TRUE, 0, printf_flag, printf_modifier_info, NULL},
+  {'n', FALSE, PA_INT | PA_FLAG_PTR, printf_count, printf_generic_info, NULL},
+  {'o', FALSE, PA_INT | PA_FLAG_UNSIGNED,
+   printf_integer, printf_generic_info, (snv_pointer) 8},
+  {'p', FALSE, PA_POINTER, printf_pointer, NULL, (snv_pointer) 16},
+  {'q', TRUE, 0, printf_flag, printf_modifier_info, NULL},
+  {'s', FALSE, PA_STRING, printf_string, NULL, NULL},
+  {'t', TRUE, 0, printf_flag, printf_modifier_info, NULL},
+  {'u', FALSE, PA_INT | PA_FLAG_UNSIGNED,
+   printf_integer, printf_generic_info, (snv_pointer) 10},
+  {'x', FALSE, PA_INT | PA_FLAG_UNSIGNED,
+   printf_integer, printf_generic_info, (snv_pointer) 16},
+  {'X', FALSE, PA_INT | PA_FLAG_UNSIGNED,
+   printf_integer, printf_generic_info, (snv_pointer) 16},
+  {'z', TRUE, 0, printf_flag, printf_modifier_info, NULL},
+  {'\0', FALSE, PA_LAST, NULL, NULL, NULL}
+};
+
 /* format.c ends here */
