@@ -1,3 +1,4 @@
+#line 1 "./printf.in"
 /*  -*- Mode: C -*-  */
 
 /* printf.in --- printf clone for argv arrays
@@ -44,7 +45,7 @@ extern "C"
 {
 #endif
 
-/* The type of each element in the spec_table exported by the fsm. */
+/* The type of each element in the table of printf specifiers. */
 struct spec_entry;
 
 typedef enum
@@ -156,9 +157,9 @@ typedef union printf_arg
 #define PRINTF_ERROR(pi, str)                                         \
         printf_error(pi, __FILE__, __LINE__, SNV_ASSERT_FMT, str);
 
-typedef int printf_function PARAMS ((STREAM *stream, struct printf_info *pparser, union printf_arg const * args));
+typedef int printf_function (STREAM *stream, struct printf_info *pparser, union printf_arg const * args);
 
-typedef int printf_arginfo_function PARAMS ((struct printf_info *pparser, size_t n, int *argtypes));
+typedef int printf_arginfo_function (struct printf_info *pparser, size_t n, int *argtypes);
 
 /**
  * spec_entry:
@@ -203,7 +204,7 @@ spec_entry;
  * Returns %NULL if @func was not successfully registered, a
  * %spec_entry with the information on the function if it was.
  **/
-typedef spec_entry *register_callback_function PARAMS ((unsigned spec, printf_function *func, printf_arginfo_function *arginfo));
+typedef spec_entry *register_callback_function (unsigned spec, printf_function *func, printf_arginfo_function *arginfo);
 
 /* Codes to determine basic types.
 
@@ -280,7 +281,8 @@ enum
  * Return value:
  * Always 1.
  */
-extern int printf_generic_info PARAMS((struct printf_info *const pinfo, size_t n, int *argtypes));
+extern int printf_generic_info (struct printf_info *const pinfo, size_t n, int *argtypes);
+
 
 /**
  * printf_generic:   
@@ -295,8 +297,10 @@ extern int printf_generic_info PARAMS((struct printf_info *const pinfo, size_t n
  * Return value:
  * The number of characters output.
  **/
-extern int printf_generic PARAMS((STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args));
+extern int printf_generic (STREAM *stream, struct printf_info *const pinfo, union printf_arg const *args);
 
+
+#line 268 "./printf.in"
 /**
  * register_printf_function:  
  * @spec: the character which will trigger @func, cast to an unsigned int.
@@ -310,7 +314,8 @@ extern int printf_generic PARAMS((STREAM *stream, struct printf_info *const pinf
  * Returns %NULL if @func was not successfully registered, a
  * %spec_entry with the information on the function if it was.
  **/
-extern spec_entry * register_printf_function PARAMS((unsigned spec, printf_function *fmt, printf_arginfo_function *arg));
+extern spec_entry * register_printf_function (unsigned spec, printf_function *fmt, printf_arginfo_function *arg);
+
 
 /**
  * printf_strerror:  
@@ -323,7 +328,8 @@ extern spec_entry * register_printf_function PARAMS((unsigned spec, printf_funct
  * parsing of a printf format string.  It is the responsibility
  * of the caller to free the string.
  */
-extern char * printf_strerror PARAMS(());
+extern char * printf_strerror (void);
+
 
 /**
  * printf_error:  
@@ -343,7 +349,8 @@ extern char * printf_strerror PARAMS(());
  * The address of the full accumulated error message in @pinfo is
  * returned.
  **/
-extern char * printf_error PARAMS((struct printf_info *pinfo, const char *file, int line, const char *func1, const char *func2, const char *func3, const char *error_message));
+extern char * printf_error (struct printf_info *pinfo, const char *file, int line, const char *func1, const char *func2, const char *func3, const char *error_message);
+
 
 /**
  * parse_printf_format:  
@@ -365,7 +372,8 @@ extern char * printf_error PARAMS((struct printf_info *pinfo, const char *file,
  * this function again. If there is an error, then %SNV_ERROR
  * is returned instead.
  **/
-extern size_t parse_printf_format PARAMS((const char *format, int n, int *argtypes));
+extern size_t parse_printf_format (const char *format, int n, int *argtypes);
+
 
 /**
  * stream_printfv:  
@@ -381,7 +389,8 @@ extern size_t parse_printf_format PARAMS((const char *format, int n, int *argtyp
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int stream_printfv PARAMS((STREAM *stream, const char *format, snv_constpointer const *ap));
+extern int stream_printfv (STREAM *stream, const char *format, snv_constpointer const *ap);
+
 
 /**
  * stream_vprintf:  
@@ -397,7 +406,8 @@ extern int stream_printfv PARAMS((STREAM *stream, const char *format, snv_constp
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int stream_vprintf PARAMS((STREAM *stream, const char *format, va_list ap));
+extern int stream_vprintf (STREAM *stream, const char *format, va_list ap);
+
 
 /**
  * stream_printf:  
@@ -417,7 +427,7 @@ extern int stream_printf SNV_GNUC_PRINTF((STREAM * stream, const char *format, .
 
 
 /**
- * fdputc:  
+ * snv_fdputc:  
  * @ch: A single character to be added to @stream.
  * @stream: The stream in which to write @ch.
  * 
@@ -428,7 +438,8 @@ extern int stream_printf SNV_GNUC_PRINTF((STREAM * stream, const char *format, .
  * The value of @ch that has been put in @stream, or -1 in case of
  * an error (errno will be set to indicate the type of error).
  **/
-extern int fdputc PARAMS((int ch, STREAM *stream));
+extern int snv_fdputc (int ch, STREAM *stream);
+
 
 /**
  * snv_dprintf:  
@@ -459,10 +470,11 @@ extern int snv_dprintf SNV_GNUC_PRINTF((int fd, const char *format, ...), 2, 3);
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int snv_vdprintf PARAMS((int fd, const char *format, va_list ap));
+extern int snv_vdprintf (int fd, const char *format, va_list ap);
+
 
 /**
- * dprintfv:  
+ * snv_dprintfv:  
  * @fd: an open file descriptor.
  * @format: a % delimited format string.
  * @args: a vector of argument addresses to match @format.
@@ -474,10 +486,11 @@ extern int snv_vdprintf PARAMS((int fd, const char *format, va_list ap));
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int dprintfv PARAMS((int fd, const char *format, snv_constpointer const args[]));
+extern int snv_dprintfv (int fd, const char *format, snv_constpointer const args[]);
+
 
 /**
- * fileputc:  
+ * snv_fileputc:  
  * @ch: A single character to be added to @stream.
  * @stream: The stream in which to write @ch.
  * 
@@ -487,7 +500,8 @@ extern int dprintfv PARAMS((int fd, const char *format, snv_constpointer const a
  * Return value: 
  * The value of @ch that has been put in @stream.
  **/
-extern int fileputc PARAMS((int ch, STREAM *stream));
+extern int snv_fileputc (int ch, STREAM *stream);
+
 
 /**
  * snv_printf:  
@@ -516,10 +530,11 @@ extern int snv_printf SNV_GNUC_PRINTF((const char *format, ...), 1, 2);
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int snv_vprintf PARAMS((const char *format, va_list ap));
+extern int snv_vprintf (const char *format, va_list ap);
+
 
 /**
- * printfv:  
+ * snv_printfv:  
  * @format: a % delimited format string.
  * @args: a vector of argument addresses to match @format.
  * 
@@ -530,7 +545,8 @@ extern int snv_vprintf PARAMS((const char *format, va_list ap));
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int printfv PARAMS((const char *format, snv_constpointer const args[]));
+extern int snv_printfv (const char *format, snv_constpointer const args[]);
+
 
 /**
  * snv_fprintf:  
@@ -561,10 +577,11 @@ extern int snv_fprintf SNV_GNUC_PRINTF((FILE * file, const char *format, ...), 2
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int snv_vfprintf PARAMS((FILE *file, const char *format, va_list ap));
+extern int snv_vfprintf (FILE *file, const char *format, va_list ap);
+
 
 /**
- * fprintfv:  
+ * snv_fprintfv:  
  * @file: a stdio.h FILE* stream.
  * @format: a % delimited format string.
  * @args: a vector of argument addresses to match @format.
@@ -576,10 +593,11 @@ extern int snv_vfprintf PARAMS((FILE *file, const char *format, va_list ap));
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int fprintfv PARAMS((FILE *file, const char *format, snv_constpointer const args[]));
+extern int snv_fprintfv (FILE *file, const char *format, snv_constpointer const args[]);
+
 
 /**
- * bufputc:  
+ * snv_bufputc:  
  * @ch: A single character to be added to @stream.
  * @stream: The stream in which to write @ch.
  * 
@@ -589,7 +607,7 @@ extern int fprintfv PARAMS((FILE *file, const char *format, snv_constpointer con
  * Return value:
  * The value of @ch that has been put in @stream.
  **/
-extern int bufputc PARAMS((int ch, STREAM *stream));
+extern int snv_bufputc (int ch, STREAM *stream);
 
 /**
  * snv_sprintf:  
@@ -620,10 +638,11 @@ extern int snv_sprintf SNV_GNUC_PRINTF((char buffer[], const char *format, ...),
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int snv_vsprintf PARAMS((char buffer[], const char *format, va_list ap));
+extern int snv_vsprintf (char buffer[], const char *format, va_list ap);
+
 
 /**
- * sprintfv:  
+ * snv_sprintfv:  
  * @buffer: a preallocated char* buffer.
  * @format: a % delimited format string.
  * @args: a vector of argument addresses to match @format.
@@ -635,7 +654,8 @@ extern int snv_vsprintf PARAMS((char buffer[], const char *format, va_list ap));
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int sprintfv PARAMS((char buffer[], const char *format, snv_constpointer const args[]));
+extern int snv_sprintfv (char buffer[], const char *format, snv_constpointer const args[]);
+
 
 /**
  * snv_snprintf:  
@@ -670,10 +690,11 @@ extern int snv_snprintf SNV_GNUC_PRINTF((char buffer[], unsigned long limit, con
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int snv_vsnprintf PARAMS((char buffer[], unsigned long limit, const char *format, va_list ap));
+extern int snv_vsnprintf (char buffer[], unsigned long limit, const char *format, va_list ap);
+
 
 /**
- * snprintfv:  
+ * snv_snprintfv:  
  * @buffer: a preallocated char* buffer.
  * @limit: the maximum number of characters to write into @buffer.
  * @format: a % delimited format string.
@@ -687,10 +708,11 @@ extern int snv_vsnprintf PARAMS((char buffer[], unsigned long limit, const char
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int snprintfv PARAMS((char buffer[], unsigned long limit, const char *format, snv_constpointer const args[]));
+extern int snv_snprintfv (char buffer[], unsigned long limit, const char *format, snv_constpointer const args[]);
+
 
 /**
- * filputc:  
+ * snv_filputc:  
  * @ch: A single character to be added to @stream.
  * @stream: The stream in which to write @ch.
  * 
@@ -700,7 +722,7 @@ extern int snprintfv PARAMS((char buffer[], unsigned long limit, const char *for
  * Return value: 
  * The value of @ch that has been put in @stream.
  **/
-extern int filputc PARAMS((int ch, STREAM *stream));
+extern int snv_filputc (int ch, STREAM *stream);
 
 /**
  * snv_asprintf:  
@@ -742,10 +764,11 @@ extern int snv_asprintf SNV_GNUC_PRINTF((char **result, const char *format, ...)
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int snv_vasprintf PARAMS((char **result, const char *format, va_list ap));
+extern int snv_vasprintf (char **result, const char *format, va_list ap);
+
 
 /**
- * asprintfv:  
+ * snv_asprintfv:  
  * @result: the address of a char * variable.
  * @format: a % delimited format string.
  * @args: a vector of argument addresses to match @format.
@@ -761,9 +784,11 @@ extern int snv_vasprintf PARAMS((char **result, const char *format, va_list ap))
  * The number of characters written is returned, unless there is
  * an error, when %SNV_ERROR is returned.
  **/
-extern int asprintfv PARAMS((char **result, const char *format, snv_constpointer const args[]));
+extern int snv_asprintfv (char **result, const char *format, snv_constpointer const args[]);
 
 
+#line 269 "./printf.in"
+
 /* If you don't want to use snprintfv functions for *all* of your string
    formatting API, then define COMPILING_SNPRINTFV_C and use the snv_
    prefix for the entry points below. */
@@ -780,6 +805,12 @@ extern int asprintfv PARAMS((char **result, const char *format, snv_constpointer
 #undef vsnprintf
 #undef asprintf
 #undef vasprintf
+#undef asprintfv
+#undef dprintfv
+#undef fprintfv
+#undef sprintfv
+#undef printfv
+#undef snprintfv
 #define printf          snv_printf
 #define vprintf         snv_vprintf
 #define dprintf         snv_dprintf
@@ -792,6 +823,12 @@ extern int asprintfv PARAMS((char **result, const char *format, snv_constpointer
 #define vsnprintf       snv_vsnprintf
 #define asprintf        snv_asprintf
 #define vasprintf       snv_vasprintf
+#define asprintfv	snv_asprintfv
+#define dprintfv	snv_dprintfv
+#define fprintfv	snv_fprintfv
+#define sprintfv	snv_sprintfv
+#define printfv		snv_printfv
+#define snprintfv	snv_snprintfv
 #endif				/* !COMPILING_SNPRINTFV_C */
 #ifdef __cplusplus
 }
